//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: AppOption.cc,v 1.21 2001/03/19 05:33:40 tujikawa Exp $

#include "AppOption.h"

extern void *Download_thread_main();
extern void *Set_clist_column__cursize(int rowindex, const string& size_string);
extern void *Set_clist_column__totsize(int rowindex, const string& size_string);
extern void Set_clist_column__icon(int rowindex, ItemCell::ItemStatusType status);
extern void Create_default_iconset(GdkPixmap **stat_icon, GdkBitmap **stat_iconm);

extern GtkWidget *Create_CheckCList(GtkWidget **clist_ptr, char **titles, int n_titles);
extern gboolean Autosave_list(gpointer data);
extern gboolean Timer_start(gpointer data);
extern gboolean Timer_stop(gpointer data);

extern ThreadManager *thread_manager;
extern GtkWidget *itemlistwidget;
extern pthread_mutex_t itemlistlock;
extern void Toolbar_set_sensitive__thread();
extern ServerTemplateList svt_list;
extern Dialog *cdialog;
extern ItemList *itemlist_global;
extern ItemManager *item_manager;
extern CommandList command_list;
extern AppOption *appoption;
extern unsigned int thread_limit;

static GdkPixmap *stat_icon_opt[ICON_TOTAL];
static GdkBitmap *stat_iconm_opt[ICON_TOTAL];

//extern GdkPixmap *stat_icon_def[ICON_TOTAL];
//extern GdkBitmap *stat_iconm_def[ICON_TOTAL];

extern GdkPixmap *stat_icon[ICON_TOTAL];
extern GdkBitmap *stat_iconm[ICON_TOTAL];

#define ICONSET_PATH "/iconset/"

static void Option_OK(GtkWidget *w, AppOption *appoption)
{
  gtk_widget_hide(appoption->ret_Option_Window());

  appoption->Process_changes();
}

static void Option_cancel(GtkWidget *w, GtkWidget *option_window)
{
  gtk_widget_hide(option_window);
}

static gboolean Option_quit(GtkWidget *w, GtkWidget *option_window)
{
  gtk_widget_hide(w);

  return TRUE;
}

void AppOption::set_svt_clist(const list<string>& names)
{
  gtk_clist_clear(GTK_CLIST(svt_clist));
  for(vector<ServerTemplate>::iterator svt_itr = svt_list.ret_server_template_list().begin(); svt_itr != svt_list.ret_server_template_list().end()-1; ++svt_itr) {
    char *clist_item[2];
    clist_item[0] = strdup(svt_itr->ret_template_name().c_str());
    clist_item[1] = strdup(svt_itr->ret_template_comment().c_str());

    int rowindex = gtk_clist_append(GTK_CLIST(svt_clist), clist_item);

    delete [] clist_item[0];
    for(list<string>::const_iterator name_itr = names.begin(); name_itr != names.end(); ++name_itr) {
      if(svt_itr->ret_template_name() == *name_itr) {
	gtk_clist_select_row(GTK_CLIST(svt_clist), rowindex, 0);
	svt_list.set_valid_safely(svt_itr, true);
      }
    }
  } 
}

void AppOption::set_com_clist(const list<string>& names)
{
  gtk_clist_clear(GTK_CLIST(com_clist));
  for(vector<Command>::iterator com_itr = command_list.ret_command_list().begin(); com_itr != command_list.ret_command_list().end()-1; ++com_itr) {
    char *clist_item[2];
    clist_item[0] = strdup(com_itr->ret_command_name().c_str());
    clist_item[1] = strdup(com_itr->ret_command_comment().c_str());

    int rowindex = gtk_clist_append(GTK_CLIST(com_clist), clist_item);

    delete [] clist_item[0];
    for(list<string>::const_iterator name_itr = names.begin(); name_itr != names.end(); ++name_itr) {
      if(com_itr->ret_command_name() == *name_itr) {
	gtk_clist_select_row(GTK_CLIST(com_clist), rowindex, 0);
	command_list.set_valid_safely(com_itr, true);
      }
    }
  } 
}

unsigned int get_interval(time_t timer_time)
{
  time_t curtime = time(NULL);
  unsigned int interval;
  
  int diff = timer_time-curtime;
  if(diff <= 60 && diff >= 0) {
    interval = 1000;
  } else {
    interval = 60000;
  }
  return interval;
}

AppOption::AppOption(GtkWidget *app_window)
{
  pthread_mutex_init(&option_lock, NULL);
  option_window = gtk_dialog_new();
  gtk_widget_set_usize(GTK_WIDGET(option_window), 600, 400);
  gtk_window_set_title(GTK_WINDOW(option_window), _("Application Option"));
  gtk_window_set_modal(GTK_WINDOW(option_window), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(option_window), GTK_WINDOW(app_window));

  gtk_signal_connect(GTK_OBJECT(option_window),
		     "delete_event",
		     GTK_SIGNAL_FUNC(Option_quit),
		     NULL);
  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(option_window)->vbox),
		     hbox, TRUE, TRUE, 10);
  {
    notebook = gtk_notebook_new();
    gtk_notebook_set_scrollable(GTK_NOTEBOOK(notebook), TRUE);//added 2001/3/18
    gtk_notebook_popup_enable(GTK_NOTEBOOK(notebook)); //added 2001/3/18

    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
    gtk_box_pack_start(GTK_BOX(hbox),
		       notebook, TRUE, TRUE, 10);
    gtk_widget_show(notebook);
    gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), TRUE);
    GtkWidget *tab1_label = gtk_label_new(_("DOWNLOAD"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_DOWNLOAD_page(),
			     tab1_label);
    GtkWidget *tab2_label = gtk_label_new(_("CONFORMATION"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_CONFORMATION_page(),
			     tab2_label);
    GtkWidget *tab3_label = gtk_label_new(_("DISPLAY"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_DISPLAY_page(),
			     tab3_label);
    GtkWidget *tab4_label = gtk_label_new(_("TIMER"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_TIMER_page(),
			     tab4_label);

    GtkWidget *tab5_label = gtk_label_new(_("COMMAND"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_COMMANDLIST_page(),
			     tab5_label);

    GtkWidget *tab6_label = gtk_label_new(_("SERVER"));
    gtk_notebook_append_page(GTK_NOTEBOOK(notebook),
			     Create_SERVERTEMPLATE_page(),
			     tab6_label);
   
    //// action area
    GtkWidget *bbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
    gtk_box_pack_start(GTK_BOX(GTK_DIALOG(option_window)->action_area),
		       bbox, FALSE, FALSE, 0);
    gtk_widget_show(bbox);
    // OK button
    GtkWidget *OK_button = gtk_button_new_with_label(_("OK"));
    GTK_WIDGET_SET_FLAGS(OK_button, GTK_CAN_DEFAULT);
    gtk_window_set_default(GTK_WINDOW(option_window), OK_button);
    gtk_signal_connect(GTK_OBJECT(OK_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(Option_OK),
		       (void *)this);
    gtk_box_pack_start(GTK_BOX(bbox),
		       OK_button, TRUE, TRUE, 0);
    gtk_widget_show(OK_button);
    // Cancel
    GtkWidget *Cancel_button = gtk_button_new_with_label(_("CANCEL"));
    //GTK_WIDGET_SET_FLAGS(Cancel_button, GTK_CAN_DEFAULT);
    gtk_signal_connect(GTK_OBJECT(Cancel_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(Option_cancel),
		       GTK_OBJECT(option_window));
    gtk_box_pack_start(GTK_BOX(bbox),
		       Cancel_button, TRUE, TRUE, 0);
    gtk_widget_show(Cancel_button);
  }
  // initialize input tag
  autosave_tag = -1;
  timer_start_tag = -1;
  timer_stop_tag = -1;
}

static gboolean Update_commandlist(GtkWidget *button, GtkWidget *option_window)
{
  if(command_list.Read_from_file(itemlist_global->ret_file_command_list())) {
    cdialog->setup(_("Update Command list"),
		   _("Update successful"));
  } else {
    cdialog->setup(_("Update Command list"),
		   _("Update failed"));
  }
  cdialog->set_no_button_visible(false);
  cdialog->set_cancel_button_visible(false);
  cdialog->set_transient(GTK_WINDOW(option_window));
  cdialog->show();

  list<string> names = appoption->ret_com_name_list();
  appoption->set_com_clist(names);

  return TRUE;
}

GtkWidget *AppOption::Create_COMMANDLIST_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);

  
  // toggle command list
  use_commandlist_toggle = gtk_check_button_new_with_label(_("Use Command list"));
  gtk_widget_set_name(use_commandlist_toggle, "check_button");
  gtk_box_pack_start(GTK_BOX(vbox), use_commandlist_toggle, FALSE, FALSE, 0);
  gtk_widget_show(use_commandlist_toggle);

  // ޥɥꥹȹܥ
  GtkWidget *update_button = gtk_button_new_with_label(_("Update Command list"));
  gtk_widget_set_name(update_button, "button");
  gtk_widget_show(update_button);
  gtk_box_pack_start(GTK_BOX(vbox), update_button, FALSE, FALSE, 10);
  gtk_signal_connect(GTK_OBJECT(update_button), "clicked",
		     GTK_SIGNAL_FUNC(Update_commandlist),
		     GTK_OBJECT(option_window));

  // ޥɥꥹȤCList
  GtkWidget *clist_label = gtk_label_new(_("choose commands you use"));
  gtk_widget_set_name(clist_label, "label");
  gtk_box_pack_start(GTK_BOX(vbox), clist_label, FALSE, FALSE, 10);
  gtk_widget_show(clist_label);
  int n_titles = 2;
  char *titles[n_titles];
  titles[0] = _("Command");
  titles[1] = _("Comment");

  // 륦ɥCListХå
  GtkWidget *scrolled_window = Create_CheckCList(&com_clist, titles, n_titles);
  gtk_clist_set_column_auto_resize(GTK_CLIST(com_clist), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(com_clist), 1, TRUE);

  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, FALSE, FALSE, 0);

  gtk_widget_show(scrolled_window);
  return hbox;
}

static gboolean Update_servertemplate(GtkWidget *button, GtkWidget *option_window)
{
  if(svt_list.Read_config_file(itemlist_global->ret_file_server_settings())) {
    cdialog->setup(_("Update Server Template"),
		   _("Update successful"));
  } else {
    cdialog->setup(_("Update Server Template"),
		   _("Update failed"));
  }
  cdialog->set_no_button_visible(false);
  cdialog->set_cancel_button_visible(false);
  cdialog->set_transient(GTK_WINDOW(option_window));
  cdialog->show();

  list<string> names = appoption->ret_svt_name_list();
  appoption->set_svt_clist(names);
  return TRUE;
}

static void Show_icon_error(GtkWidget *w)
{
  cdialog->setup(_("error"),
		 _("cannot read icon file"));
  cdialog->set_no_button_visible(false);
  cdialog->set_cancel_button_visible(false);
  cdialog->set_transient(GTK_WINDOW(gtk_widget_get_toplevel(w)));
  cdialog->show();
}


GtkWidget *AppOption::Create_SERVERTEMPLATE_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);

  // toggle enable/disable Server Template feature
  use_servertemplate_toggle = gtk_check_button_new_with_label(_("Use Server Template"));
  gtk_widget_set_name(use_servertemplate_toggle, "check_button");
  gtk_box_pack_start(GTK_BOX(vbox), use_servertemplate_toggle, FALSE, FALSE, 0);
  gtk_widget_show(use_servertemplate_toggle);

  // server template update button 
  GtkWidget *update_button = gtk_button_new_with_label(_("Update Server Template"));
  gtk_widget_set_name(update_button, "button");
  gtk_widget_set_name(update_button, "option_button");
  gtk_widget_show(update_button);
  gtk_box_pack_start(GTK_BOX(vbox), update_button, FALSE, FALSE, 10);
  gtk_signal_connect(GTK_OBJECT(update_button), "clicked",
		     GTK_SIGNAL_FUNC(Update_servertemplate),
		     GTK_OBJECT(option_window));

  // CList for Server Template
  GtkWidget *clist_label = gtk_label_new(_("choose templates you use"));
  gtk_box_pack_start(GTK_BOX(vbox), clist_label, FALSE, FALSE, 10);
  gtk_widget_show(clist_label);
  int n_titles = 2;
  char *titles[n_titles];
  titles[0] = _("Template");
  titles[1] = _("Comment");
  GtkWidget *scrolled_window = Create_CheckCList(&svt_clist, titles, n_titles);
  gtk_clist_set_column_auto_resize(GTK_CLIST(svt_clist), 0, TRUE);
  gtk_clist_set_column_auto_resize(GTK_CLIST(svt_clist), 1, TRUE);

  gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, FALSE, FALSE, 0);

  gtk_widget_show(scrolled_window);

  return hbox;
}

static gboolean Create_pixmap_from_file(GdkPixmap **pixmap_ptr,
					GdkBitmap **bitmap_ptr,
					const string& dir,
					const string& file,
					GtkWidget *clist)
{

  char *home_dir = g_get_home_dir();
  string filename = home_dir;
  filename += "/.aria"ICONSET_PATH+dir+'/'+file;
  //cerr << filename << endl;
  if(*pixmap_ptr != NULL) {
    gdk_pixmap_unref(*pixmap_ptr);
    *pixmap_ptr = NULL;
  }

  if(*bitmap_ptr != NULL) {
    gdk_bitmap_unref(*bitmap_ptr);
    *bitmap_ptr = NULL;
  }
  
  GtkStyle *style = gtk_widget_get_style(clist);

  *pixmap_ptr = gdk_pixmap_create_from_xpm(clist->window, bitmap_ptr, &style->bg[GTK_STATE_NORMAL], filename.c_str());
  if(*pixmap_ptr) {
    return true;
  } else {
    return false;
  }
}

static void CLIST_SET_PIXMAP(GtkWidget *clist, int rowindex, int column, int pixmap_n) {
  gtk_clist_set_pixmap(GTK_CLIST(clist), rowindex, column, stat_icon_opt[pixmap_n], stat_iconm_opt[pixmap_n]);
}

static gboolean Create_new_iconset(GdkPixmap **stat_icon_opt,
				   GdkBitmap **stat_iconm_opt,
				   const string& dirname)
{
  gboolean retval = FALSE;
  if(dirname != DEFAULT_ICONSET) {
    if(Create_pixmap_from_file(&stat_icon_opt[ICON_COMPLETE], &stat_iconm_opt[ICON_COMPLETE], dirname, "complete.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_DOWNLOAD], &stat_iconm_opt[ICON_DOWNLOAD], dirname, "download.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_LOCK], &stat_iconm_opt[ICON_LOCK], dirname, "lock.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_STOP], &stat_iconm_opt[ICON_STOP], dirname, "stop.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_DIVIDE], &stat_iconm_opt[ICON_DIVIDE], dirname, "divide.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_ERROR], &stat_iconm_opt[ICON_ERROR], dirname, "error.xpm", itemlistwidget) &&
       Create_pixmap_from_file(&stat_icon_opt[ICON_READY], &stat_iconm_opt[ICON_READY], dirname, "ready.xpm", itemlistwidget)) {
      retval = TRUE;
    }
      
  } else {
    Create_default_iconset(stat_icon_opt, stat_iconm_opt);
    retval = TRUE;
  }

  return retval;
}

static gboolean Change_iconset(GtkWidget *apply_button, GtkWidget *clist)
{
  if(GTK_CLIST(clist)->selection == NULL) return TRUE;
  char *dirname;
  gtk_clist_get_text(GTK_CLIST(clist), GPOINTER_TO_INT(GTK_CLIST(clist)->selection->data), 0, &dirname);

  GdkPixmap *stat_icon_opt[ICON_TOTAL];
  GdkBitmap *stat_iconm_opt[ICON_TOTAL];
  for(int i = 0; i < ICON_TOTAL; ++i) {
    stat_icon_opt[i] = NULL;
    stat_iconm_opt[i] = NULL;
  }
  if(!Create_new_iconset(stat_icon_opt, stat_iconm_opt, dirname)) {
    Show_icon_error(clist);
    for(int i = 0; i < ICON_TOTAL; ++i) {
      if(stat_icon_opt[i] != NULL) gdk_pixmap_unref(stat_icon_opt[i]);
      if(stat_iconm_opt[i] != NULL) gdk_bitmap_unref(stat_iconm_opt[i]);
    }
    return TRUE;
  }

  pthread_mutex_lock(&itemlistlock);
  for(int i = 0; i < ICON_TOTAL; ++i) {
    gdk_pixmap_unref(stat_icon[i]);
    gdk_bitmap_unref(stat_iconm[i]);
    stat_icon[i] = stat_icon_opt[i];
    stat_iconm[i] = stat_iconm_opt[i];
  }
  for(int rowindex = 0; rowindex < GTK_CLIST(itemlistwidget)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    Set_clist_column__icon(rowindex, itemcell->ret_Status());
  }
  pthread_mutex_unlock(&itemlistlock);
  appoption->set_icon_dir(dirname);// not smart..

  return TRUE;
}

static gboolean Icon_dir_selection_made(GtkWidget *clist,
					unsigned int row,
					unsigned int column,
					GdkEventButton *event,
					GtkWidget *preview_clist)
{
   char *dirname;
   gtk_clist_get_text(GTK_CLIST(clist), row, 0, &dirname);

  switch(event->button) {
  case 1:
  case 3: {
    /*
    bool retval = false;
    if(strcmp(dirname, DEFAULT_ICONSET)) {
      if(Create_pixmap_from_file(&stat_icon_opt[ICON_COMPLETE], &stat_iconm_opt[ICON_COMPLETE], dirname, "complete.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_DOWNLOAD], &stat_iconm_opt[ICON_DOWNLOAD], dirname, "download.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_LOCK], &stat_iconm_opt[ICON_LOCK], dirname, "lock_item.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_STOP], &stat_iconm_opt[ICON_STOP], dirname, "stop.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_DIVIDE], &stat_iconm_opt[ICON_DIVIDE], dirname, "divide.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_ERROR], &stat_iconm_opt[ICON_ERROR], dirname, "error.xpm", preview_clist) &&
	 Create_pixmap_from_file(&stat_icon_opt[ICON_READY], &stat_iconm_opt[ICON_READY], dirname, "ready.xpm",preview_clist)) {
	retval = true;
      }
    } else {
      Create_default_iconset(stat_icon_opt, stat_iconm_opt);
      retval = true;
    }
    */
    if(!Create_new_iconset(stat_icon_opt, stat_iconm_opt, dirname)) {
      Show_icon_error(clist);
      return TRUE;
    }

    CLIST_SET_PIXMAP(preview_clist, 0, 0, ICON_READY);
    CLIST_SET_PIXMAP(preview_clist, 0, 1, ICON_DOWNLOAD);
    CLIST_SET_PIXMAP(preview_clist, 0, 2, ICON_STOP);
    CLIST_SET_PIXMAP(preview_clist, 0, 3, ICON_COMPLETE);
    CLIST_SET_PIXMAP(preview_clist, 0, 4, ICON_ERROR);
    CLIST_SET_PIXMAP(preview_clist, 0, 5, ICON_DIVIDE);
    CLIST_SET_PIXMAP(preview_clist, 0, 6, ICON_LOCK);
 
    break;
  }
  }
  return TRUE;
}

GtkWidget *AppOption::Create_DISPLAY_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);

  // enable/disable human readable
  {
    use_size_human_readable_toggle = gtk_check_button_new_with_label(_("Display sizes in human readable format (e.g., 1.2K 234.1M 2.2G)"));
    gtk_widget_set_name(use_size_human_readable_toggle, "check_button");
    gtk_box_pack_start(GTK_BOX(vbox), use_size_human_readable_toggle, FALSE, FALSE, 0);
    gtk_widget_show(use_size_human_readable_toggle);
  }

  // icon set
  {
    GtkWidget *frame = gtk_frame_new(_("Icons"));
    gtk_widget_show(frame);
    gtk_box_pack_start(GTK_BOX(vbox), frame, FALSE, FALSE, 10);

    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
    gtk_container_add(GTK_CONTAINER(frame), hbox);
    gtk_widget_show(hbox);

    GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
    gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 10);
    gtk_widget_show(vbox);

    // clist for preview
    {
      GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
      gtk_widget_show(scrolled_window);
      gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, FALSE, FALSE, 0);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
      // set minimum size of scrolled window
      gtk_widget_set_usize(scrolled_window, 210, 35);
      // CList 롣Ǥ n_titles Ȥ
      icon_preview_clist = gtk_clist_new_with_titles(7, NULL);
      gtk_clist_set_row_height(GTK_CLIST(icon_preview_clist), 25);
      gtk_clist_column_titles_passive(GTK_CLIST(icon_preview_clist));

      for(int i = 0; i < ICON_TOTAL; ++i) {
	gtk_clist_set_column_auto_resize(GTK_CLIST(icon_preview_clist), i, true);
      }
      // set selection mode SINGLE
      //gtk_clist_set_selection_mode(GTK_CLIST(icon_preview_clist), GTK_SELECTION_SINGLE);
      // no callbacks needed
      
      // ˱Ƥդɬפʤɤʤиɤɤʤ :)
      gtk_clist_set_shadow_type(GTK_CLIST(icon_preview_clist), GTK_SHADOW_ETCHED_OUT);
      
      // CList åȤľܥå˲äɽ
      gtk_container_add(GTK_CONTAINER(scrolled_window), icon_preview_clist);
      gtk_widget_show(icon_preview_clist);
      // add 1 row
      char *titles[ICON_TOTAL];
      for(int i = 0; i < ICON_TOTAL; ++i) {
	titles[i] = NULL;
      }
      gtk_clist_append(GTK_CLIST(icon_preview_clist), titles);
      gtk_clist_set_selectable(GTK_CLIST(icon_preview_clist), 0, false);

      // init pixmap and bitmap array
      for(int i = 0; i < ICON_TOTAL; ++i) {
	stat_icon_opt[i] = NULL;
	stat_iconm_opt[i] = NULL;
      }
    }
    // clist for browse
    {
      GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
      gtk_widget_show(scrolled_window);
      gtk_box_pack_start(GTK_BOX(vbox), scrolled_window, FALSE, FALSE, 0);
      gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				     GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
      // set minimum size of scrolled window
      gtk_widget_set_usize(scrolled_window, 300, 140);

      // CList 롣Ǥ n_titles Ȥ
      char *titles[1];
      titles[0] = _("icon set");
      icon_dir_clist = gtk_clist_new_with_titles(1, titles);
      gtk_clist_column_titles_passive(GTK_CLIST(icon_dir_clist));
      // set selection mode SINGLE
      gtk_clist_set_selection_mode(GTK_CLIST(icon_dir_clist), GTK_SELECTION_SINGLE);
      
      // selection callbacks
      gtk_signal_connect(GTK_OBJECT(icon_dir_clist), "select_row",
      			 GTK_SIGNAL_FUNC(Icon_dir_selection_made),
			 GTK_OBJECT(icon_preview_clist));

      // ˱Ƥդɬפʤɤʤиɤɤʤ :)
      gtk_clist_set_shadow_type(GTK_CLIST(icon_dir_clist), GTK_SHADOW_ETCHED_OUT);
      
      // CList åȤľܥå˲äɽ
      gtk_container_add(GTK_CONTAINER(scrolled_window), icon_dir_clist);
      gtk_widget_show(icon_dir_clist);
    }
    {
      GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
      gtk_widget_show(hbox);
      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
      GtkWidget *apply_button = gtk_button_new_with_label(_("Apply"));
      gtk_widget_show(apply_button);
      gtk_box_pack_start(GTK_BOX(hbox), apply_button, FALSE, FALSE, 10);

      gtk_signal_connect(GTK_OBJECT(apply_button), "clicked",
			 GTK_SIGNAL_FUNC(Change_iconset),
			 GTK_OBJECT(icon_dir_clist));
    }
  }
  return hbox;
}

GtkWidget *AppOption::Create_CONFORMATION_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_widget_show(hbox);
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 5);
  gtk_widget_show(vbox);

  // ƥȤ˳ǧ뤫ɤ
  conform_delete_toggle = gtk_check_button_new_with_label(_("Conform delete items"));
  gtk_widget_set_name(conform_delete_toggle, "check_button");
  gtk_box_pack_start(GTK_BOX(vbox), conform_delete_toggle, FALSE, FALSE, 0);
  gtk_widget_show(conform_delete_toggle);

  // ץཪλ˳ǧ뤫ɤ
  conform_exit_toggle = gtk_check_button_new_with_label(_("Conform quit program"));
  gtk_widget_set_name(conform_exit_toggle, "check_button");
  gtk_box_pack_start(GTK_BOX(vbox), conform_exit_toggle, FALSE, FALSE, 0);
  gtk_widget_show(conform_exit_toggle);

  return hbox;
}

GtkWidget *AppOption::Create_TIMER_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_widget_show(hbox);

  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  {
    use_timer_toggle = gtk_check_button_new_with_label(_("Use timer"));
    gtk_widget_show(use_timer_toggle);
    gtk_box_pack_start(GTK_BOX(vbox), use_timer_toggle, FALSE, FALSE, 0);
    {
      GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
      gtk_widget_show(hbox);
      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
      {
	GtkWidget *label = gtk_label_new(_("Start(hour:min) "));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	GtkObject *adjustment = gtk_adjustment_new(0,// initial value
						   0,// minimal value
						   23,// maximal value
						   1,
						   23,
						   0);
	timer_hour_start_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment),
						  1.0,
						    0);
	gtk_widget_show(timer_hour_start_spin);
	gtk_box_pack_start(GTK_BOX(hbox), timer_hour_start_spin, FALSE, FALSE, 0);
      }
      GtkWidget *label = gtk_label_new(":");
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
      {
	GtkObject *adjustment = gtk_adjustment_new(0,// initial value
						   0,// minimal value
						   59,// maximal value
						   1,
						   10,
						   0);
	timer_min_start_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment),
						   1.0,
						   0);
	gtk_widget_show(timer_min_start_spin);
	gtk_box_pack_start(GTK_BOX(hbox), timer_min_start_spin, FALSE, FALSE, 0);
      }
    }
    {
      GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
      gtk_widget_show(hbox);
      gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
      {
	GtkWidget *label = gtk_label_new(_("Stop(hour:min) "));
	gtk_widget_show(label);
	gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

	GtkObject *adjustment = gtk_adjustment_new(0,// initial value
						   0,// minimal value
						   23,// maximal value
						   1,
						   23,
						   0);
	timer_hour_stop_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment),
						  1.0,
						    0);
	gtk_widget_show(timer_hour_stop_spin);
	gtk_box_pack_start(GTK_BOX(hbox), timer_hour_stop_spin, FALSE, FALSE, 0);
      }
      GtkWidget *label = gtk_label_new(":");
      gtk_widget_show(label);
      gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
      {
	GtkObject *adjustment = gtk_adjustment_new(0,// initial value
						   0,// minimal value
						   59,// maximal value
						   1,
						   10,
						   0);
	timer_min_stop_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment),
						  1.0,
						  0);
	gtk_widget_show(timer_min_stop_spin);
	gtk_box_pack_start(GTK_BOX(hbox), timer_min_stop_spin, FALSE, FALSE, 0);
      }
    }
  }

  return hbox;
}

GtkWidget *AppOption::Create_DOWNLOAD_page()
{
  GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(hbox), 5);
  gtk_widget_show(hbox);

  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);
  // timed out
  {
    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    
    GtkWidget *label_thread = gtk_label_new(_("The number of simultaneous downloads "));
    gtk_widget_set_name(label_thread, "label");
    gtk_widget_show(label_thread);
    gtk_box_pack_start(GTK_BOX(hbox), label_thread, FALSE, FALSE, 0);
    
    GtkObject *adjustment_thread = gtk_adjustment_new(1,// value
						      1,// min value
						      thread_limit,// max value
						      1,// inc/dec with 1st btn
						      1,// inc/dec with 2nd btn
						      0// don't use
						      );
    spin_thread = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment_thread),
				      1.0,
				      0);
    gtk_widget_show(spin_thread);
    gtk_widget_set_name(spin_thread, "spin");
    gtk_box_pack_start(GTK_BOX(hbox), spin_thread, FALSE, FALSE, 0);
  }
  {
    // toggle automatic start
    toggle_automatic_start = gtk_check_button_new_with_label(_("Start downloads automatically"));
    gtk_widget_set_name(toggle_automatic_start, "check_button");
    gtk_widget_show(toggle_automatic_start);
    gtk_box_pack_start(GTK_BOX(vbox), toggle_automatic_start, FALSE, FALSE, 0);
  }
  {
    // toggle ignore error item in "Start All" menu
    ignore_error_item_toggle = gtk_check_button_new_with_label(_("Ignore error item in \"Start All\" menu"));
    gtk_widget_show(ignore_error_item_toggle);
    gtk_box_pack_start(GTK_BOX(vbox), ignore_error_item_toggle, FALSE, FALSE, 0);
  }
  {
    // ignore list in find hyperlink
    ignore_extention_toggle = gtk_check_button_new_with_label(_("ignore these extentions in finding hyperlink"));
    gtk_widget_set_name(ignore_extention_toggle, "check_button");
    gtk_box_pack_start(GTK_BOX(vbox), ignore_extention_toggle, FALSE, FALSE, 0);
    gtk_widget_show(ignore_extention_toggle);
    // entry
    ignore_extention_entry = gtk_entry_new();
    gtk_widget_set_usize(ignore_extention_entry, 300, -1);
    gtk_widget_set_name(ignore_extention_entry, "entry");
    gtk_box_pack_start(GTK_BOX(vbox), ignore_extention_entry, FALSE, FALSE, 0);
    gtk_widget_show(ignore_extention_entry);
  }
  {
    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);

    // toggle autosave
    autosave_toggle = gtk_check_button_new_with_label(_("Autosave (list, history, options) every "));
    gtk_widget_show(autosave_toggle);
    gtk_box_pack_start(GTK_BOX(hbox), autosave_toggle, FALSE, FALSE, 0);

    GtkObject *adjustment = gtk_adjustment_new(10,
					       1,
					       MAXAUTOSAVEINTERVAL,
					       1,
					       30,
					       0);
    autosave_spin = gtk_spin_button_new(GTK_ADJUSTMENT(adjustment),
					1.0,
					0);
    gtk_widget_show(autosave_spin);
    gtk_box_pack_start(GTK_BOX(hbox), autosave_spin, FALSE, FALSE, 0);

    GtkWidget *label = gtk_label_new(_("minute(s)"));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
  }
  {
    // execute arbitrary command when all downloads are over(ignore stopped or error items)
    use_arb_command_toggle = gtk_check_button_new_with_label(_("Execute command when all downloads are over or stopped by timer"));
    gtk_widget_show(use_arb_command_toggle);
    gtk_box_pack_start(GTK_BOX(vbox), use_arb_command_toggle, FALSE, FALSE, 0);

    GtkWidget *hbox = gtk_hbox_new(FALSE, 5);
    gtk_widget_show(hbox);
    gtk_box_pack_start(GTK_BOX(vbox), hbox, FALSE, FALSE, 0);
    GtkWidget *label = gtk_label_new(_("Command:"));
    gtk_widget_show(label);
    gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);
    //// command entry
    arb_command_entry = gtk_entry_new();
    gtk_widget_set_usize(arb_command_entry, 300, -1);
    gtk_box_pack_start(GTK_BOX(hbox), arb_command_entry, FALSE, FALSE, 0);
    gtk_widget_show(arb_command_entry); 
  }
  return hbox;
}

bool AppOption::ret_use_ignore_extention_list()
{
  pthread_mutex_lock(&option_lock);
  pthread_mutex_unlock(&option_lock);
  return use_ignore_extention_list;

}

const list<string>& AppOption::ret_ignore_extention_list()
{
  pthread_mutex_lock(&option_lock);
  pthread_mutex_unlock(&option_lock);

  return ignore_extention_list;
}

void AppOption::Set_Option_Values(unsigned int maxthread_in,
				  bool use_automatic_start_in,
				  bool use_ignore_error_item_in,
				  bool use_ignore_extention_list_in,
				  const list<string>& ignore_extention_list_in,
				  bool use_autosave_in,
				  unsigned int autosave_interval_in,
				  bool use_arb_command_in,
				  const string& arb_command_in,
				  bool use_timer_in,
				  const TimerData& timerdata_in,
				  bool conform_delete_in,
				  bool conform_exit_in,
				  bool use_servertemplate_in,
				  bool use_commandlist_in,
				  bool use_size_human_readable_in,
				  const string& icon_dir_in,
				  const list<string>& svt_name_list,
				  const list<string>& com_name_list)
{
  if(maxthread_in > thread_limit) {
    maxthread = thread_limit;
  } else {
    maxthread = maxthread_in;
  }
  use_automatic_start = use_automatic_start_in;
  use_ignore_error_item = use_ignore_error_item_in;
  use_ignore_extention_list = use_ignore_extention_list_in;
  ignore_extention_list = ignore_extention_list_in;
  conform_delete = conform_delete_in;
  conform_exit = conform_exit_in;
  use_servertemplate = use_servertemplate_in;
  use_commandlist = use_commandlist_in;
  use_size_human_readable = use_size_human_readable_in;
  set_svt_clist(svt_name_list);
  set_com_clist(com_name_list);

  use_autosave = use_autosave_in;
  if(autosave_interval_in < 1) {
    autosave_interval = 10;
  } else {
    autosave_interval = autosave_interval_in;
  }

  if(autosave_tag == -1 && use_autosave) {
    //gtk_timeout_remove(autosave_tag);
   autosave_tag = gtk_timeout_add(autosave_interval*60000,
				   GtkFunction(Autosave_list),
				   NULL);
  }

  use_arb_command = use_arb_command_in;
  arb_command = Remove_white(arb_command_in);

  use_timer = use_timer_in;
  timerdata = timerdata_in;
  if(timer_start_tag == -1 && use_timer) {
    timer_start_tag = gtk_timeout_add(get_interval(timerdata.ret_start_time()),
				      GtkFunction(Timer_start),
				      NULL);
    timer_stop_tag = gtk_timeout_add(get_interval(timerdata.ret_stop_time()),
				     GtkFunction(Timer_stop),
				     NULL);
  }

  // set iconset
  GdkPixmap *stat_icon_opt[ICON_TOTAL];
  GdkBitmap *stat_iconm_opt[ICON_TOTAL];
  for(int i = 0; i < ICON_TOTAL; ++i) {
    stat_icon_opt[i] = NULL;
    stat_iconm_opt[i] = NULL;
  }
  if(icon_dir_in.empty()) icon_dir = DEFAULT_ICONSET;
  else icon_dir = icon_dir_in;

  if(!Create_new_iconset(stat_icon_opt, stat_iconm_opt, icon_dir)) {
    for(int i = 0; i < ICON_TOTAL; ++i) {
      if(stat_icon_opt[i] != NULL) gdk_pixmap_unref(stat_icon_opt[i]);
      if(stat_iconm_opt[i] != NULL) gdk_bitmap_unref(stat_iconm_opt[i]);
    }
    icon_dir = DEFAULT_ICONSET;
    Create_new_iconset(stat_icon_opt, stat_iconm_opt, DEFAULT_ICONSET);
  }
  for(int i = 0; i < ICON_TOTAL; ++i) {
    stat_icon[i] = stat_icon_opt[i];
    stat_iconm[i] = stat_iconm_opt[i];
  }
}

void AppOption::Set_icon_dir()
{
  string home_dir = g_get_home_dir();
  string icon_path = home_dir+"/.aria"ICONSET_PATH;

  DIR *dir = opendir(icon_path.c_str());
  gtk_clist_clear(GTK_CLIST(icon_dir_clist));
  char *titles[1];
  titles[0] = DEFAULT_ICONSET;
  gtk_clist_append(GTK_CLIST(icon_dir_clist), titles);
  if(dir) {
    struct dirent *entry;
    while((entry = readdir(dir)) != NULL) {
      if(strcmp(entry->d_name, ".") && strcmp(entry->d_name, "..")) {
	//char *titles[1];
	titles[0] = entry->d_name;
	gtk_clist_append(GTK_CLIST(icon_dir_clist), titles);
      }
    }
  }
}

list<string> AppOption::ret_name_list(GtkWidget *clist)
{
  list<string> names;
  GList *node = GTK_CLIST(clist)->selection;
  while(node) {
    int rowindex = GPOINTER_TO_UINT(node->data);
    char *name;
    gtk_clist_get_text(GTK_CLIST(clist), rowindex, 0, &name);
    names.push_back(name);
    node = g_list_next(node);
  }
  return names;
}

list<string> AppOption::ret_svt_name_list()
{
  return ret_name_list(svt_clist);
}

list<string> AppOption::ret_com_name_list()
{
  return ret_name_list(com_clist);
}
		    
bool AppOption::Is_in_ignore_extention_list(const string& filename)
{
  pthread_mutex_lock(&option_lock);
  bool retval = false;
  for(list<string>::const_iterator itr = ignore_extention_list.begin();
      itr != ignore_extention_list.end(); ++itr) {
    if(itr->size() < filename.size()) {
      if(filename.substr(filename.size()-itr->size()) == *itr) retval =  true;
      break;
    }
  }
  pthread_mutex_unlock(&option_lock);
  return retval;
}

void set_timer_spin(GtkWidget *spin, int value, int min, int max, int step_min, int step_max)
{
  GtkObject *adjustment = gtk_adjustment_new(value,
					     min,
					     max,
					     step_min,
					     step_max,
					     0);
  gtk_spin_button_configure(GTK_SPIN_BUTTON(spin),
			    GTK_ADJUSTMENT(adjustment),
			    1.0, 0);
}

void AppOption::Show_option_window()
{
  GtkObject *adjustment_thread = gtk_adjustment_new(maxthread,
						    1,
						    thread_limit,
						    1,
						    1,
						    0);
  gtk_spin_button_configure(GTK_SPIN_BUTTON(spin_thread),
			    GTK_ADJUSTMENT(adjustment_thread),
			    1.0, 0);


  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(toggle_automatic_start), use_automatic_start);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ignore_error_item_toggle), use_ignore_error_item);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(ignore_extention_toggle), use_ignore_extention_list);

  string ignore_extention_string;
  for(list<string>::const_iterator itr = ignore_extention_list.begin();
      itr != ignore_extention_list.end(); ++itr) {
    ignore_extention_string += *itr+' ';
  }
  gtk_entry_set_text(GTK_ENTRY(ignore_extention_entry), ignore_extention_string.c_str());

  // autosave
  GtkObject *adjustment_autosave = gtk_adjustment_new(autosave_interval,
						      1,
						      MAXAUTOSAVEINTERVAL,
						      1,
						      30,
						      0);
  gtk_spin_button_configure(GTK_SPIN_BUTTON(autosave_spin),
			    GTK_ADJUSTMENT(adjustment_autosave),
			    1.0, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(autosave_toggle), use_autosave);
  // execute command when ....
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_arb_command_toggle), use_arb_command);
  gtk_entry_set_text(GTK_ENTRY(arb_command_entry), arb_command.c_str());

  // timer
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_timer_toggle), use_timer);
  set_timer_spin(timer_hour_start_spin, timerdata.ret_hour_start(), 0, 23, 1, 23);
  set_timer_spin(timer_min_start_spin, timerdata.ret_min_start(), 0, 59, 1, 10);
  set_timer_spin(timer_hour_stop_spin, timerdata.ret_hour_stop(), 0, 23, 1, 23);
  set_timer_spin(timer_min_stop_spin, timerdata.ret_min_stop(), 0, 59, 1, 10);

  // conformation
  if(conform_delete) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(conform_delete_toggle), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(conform_delete_toggle), FALSE);
  }

  if(conform_exit) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(conform_exit_toggle), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(conform_exit_toggle), FALSE);
  }

  if(use_size_human_readable) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_size_human_readable_toggle), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_size_human_readable_toggle), FALSE);
  }

  if(use_servertemplate) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_servertemplate_toggle), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_servertemplate_toggle), FALSE);
  }

  if(use_commandlist) {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_commandlist_toggle), TRUE);
  } else {
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_commandlist_toggle), FALSE);
  }

  Set_icon_dir();

  gtk_notebook_set_page(GTK_NOTEBOOK(notebook), 0);
  gtk_widget_show(option_window);
}

GtkWidget *AppOption::ret_Option_Window() const
{
  return(option_window);
}

bool AppOption::Increase_thread()
{
  if(maxthread == thread_limit) return false;
  else {
    ++maxthread;
    Toolbar_set_sensitive__thread();
    pthread_mutex_lock(&itemlistlock);
    thread_manager->ManageThread(maxthread);
    pthread_mutex_unlock(&itemlistlock);
  }
  return true;
}

bool AppOption::Decrease_thread()
{
  if(maxthread == 1) return false;
  else {
    --maxthread;
    Toolbar_set_sensitive__thread();
    pthread_mutex_lock(&itemlistlock);
    thread_manager->ManageThread(maxthread);
    pthread_mutex_unlock(&itemlistlock);
  }
  return true;
}

unsigned int AppOption::ret_Maxthread() const
{
  return(maxthread);
}

void AppOption::set_Maxthread(unsigned int maxthread_in)
{
  maxthread = maxthread_in;
}

void AppOption::Start_start_timer(unsigned int check_interval)
{
  if(timer_start_tag > -1) {
    gtk_timeout_remove(timer_start_tag);
  }
  timer_start_tag = gtk_timeout_add(check_interval,
			GtkFunction(Timer_start),
			NULL);
}

void AppOption::Start_stop_timer(unsigned int check_interval)
{
  if(timer_stop_tag > -1) {
    gtk_timeout_remove(timer_stop_tag);
  }
  timer_stop_tag = gtk_timeout_add(check_interval,
			GtkFunction(Timer_stop),
			NULL);
}

void AppOption::Process_changes()
{
  pthread_mutex_lock(&option_lock);


  use_automatic_start = false;
  if(GTK_TOGGLE_BUTTON(toggle_automatic_start)->active) {
    use_automatic_start = true;
  }

  use_ignore_error_item = false;
  if(GTK_TOGGLE_BUTTON(ignore_error_item_toggle)->active) {
    use_ignore_error_item = true;
  }

  unsigned int autosave_interval_in = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(autosave_spin));
  if(GTK_TOGGLE_BUTTON(autosave_toggle)->active) {
    if(use_autosave && autosave_interval == autosave_interval_in) {
      autosave_interval = autosave_interval_in;
    } else {
      autosave_interval = autosave_interval_in;
      use_autosave = true;
      if(autosave_tag > -1) {
	gtk_timeout_remove(autosave_tag);
      }
      autosave_tag = gtk_timeout_add(autosave_interval*60000,
				     GtkFunction(Autosave_list),
				     NULL);
    }
  } else {
    if(autosave_tag > -1) {
      gtk_timeout_remove(autosave_tag);
      autosave_tag = -1;
    }
    use_autosave = false;
    autosave_interval = autosave_interval_in;
  }
  // timer
  int hour_start = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timer_hour_start_spin));
  int min_start = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timer_min_start_spin));
  int hour_stop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timer_hour_stop_spin));
  int min_stop = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(timer_min_stop_spin));
  timerdata.set_start_time(hour_start, min_start);
  timerdata.set_stop_time(hour_stop, min_stop);

  if(GTK_TOGGLE_BUTTON(use_timer_toggle)->active) {
    use_timer = true;
    Start_start_timer(get_interval(timerdata.ret_start_time()));
    Start_stop_timer(get_interval(timerdata.ret_stop_time()));
  } else {
    use_timer = false;
    if(timer_start_tag > -1) {
      gtk_timeout_remove(timer_start_tag);
      gtk_timeout_remove(timer_stop_tag);
      timer_start_tag = -1;
      timer_stop_tag = -1;
    }
  }
  // execute command when ...
  if(GTK_TOGGLE_BUTTON(use_arb_command_toggle)->active) {
    use_arb_command = true;
  } else {
    use_arb_command = false;
  }
  arb_command = Remove_white(gtk_entry_get_text(GTK_ENTRY(arb_command_entry)));

  // max thread
  unsigned int maxthread_in = gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spin_thread));

  if(maxthread_in > 0 && maxthread_in <= thread_limit) {
    maxthread = maxthread_in;
    Toolbar_set_sensitive__thread();
    pthread_mutex_lock(&itemlistlock);
    thread_manager->ManageThread(maxthread_in);
    pthread_mutex_unlock(&itemlistlock);
  }

  use_ignore_extention_list = false;
  if(GTK_TOGGLE_BUTTON(ignore_extention_toggle)->active) {
    use_ignore_extention_list = true;
  }

  string ignore_extention_string = Remove_white(gtk_entry_get_text(GTK_ENTRY(ignore_extention_entry)));
  list<string> ignore_extention_list_in;
  while(!ignore_extention_string.empty()) {
    ignore_extention_list_in.push_back(Token_splitter(ignore_extention_string, " ,"));
  }					       
  ignore_extention_list = ignore_extention_list_in;

  conform_delete = false;
  if(GTK_TOGGLE_BUTTON(conform_delete_toggle)->active) {
    conform_delete = true;
  }

  conform_exit = false;
  if(GTK_TOGGLE_BUTTON(conform_exit_toggle)->active) {
    conform_exit = true;
  }

  use_servertemplate = false;
  if(GTK_TOGGLE_BUTTON(use_servertemplate_toggle)->active) {
    use_servertemplate = true;
  }
  svt_list.update_validity(GTK_CLIST(svt_clist)->selection);

  use_commandlist = false;
  if(GTK_TOGGLE_BUTTON(use_commandlist_toggle)->active) {
    use_commandlist = true;
  }
  command_list.update_validity(GTK_CLIST(com_clist)->selection);

  bool use_size_human_readable_temp = use_size_human_readable;
  use_size_human_readable = false;
  if(GTK_TOGGLE_BUTTON(use_size_human_readable_toggle)->active) {
    use_size_human_readable = true;
  }

  pthread_mutex_unlock(&option_lock);
  // clear icon preview clist
  gtk_clist_clear(GTK_CLIST(icon_preview_clist));
  // add 1 row
  char *titles[ICON_TOTAL];
  for(int i = 0; i < ICON_TOTAL; ++i) {
    titles[i] = NULL;
  }
  gtk_clist_append(GTK_CLIST(icon_preview_clist), titles);

  if(use_size_human_readable && !use_size_human_readable_temp ||
     !use_size_human_readable && use_size_human_readable_temp) {
    gtk_clist_freeze(GTK_CLIST(itemlistwidget));
    for(list<int>::const_iterator id_ptr = item_manager->ret_id_list().begin();
	id_ptr != item_manager->ret_id_list().end(); ++id_ptr) {
      ItemCell *itemcell = item_manager->ret_itemaddr(*id_ptr);
      switch(itemcell->ret_Status()) {
      case ItemCell::ITEM_READY:
      case ItemCell::ITEM_READY_AGAIN:
      case ItemCell::ITEM_STOP:
      case ItemCell::ITEM_ERROR:
      case ItemCell::ITEM_COMPLETE:
      case ItemCell::ITEM_LOCK:
      default:////modified 2001/3/1
	{
	  /*
	  ItemStatusDynamic itemstatus(itemcell,
				       itemcell->ret_Status(),
				       itemcell->ret_Size_Current(),
				       itemcell->ret_Size_Total());
	  itemstatus.set_UpdateFlag(ItemStatusDynamic::progress_col);
	  itemstatus.Update();
	  */
	  int rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	  if(use_size_human_readable) {
	    Set_clist_column__cursize(rowindex, get_human_readable_size(itemcell->ret_Size_Current()));
	    Set_clist_column__totsize(rowindex, get_human_readable_size(itemcell->ret_Size_Total()));
	  } else {
	    Set_clist_column__cursize(rowindex, itos(itemcell->ret_Size_Current(), true));
	    Set_clist_column__totsize(rowindex, itos(itemcell->ret_Size_Total(), true));
	  }
	}
	break;
	//default:///modified 2001/3/1
	//break;
      }
    }
    gtk_clist_thaw(GTK_CLIST(itemlistwidget));
  }
}

bool AppOption::Whether_use_automatic_start()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_automatic_start;
  pthread_mutex_unlock(&option_lock);
  return retval;
}

bool AppOption::ret_use_ignore_error_item()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_ignore_error_item;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_conform_delete()
{
  pthread_mutex_lock(&option_lock);
  bool retval = conform_delete;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_conform_exit()
{
  pthread_mutex_lock(&option_lock);
  bool retval = conform_exit;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_use_servertemplate()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_servertemplate;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_use_commandlist()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_commandlist;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_use_size_human_readable()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_size_human_readable;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_use_autosave()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_autosave;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

unsigned int AppOption::ret_autosave_interval()
{
  pthread_mutex_lock(&option_lock);
  unsigned int retval = autosave_interval;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

int AppOption::ret_autosave_tag()
{
  pthread_mutex_lock(&option_lock);
  int retval = autosave_tag;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

bool AppOption::ret_use_timer()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_timer;
  pthread_mutex_unlock(&option_lock);

  return retval;
}
 
time_t AppOption::ret_timer_start_time()
{
  pthread_mutex_lock(&option_lock);
  time_t retval = timerdata.ret_start_time();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

time_t AppOption::ret_timer_stop_time()
{
  pthread_mutex_lock(&option_lock);
  time_t retval = timerdata.ret_stop_time();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

int AppOption::ret_timer_hour_start()
{
  pthread_mutex_lock(&option_lock);
  int retval = timerdata.ret_hour_start();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

int AppOption::ret_timer_min_start()
{
  pthread_mutex_lock(&option_lock);
  int retval = timerdata.ret_min_start();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

int AppOption::ret_timer_hour_stop()
{
  pthread_mutex_lock(&option_lock);
  int retval = timerdata.ret_hour_stop();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

int AppOption::ret_timer_min_stop()
{
  pthread_mutex_lock(&option_lock);
  int retval = timerdata.ret_min_stop();
  pthread_mutex_unlock(&option_lock);

  return retval;
}

void AppOption::Update_timer_start()
{
  pthread_mutex_lock(&option_lock);
  timerdata.Update_start_time();
  pthread_mutex_unlock(&option_lock);
}

void AppOption::Update_timer_stop()
{
  pthread_mutex_lock(&option_lock);
  timerdata.Update_stop_time();
  pthread_mutex_unlock(&option_lock);
}

bool AppOption::ret_use_arb_command()
{
  pthread_mutex_lock(&option_lock);
  bool retval = use_arb_command;
  pthread_mutex_unlock(&option_lock);

  return retval;
}
 

string AppOption::ret_arb_command()
{
  pthread_mutex_lock(&option_lock);
  string retval = arb_command;
  pthread_mutex_unlock(&option_lock);

  return retval;
}

void AppOption::set_icon_dir(const string& icon_dir_in)
{
  icon_dir = icon_dir_in;
}

const string& AppOption::ret_icon_dir()
{
  return icon_dir;
}
