//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: HTTPcontainer.cc,v 1.9 2001/03/09 06:49:09 tujikawa Exp $

// implementation of class HTTPcontainer
#include "HTTPcontainer.h"

// constructor
HTTPcontainer::HTTPcontainer()
{
  etag = "";
  contenttype = "";
  contentlength = 0;
}

// destructor
HTTPcontainer::~HTTPcontainer()
{
}

int HTTPcontainer::Parse_HTTP_header(const HTTPHeaderList& httpheaderlist)
{
  int retval = 1;//HTTPCON_NORMAL;

  HTTPHeaderList::const_iterator hl_itr;
  for(hl_itr = httpheaderlist.begin(); hl_itr != httpheaderlist.end(); ++hl_itr) {
    if(hl_itr->ret_Item() == "Content-Length" ||
       hl_itr->ret_Item() == "Content-length") {
      set_ContentLength(hl_itr->ret_Arg());
    } else if(hl_itr->ret_Item() == "ETag") {
      set_ETag(hl_itr->ret_Arg());
    } else if(hl_itr->ret_Item() == "Content-Type" ||
	      hl_itr->ret_Item() == "Content-type") {
      set_ContentType(hl_itr->ret_Arg());
    } else if(hl_itr->ret_Item() == "status") {
      set_HTTP_Status(hl_itr->ret_Arg());
    } else if(hl_itr->ret_Item() == "Location") {
      switch(ret_HTTP_Status()) {
      case MultipleChoices:
      case MovedPermanently:
      case Found:
      case SeeOther:
	set_Location(hl_itr->ret_Arg());
	retval = 1;
	break;
      case UseProxy:
	return -1;//throw HTTPCON_EUNSUPP;
      default:
	break;
      }
    } else if(hl_itr->ret_Item() == "Set-Cookie") {
      add_Cookie_string(hl_itr->ret_Arg());
    }
  }
  return retval;
}

int HTTPcontainer::ret_HTTP_Status() const
{
  return status;
}

unsigned int HTTPcontainer::ret_ContentLength() const
{
  return contentlength;
}

void HTTPcontainer::set_ContentLength(const string& clength_str)
{
  contentlength = strtol(clength_str.c_str(), (char**)NULL, 10);
}

void HTTPcontainer::set_ETag(const string& etag_in)
{
  etag = etag_in;
}

void HTTPcontainer::set_Location(const string& location_in)
{
  location = location_in;
}

const list<string>& HTTPcontainer::ret_Cookie_list() const
{
  return cookie_list;
}

void HTTPcontainer::add_Cookie_string(const string& cookie_string)
{
  cookie_list.push_back(cookie_string);
}

const string& HTTPcontainer::ret_Location() const
{
  return(location);
}

void HTTPcontainer::set_ContentType(const string& contenttype_in)
{
  contenttype = contenttype_in;
}

void HTTPcontainer::set_HTTP_Status(const string& status_str)
{
  status = strtol(status_str.c_str(), (char**)NULL, 10);
}
