//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemCell_HTTP.cc,v 1.37 2001/03/22 08:42:17 tujikawa Exp $

// class ItemListCell implementation
#include "ItemCell_HTTP.h"

extern ServerTemplateList svt_list;
extern UseragentList *useragent_list;
extern ItemList *itemlist_global;
extern AppOption *appoption;

ItemCell_HTTP::ItemCell_HTTP(const string& url_in,
			     const URLcontainer& urlcon_in,
			     const Options& options_in,
			     const string& initial_log)
  : ItemCell(url_in, urlcon_in, options_in, initial_log)
{
  session_counter = 1;
  retrieved_urlcon = urlcon_in;
}

ItemCell_HTTP::~ItemCell_HTTP()
{
}

//
// WWWС׵å
//
void ItemCell_HTTP::Send_Request(const Socket& sock, unsigned int startingbyte)
{
  string command, command_line;

  // GET command
  bool proxy_in_use = ret_Options().ret_use_http_proxy() && !ret_Options().ret_http_proxy().ret_Server().empty();
  if(proxy_in_use) {
    command_line = "GET "+
      ret_URL_Container().ret_Protocol()+
      "//"+
      URLcontainer::URL_Encode(ret_URL_Container().ret_Hostname()+
			       ret_URL_Container().ret_Dir()+
			       ret_URL_Container().ret_File())+
      ret_URL_Container().ret_Query()+
      " HTTP/"+ret_Options().ret_HTTP_version()+"\r\n"; // modified 2001/3/20
    //" HTTP/1.0\r\n";
  } else {
    command_line = "GET "+
      //URLcontainer::URL_Encode(ret_URL_Container().ret_Dir()+
      //		       ret_URL_Container().ret_File())+
      ret_URL_Container().ret_Dir()+
      ret_URL_Container().ret_File()+
      ret_URL_Container().ret_Query()+
      " HTTP/"+ret_Options().ret_HTTP_version()+"\r\n"; // modified 2001/3/20
    
    //" HTTP/1.0\r\n";
  }
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command_line;

  // Referer
  Options::RefererType referer_type;
  string referer_string;

  referer_type = ret_Options().ret_Referer_Type();
  referer_string = ret_Options().ret_Referer();

  switch(referer_type) {
  case Options::REFERER_NONE:
    command_line = "Referer:\r\n";
    break;
  case Options::REFERER_USER_DEFINED:
    command_line = "Referer: "+referer_string+"\r\n";
    break;
  case Options::REFERER_URL:
    command_line = "Referer: "+ret_URL()+"\r\n";
    break;
  case Options::REFERER_NOSEND:
    command_line = "";
  default:
  case Options::REFERER_INDEX:
    command_line = "Referer: "+
      ret_URL_Container().ret_Protocol()+
      "//"+
      URLcontainer::URL_Encode(ret_URL_Container().ret_Hostname()+
			       ret_URL_Container().ret_Dir())+
      "/index.html\r\n";
    break;
  }
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  // User-Agent
  if(ret_Options().ret_Random_useragent()) {
    unsigned int list_size = useragent_list->ret_vector().size();
    const vector<string>& useragent_vector = useragent_list->ret_vector();
    command_line = "User-Agent: "+useragent_vector[(int)((float)list_size*random()/(RAND_MAX+1.0))]+"\r\n";
  } else {
    command_line = "User-Agent: "+ret_Options().ret_Useragent()+"\r\n";
  }
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  // authentication
  if(proxy_in_use) {
    if(ret_Options().ret_use_http_proxy_authentication() &&
       !ret_Options().ret_http_proxy_User().empty()) {
      string user_pass = ret_Options().ret_http_proxy_User()+":"+ret_Options().ret_http_proxy_Password();
      string encoded_user_pass = Base64::encode(user_pass);
      command_line = "Proxy-Authorization: Basic "+encoded_user_pass+"\r\n";
      Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
      command = command+command_line;
    }
  } else if(ret_Options().Whether_use_authentication() &&
	    !ret_Options().ret_User().empty()) {
    string user_pass = ret_Options().ret_User()+":"+ret_Options().ret_Password();
    string encoded_user_pass = Base64::encode(user_pass);
    command_line = "Authorization: Basic "+encoded_user_pass+"\r\n";
    Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
    command = command+command_line;
  }

  // Cookie: ŬǤߤޤ i apologize for lazy implementation of cookie
  if(!svt.Is_valid(session_counter) && !ret_Options().ret_Cookie_nosend() ||
     svt.Is_valid(session_counter) && !svt.ret_session(session_counter).Is_nocookie()) {
    string key_value = cookie_list.ret_valid_cookie_string(ret_URL_Container().ret_Hostname(), ret_URL_Container().ret_Dir());
    if(key_value.size()) {
      command_line = "Cookie: "+key_value+"\r\n";
      Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
      command = command+command_line;
    }
  }
  if(svt.Is_valid(session_counter) && svt.ret_session(session_counter).Is_deletecookie()) {
    cookie_list.all_clear();
  }

  // Connection: close
  // In HTTP/1.0, connection is closed per request-response by default,
  // in HTTP/1.1, connection is keep alive by default.
  command_line =  "Connection: close\r\n";
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  // Accept
  command_line = "Accept: */*\r\n";
  /*
   */
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  command_line = "Accept-Languages: en, ja\r\n";
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  command_line = "Accept-Encoding: gzip, deflate\r\n";
  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command = command+command_line;

  // Host
  command_line = "Host: "+ret_URL_Container().ret_Hostname()+"\r\n";

  Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
  command += command_line;

  //Pragma (?)
  if(ret_Options().ret_use_http_proxy() && !ret_Options().ret_http_proxy().ret_Server().empty() && !ret_Options().ret_use_http_cache()) {
    command_line = "Pragma: no-cache\r\n";
    Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
    command += command_line;
  }    

  // Range
  if(startingbyte != 0) {
    command_line = "Range: bytes="+itos(startingbyte)+"-\r\n";
    Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
    command = command+command_line;

    // if-mod-since
    if(ret_Options().ret_downm_type() == Options::DOWNM_IFMODSINCE) {
      command_line = "If-Modified-Since: "+get_file_mod_date(ret_Options().ret_Store_Dir()+ret_Filename())+"\r\n";
      Send_message_to_gui(command_line, MSG_DOWNLOAD_SEND);
      command = command+command_line;
    }
  }
  command += "\r\n";
  SEND(command, sock);
}

//
// wwwСαåΥإå**httpheaer˳Ǽ
//
void ItemCell_HTTP::Get_HTTP_header(const Socket& socket, HTTPHeaderList& http_header_list)
{

  string report;
  unsigned int timedout = ret_Options().ret_Timed_Out();
  bool first_line = true;
  list<int> fd_list;

  while(1) {
    string header_string;
    char ch[1024];
    char *ch_tail = ch;
    while(1) {
      fd_list.clear();
      fd_list.push_back(ret_Desc_r());
      int retval = socket.is_readready(timedout, fd_list);
      if(retval && Socket::is_set(ret_Desc_r(), fd_list)) {
	ItemCommand itemcommand;
	read(ret_Desc_r(), &itemcommand, sizeof(ItemCommand));
	//set_Command(Process_command(itemcommand));
	set_Command(itemcommand);
	//thread_manager->release_token(pthread_self());
	if(ret_Dl_status() != DLCHANGE) {
	  throw ITEM_EINTER;
	}
      } else if(retval && socket.is_set(fd_list)) {
	int size;
	size = socket.Recv(ch_tail, sizeof(ch)-(ch_tail-ch), MSG_PEEK);
	//thread_manager->get_token(pthread_self());
	//thread_manager->release_token(pthread_self());
	if(size <= 0) throw ITEM_ERECV;
	int crlf_pos = 0;
	while(1) {
	  if(crlf_pos == size) break;
	  if(ch[crlf_pos] == '\r' || ch[crlf_pos] == '\n') {
	    break;
	  }
	  ++crlf_pos;
	}
	if(crlf_pos == 0) {
	  if(ch[crlf_pos] == '\r') {
	    size = socket.Recv(ch, 2, 0);
	  } else {
	    size = socket.Recv(ch, 1, 0);
	  }
	  header_string = "";
	  break;
	} else if(crlf_pos < size) {
	  if(ch[crlf_pos] == '\r') {
	    size = socket.Recv(ch, crlf_pos+2, 0);
	    ch[size-2] = '\0';
	  } else {
	    size = socket.Recv(ch, crlf_pos+1, 0);
	    ch[size-1] = '\0';
	  }
	  header_string = ch;
	  break;
	} else {
	  //ch[size] = '\0';// dangerous??
	  ch_tail = ch+size;
	}
      } else {
	throw ITEM_ETIMEDOUT;
      }
    }
    if(header_string.empty()) break;
    string header_string_temp = header_string;
    if(first_line) {
      first_line = false;
      //if(header_vector.size() < 2) throw ITEM_EPROT;
      string HTTP_version = Token_splitter(header_string_temp);
      int HTTP_status = stoi(Token_splitter(header_string_temp));
      string arg;
      //vector<string> arg_v;
      //copy(istream_iterator<string>(msgbuf), istream_iterator<string>(),
      //   back_inserter(arg_v));
      //cerr << header_string << endl;
      Send_message_to_gui(header_string, MSG_DOWNLOAD_RECV);
      switch(HTTP_status) {
      case OK:
      case MultipleChoices:
      case MovedPermanently:
      case Found:
      case NotModified:
      case SeeOther:
	break;
      case UseProxy:
	Send_message_to_gui(_("UseProxy is not supported yet"), MSG_DOWNLOAD_ERROR);
	set_Command(DLERRORSTOP);
	throw ITEM_EPROT;
      case PartialContent:
	break;
      case Forbidden:
	Send_message_to_gui(_("content forbidden"), MSG_DOWNLOAD_ERROR);
	set_Command(DLERROR);
	throw ITEM_EPROT;
      case NotFound:
	Send_message_to_gui(_("content not found"), MSG_DOWNLOAD_ERROR);
	if(ret_Options().ret_force_retry_404()) {
	  set_Command(DLERROR);
	} else {
	  // don't retry
	  set_Command(DLERRORSTOP);
	}
	throw ITEM_EINTER;
      case ServiceUnavailable:
	if(ret_Options().ret_force_retry_503()) {
	  set_Command(DLERROR);
	} else {
	  // don't retry
	  set_Command(DLERRORSTOP);
	}
      default:
	Send_message_to_gui(_("unexpected error occured"), MSG_DOWNLOAD_ERROR);
	set_Command(DLERRORSTOP);
	throw ITEM_EPROT;
      }
      HTTP_Header httpheader("status", itos(HTTP_status));
      http_header_list.push_back(httpheader);
    } else {
      string header = Token_splitter(header_string_temp, ":");
      string arg = Remove_white(header_string_temp);
      //cerr << "[" << header << "]" << endl;
      //cerr << "[" << arg << "]" << endl;
      Send_message_to_gui(header_string, MSG_DOWNLOAD_RECV);
      HTTP_Header httpheader(header, arg);
      http_header_list.push_back(httpheader);
    }
  }
}

void ItemCell_HTTP::Apply_new_options()
{
  ItemCell::Apply_new_options();
}

//
// ʬɤΤν
//
// divideĤʬƥ
// return:
//     false : 顼(ʬԲ)
//     true : ʬ
//
bool ItemCell_HTTP::Create_partial_item_entry(unsigned int divide, unsigned int total_size)
{
  unsigned int downloaded_size = 0;//Get_starting_byte();
  /*
  if(downloaded_size > 0) {
    string old_filename = ret_Options().ret_Store_Dir()+ret_Filename();
    string new_filename = old_filename+".-1";
    if(rename(old_filename.c_str(), new_filename.c_str()) < 0) {
      downloaded_size = 0;
    }
  }
  */
  //else {
  //  struct stat filestat;
  //  string filename = ret_Options().ret_Store_Dir()+ret_Filename()+".-1";
  // if(stat(filename.c_str(), &filestat) >= 0) {
  //   downloaded_size = filestat.st_size;
  // }
  //
  unsigned int unit, unit_reminder;
  while(1) {
    unit = (total_size-downloaded_size)/divide;
    if(unit > 0) {
      unit_reminder = (total_size-downloaded_size)%divide;
      break;
    }
    if(--divide == 1) return false;
  }
  begin_split();
  ret_Options().set_Divide(divide);
  for(unsigned int i = 0; i < divide; i++) {
    unsigned int start_range = unit*i+downloaded_size;
    unsigned int end_range;
    if(i == divide-1) {
      end_range = unit*(i+1)+downloaded_size+unit_reminder;
    } else {
      end_range = unit*(i+1)+downloaded_size;
    }
    URLcontainer urlcon_partial = ret_URL_Container();
    Options options_partial = ret_Options();
    options_partial.set_downm_type(Options::DOWNM_ALWAYSRESUME);
    options_partial.set_Divide(1);
    ItemCell_HTTP_p *itemcell_partial = new ItemCell_HTTP_p(ret_URL(),
							    urlcon_partial,
							    options_partial,
							    "created",
							    this,
							    i,
							    start_range,
							    end_range);
    itemcell_partial->cookie_list = cookie_list;
    itemcell_partial->set_Status(ITEM_READY);

    Send_partial(itemcell_partial);
    Send_message_to_gui(_("partial item added"), MSG_DOWNLOAD_INFO);
  }
  end_split();
  return true;
}

ItemCell::DownloadStatusType ItemCell_HTTP::Download_Main()
{
  unsigned int startingbyte = 0;
  string report;
  ItemStatusType initialstatus;

  if(session_counter == 1 && ret_Status() != ITEM_DOWNLOAD_INTERNAL_AGAIN) {
    ret_URL_Container().Parse_URL(ret_URL());
    retrieved_urlcon.clear();
    if(ret_Options().ret_Cookie_delete_on_restart()) cookie_list.all_clear();
  }

  // Ƶ > 1ǥɥȥ롼Ȥʤɥȥ롼Ȥ򸽺ߤ
  // ¸ǥ쥯ȥꤹ
  if(ret_Options().ret_recurse_count() > 1 && ret_documentroot_dir().empty()) {
    set_documentroot_dir(ret_Options().ret_Store_Dir());
    if(ret_Options().ret_with_hostname_dir()) {
      //
      string fix_dir;
      if(ret_Options().ret_convert_tilde()) {
	fix_dir = convert_tilde(ret_URL_Container().ret_Dir());
      } else {
	fix_dir = ret_URL_Container().ret_Dir();
      }
      ret_Options().set_Store_Dir(ret_documentroot_dir()+ret_URL_Container().ret_Hostname()+fix_dir);
    }
  } else if(ret_Options().ret_recurse_count() > 1 &&
	    ret_Options().ret_with_hostname_dir() &&
	    ret_Options().ret_Store_Dir().find(ret_URL_Container().ret_Hostname()+ret_URL_Container().ret_Dir()) == string::npos) {
    string fix_dir;
    if(ret_Options().ret_convert_tilde()) {
      fix_dir = convert_tilde(ret_URL_Container().ret_Dir());
    } else {
      fix_dir = ret_URL_Container().ret_Dir();
    }
    ret_Options().set_Store_Dir(ret_documentroot_dir()+ret_URL_Container().ret_Hostname()+fix_dir);
  }
  try {
    if(!appoption->ret_use_servertemplate()) {
      svt.set_valid(false);
    } else if(session_counter == 1 || !svt.Is_valid(session_counter) || svt.ret_session(session_counter).Is_readconfig()) {
      svt = svt_list.search(ret_URL_Container().ret_Hostname(), ret_URL_Container().ret_Filename());
      session_counter = 1;
    }
    if(svt.Is_valid(session_counter)) {
      //if(session_counter == 1) {
      //	cookie_list.all_clear();
      //}    
      Send_message_to_gui("Using Server Template '"+svt.ret_template_name()+"'", MSG_DOWNLOAD_INFO);
      Send_message_to_gui("Entering Session "+itos(session_counter), MSG_DOWNLOAD_INFO);
      // modified 2001/3/9
      URLcontainer orig_urlcon;
      orig_urlcon.Parse_URL(ret_URL());
      if(svt.ret_session(session_counter).ret_get_vector().size() && !ret_URL_Container().Parse_URL(svt.ret_session(session_counter).Create_URL_from_get_vector(orig_urlcon, retrieved_urlcon))) {
	throw ITEM_ESERVERCONFIG;
      }
      if(svt.ret_session(session_counter).ret_referer_vector().size()) {
	ret_Options().set_Referer_Type(Options::REFERER_USER_DEFINED);
	ret_Options().set_Referer(svt.ret_session(session_counter).Create_URL_from_referer_vector(orig_urlcon, retrieved_urlcon));
      }
    }

    if(ret_Status() != ITEM_CRCERROR &&
       ret_Status() != ITEM_EXECERROR &&
       ret_Status() != ITEM_INUSE_AGAIN &&
       ret_Status() != ITEM_DOWNLOAD_AGAIN &&
       ret_Options().ret_downm_type() != Options::DOWNM_NORESUME &&
       ret_Filename().size() &&
       (!svt.Is_valid(session_counter) || !svt.ret_session(session_counter).Is_noresume() && !svt.ret_session(session_counter).Is_nodown() && !svt.ret_session(session_counter).Is_getkeylink() && !svt.ret_session(session_counter).Is_getkeylink_force())) {
      startingbyte = Get_starting_byte();
    }
    initialstatus = ret_Status();
    set_Status(ITEM_DOWNLOAD);
    Send_status();

    // connect to www server
    Socket sock;
    if(sock.bad()) {
      throw ITEM_EIO;
    }
    Make_TCP_connection(sock, ret_URL_Container().ret_Port());

    // send GET request to www server]
    Send_Request(sock, startingbyte);

    // get HTTP header
    HTTPHeaderList httpheaderlist;

    Get_HTTP_header(sock, httpheaderlist);

    // analyze HTTP header and store it to class HTTPcontainer
    HTTPcontainer httpcon;

    httpcon.Parse_HTTP_header(httpheaderlist);
    switch(httpcon.ret_HTTP_Status()) {
    case MultipleChoices:
    case MovedPermanently:
    case Found:
    case SeeOther:
      {
	URLcontainer urlcon_new;
	string location = URLcontainer::URL_Decode(httpcon.ret_Location());
	if(!urlcon_new.Parse_URL(location)) {
	  location += '/';
	  if(!urlcon_new.Parse_URL(location)) {
	    //Send_message_to_gui(_("Location retrieval failed"), MSG_DOWNLOAD_ERROR);
	    throw ITEM_ELOCATION;
	  }
	}
	// set Cookie
	for(list<string>::const_iterator itr = httpcon.ret_Cookie_list().begin();
	    itr != httpcon.ret_Cookie_list().end(); ++itr) {
	  cookie_list.add_cookie(*itr);
	}
	ret_Options().set_Referer_Type(Options::REFERER_URL);

	if(urlcon_new.ret_Protocol() != ret_URL_Container().ret_Protocol()) {
	  get_Options_Lock();
	  set_URL_opt(location);
	  set_URL_Container_opt(urlcon_new);
	  // generally, the password for redirected site is not the same
	  // so disable authentication option here
	  ret_Options_opt().set_use_authentication(false);
	  release_Options_Lock();
	  Raise_option_update_flag();
	} else {
	  retrieved_urlcon = urlcon_new;
	  set_URL_Container(urlcon_new);
	}
	report = _("redirecting to ")+httpcon.ret_Location();
	Send_message_to_gui(report, MSG_DOWNLOAD_INFO);

	if(svt.Is_valid(session_counter)) {
	  throw ITEM_ENEXTSTAGE;
	} else {
	  return DLINTERNALAGAIN;
	}
      }
    case NotModified:
      Send_message_to_gui("'"+ret_Filename()+"'"+_(" is already downloaded"), MSG_DOWNLOAD_INFO);
      set_Size_Current(startingbyte);
      set_Size_Total(startingbyte);
      Send_status();
      throw ITEM_ESUCCESSALR;
    }
    if(ret_Filename().empty()) {
      ret_URL_Container().set_File("/index.html");
    }
    /*
    catch (HTTPconErrorType err) {
      Send_message_to_gui(_("'Use Proxy' is not supported yet"), MSG_DOWNLOAD_ERROR);
      return DLSTOP;
    }
    */
    // set Cookie
    for(list<string>::const_iterator itr = httpcon.ret_Cookie_list().begin();
	itr != httpcon.ret_Cookie_list().end(); ++itr) {
      cookie_list.add_cookie(*itr);
    }
    // at this point, httpheaderlist is no longer exists
    if(httpcon.ret_ContentLength() == 0) {
      if(httpcon.ret_ContentLength() < startingbyte) {
	Send_message_to_gui(_("resume disabled"), MSG_DOWNLOAD_INFO);
      }
      set_Size_Total(0);
      startingbyte = 0; // added
    } else if(httpcon.ret_ContentLength() == startingbyte && startingbyte != 0) {
      //Ǥ˥ɺ
      // download has been competed
      report = "'"+ret_Filename()+"'"+_(" is already downloaded");
      Send_message_to_gui(report, MSG_DOWNLOAD_INFO);
      
      set_Size_Total(startingbyte);
      set_Size_Current(startingbyte);
      Send_status();
      //Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
      //session_counter = 1;
      //return DLSUCCESSALR;
      throw ITEM_ESUCCESSALR;
    } else {
      bool resume_allowed = true;
      if(httpcon.ret_ContentLength() < startingbyte &&
	 httpcon.ret_HTTP_Status() != PartialContent) {
	// ե륵ۤʤ
	// size mismatch
	Send_message_to_gui(_("size of local file is larger than that of remote file. Download again"), MSG_DOWNLOAD_ERROR);
	startingbyte = 0;
	set_Size_Total(httpcon.ret_ContentLength());
      } else if(startingbyte > 0 && httpcon.ret_HTTP_Status() == OK) {
	// 쥸塼Բ
	// www server does not support resume
	resume_allowed = false;
	Send_message_to_gui(_("resume disabled"), MSG_DOWNLOAD_INFO);
	startingbyte = 0;
	set_Size_Total(httpcon.ret_ContentLength());
      } else {
	set_Size_Total(httpcon.ret_ContentLength()+startingbyte);
      }
      if(ret_Options().ret_Divide() > 1 && resume_allowed &&
	 (!svt.Is_valid(session_counter) || !svt.ret_session(session_counter).Is_nodown() && !svt.ret_session(session_counter).Is_getkeylink() && !svt.ret_session(session_counter).Is_getkeylink_force())) {
	set_Size_Current(0);
	// check whether or not remote server suports resume
	Send_message_to_gui(_("checking whether server supports resuming"), MSG_DOWNLOAD_INFO);
	
	sock.reopen();
	// connect to www server	
	Make_TCP_connection(sock, ret_URL_Container().ret_Port());
		
	// send GET request to www server]
	////backup downm
	Options::DownloadMethodType downm_temp = ret_Options().ret_downm_type();
      	ret_Options().set_downm_type(Options::DOWNM_ALWAYSRESUME);
	Send_Request(sock, 1);
	ret_Options().set_downm_type(downm_temp);
	// get HTTP header
	Get_HTTP_header(sock, httpheaderlist);
	// analyze HTTP header and store it to class HTTPcontainer
	HTTPcontainer httpcon_tmp;
	httpcon_tmp.Parse_HTTP_header(httpheaderlist);
	int http_status = httpcon_tmp.ret_HTTP_Status();
	
	if(http_status !=  PartialContent) {
	  Send_message_to_gui(_("resume disabled"), MSG_DOWNLOAD_ERROR);
	  Send_message_to_gui(_("starting normal download instead"), MSG_DOWNLOAD_INFO);
	} else {
	  Send_message_to_gui(_("splitting file..."), MSG_DOWNLOAD_INFO);
	  if(Create_partial_item_entry(ret_Options().ret_Divide(), ret_Size_Total())) {
	    set_Status(ITEM_DOWNLOAD_PARTIAL);
	    Send_status();
	    set_Command(DLPARTIALSUCCESS);
	    throw ITEM_EINTER;
	  } else {
	    Send_message_to_gui(_("file is too small to be devided"), MSG_DOWNLOAD_ERROR);
	    Send_message_to_gui(_("starting normal download instead"), MSG_DOWNLOAD_INFO);
	  }
	}
      }
    }

    // start downloading
    Start_Download(sock, startingbyte);
    
    // close connection
    sock.Shutdown(2);
    throw ITEM_ESUCCESS;
  } catch (ItemErrorType err) {
    switch(err) {
    case ITEM_ESUCCESS:
      Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
      //session_counter = 1; // modified 2001/3/16
      set_Command(DLSUCCESS);
      return DLSUCCESS;
    case ITEM_ESUCCESSALR:
      Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
      //session_counter = 1; // modified 2001/3/16
      set_Command(DLSUCCESSALR);
      return DLSUCCESSALR;
    case ITEM_ENEXTSTAGE:
      if(!Execute_program()) {
	session_counter = 1;
	set_Command(DLERROR);
	return ret_Dl_status();
      }
      if((svt.ret_session(session_counter).Is_getkeylink() ||
	 svt.ret_session(session_counter).Is_getkeylink_force()) &&
	 retrieved_urlcon.ret_Protocol() == "ftp:") {
	// acquire options lock
	get_Options_Lock();
	ret_URL_Container_opt() = retrieved_urlcon;
	set_URL_opt(retrieved_urlcon.ret_Protocol()+"//"+retrieved_urlcon.ret_Hostname()+retrieved_urlcon.ret_Dir()+retrieved_urlcon.ret_File());
	ret_Options_opt().set_use_authentication(false);//disable authentication for safety
	Raise_option_update_flag();
	// release lock;
	release_Options_Lock();
      } else {
	++session_counter;
	if(svt.Is_valid(session_counter)) {
	  // forward currest URL to the next session
	  // so that it can be used as referer in the next session
	  svt.ret_session(session_counter).set_session_referer(ret_URL_Container());
	}
	ret_Options().set_Referer_Type(Options::REFERER_USER_DEFINED);
	ret_Options().set_Referer("http://"+ret_URL_Container().ret_Hostname()+
				  ret_URL_Container().ret_Dir()+
				  ret_URL_Container().ret_File()+
				  ret_URL_Container().ret_Query());
      }
      set_Size_Total(0);
      set_Size_Current(0);

      if(initialstatus == ITEM_CRCERROR ||
	 initialstatus == ITEM_EXECERROR ||
	 initialstatus == ITEM_INUSE_AGAIN ||
	 initialstatus == ITEM_DOWNLOAD_AGAIN) {
	set_Command(DLAGAIN);
      } else {
	set_Command(DLINTERNALAGAIN);
      }
      return ret_Dl_status();
    case ITEM_EINTER:
      session_counter = 1;
      return ret_Dl_status();
    case ITEM_ELOCATION:// fixed
    case ITEM_ESERVERCONFIG://fixed
    case ITEM_EIOFILE:
      session_counter = 1;
      set_Command(DLERRORSTOP);
      PERROR(err);
      return DLERRORSTOP;
    case ITEM_EPROT:// protocl error // modified
      session_counter = 1;
      PERROR(err);
      return ret_Dl_status();
    default:
      session_counter = 1;
      set_Command(DLERROR);
      PERROR(err);
      return ret_Dl_status();
    }
  }
}

//
// ե
//
// return:
//     -1: 顼
//      0: 桼ˤ
//      1: 
//
void ItemCell_HTTP::Start_Download(const Socket& socket, unsigned int startingbyte)
{
  ofstream outfile;
  DownloadMode downloadmode = NORMAL_MODE;
  string line;

  Make_directory_if_needed();

  //ret_URL_Container().Parse_URL(ret_URL());
  string filename = ret_Options().ret_Store_Dir()+ret_Filename();
  string tempfilename;
  Send_status();

  try {
    if(svt.Is_valid(session_counter) &&
       svt.ret_session(session_counter).Is_nodown()) {
      throw ITEM_ENEXTSTAGE;
    }
    if(((ret_Size_Total() == 0 || ret_Options().ret_use_http_proxy() && !ret_Options().ret_http_proxy().ret_Server().empty()) &&
	svt.Is_valid(session_counter) &&
	svt.ret_session(session_counter).Is_getkeylink()) ||
       (svt.Is_valid(session_counter) &&
	svt.ret_session(session_counter).Is_getkeylink_force())) {
      tempfilename = filename+"."+itos(time(NULL))+itos((int)((float)100*random()/(RAND_MAX+1.0)));
      downloadmode = EMBEDED_URL_MODE;
      outfile.open(tempfilename.c_str(), ios::out|ios::trunc|ios::binary);
      startingbyte = 0;
    } else if(svt.Is_valid(session_counter) && svt.ret_session(session_counter).Is_addhref()) {
      tempfilename = filename+"."+itos(time(NULL))+itos((int)((float)100*random()/(RAND_MAX+1.0)));

      downloadmode = ADD_HREF_MODE;
      outfile.open(tempfilename.c_str(), ios::out|ios::trunc|ios::binary);
      startingbyte = 0;
    } else {

      line = _("starting download at ")+itos(startingbyte, true)+_("bytes");
      Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
    
      if(startingbyte == 0) {
	// ˥ɳ
	outfile.open(filename.c_str(), ios::out|ios::trunc|ios::binary);
      } else {
	// 쥸塼ह
	outfile.open(filename.c_str(), ios::out|ios::app|ios::binary);
      }
    }
    if(outfile.bad()) {
      throw ITEM_EIOFILE;
    }    
    set_Size_Current(startingbyte);
    Download_data(outfile, socket);
    outfile.close();

    switch(downloadmode) {
    case EMBEDED_URL_MODE:
      {
	Send_message_to_gui(_("retrieving embeded URL"), MSG_DOWNLOAD_INFO);
	URLcontainer trueurlcon;
	list<string> keylink_list;
	URLcontainer orig_urlcon;
	orig_urlcon.Parse_URL(ret_URL());
	for(list< vector<string> >::const_iterator itr = svt.ret_session(session_counter).ret_keylink_list().begin();
	    itr != svt.ret_session(session_counter).ret_keylink_list().end(); ++itr) {
	  string keylink = svt.ret_session(session_counter).Create_URL(orig_urlcon, retrieved_urlcon, *itr);
	  if(keylink.size())
	    keylink_list.push_back(keylink);
	}
	if(!trueurlcon.Retrieve_embedded_URL(keylink_list, tempfilename, ret_URL_Container())) {
	  unlink(tempfilename.c_str());
	  throw ITEM_ECANTFINDURL;
	}
	unlink(tempfilename.c_str());
	retrieved_urlcon = trueurlcon;
	throw ITEM_ENEXTSTAGE;
      }
    case ADD_HREF_MODE:
      {
	Send_message_to_gui(_("finding Hyperlink"), MSG_DOWNLOAD_INFO);

	itemlist_global->Find_Hyperlink_from_file(tempfilename, ret_URL_Container().ret_Protocol()+"//"+ret_URL_Container().ret_Hostname()+ret_URL_Container().ret_Dir(), ItemList::FINDHREF_ADD);
	Send_message_to_gui(_("Hyperlink finding complete"), MSG_DOWNLOAD_INFO);
	unlink(tempfilename.c_str());
	throw ITEM_ENEXTSTAGE;
      }
    case NORMAL_MODE:
    default:
      {
	if(ret_Size_Current() != ret_Size_Total() && ret_Size_Total() != 0) {
	  bool flag = true;
	  if(svt.Is_valid(session_counter)) {
	    if(!(ret_Size_Total()-svt.ret_session(session_counter).ret_post_offset_size()-ret_Size_Current())) flag = false;
	  }
	  if(flag) {
	    Send_message_to_gui(_("no match size"), MSG_DOWNLOAD_ERROR);
	    set_Command(DLERROR);
	    throw ITEM_EPROT;
	  }
	} else {
	  if(svt.Is_valid(session_counter) &&
	     svt.ret_session(session_counter).ret_post_offset_size() != 0 &&
	     svt.ret_session(session_counter).ret_post_offset_string().size()) {
	    fstream outfile(filename.c_str(), ios::in|ios::out|ios::app|ios::binary);
	    outfile.seekg(-svt.ret_session(session_counter).ret_post_offset_size(), ios::end);
	    char *buffer = new char[svt.ret_session(session_counter).ret_post_offset_size()+1];
	    outfile.read(buffer, svt.ret_session(session_counter).ret_post_offset_size());
	    buffer[svt.ret_session(session_counter).ret_post_offset_size()] = '\0';
	    if(memcmp(buffer, svt.ret_session(session_counter).ret_post_offset_string().c_str(), svt.ret_session(session_counter).ret_post_offset_size()) == 0) {
	      truncate(filename.c_str(), ret_Size_Total()-svt.ret_session(session_counter).ret_post_offset_size());
	      Send_message_to_gui(itos(svt.ret_session(session_counter).ret_post_offset_size())+_(" bytes garbage header removed"), MSG_DOWNLOAD_INFO);
	    }
	  } else {
	    set_Size_Total(ret_Size_Current());
	  }
	}
      }
    }
  } catch (ItemErrorType err) {
    if(downloadmode == EMBEDED_URL_MODE || downloadmode == ADD_HREF_MODE) {
      outfile.close();
      unlink(tempfilename.c_str());
    }
    throw err;
  }
}

bool ItemCell_HTTP::Execute_program()
{
  bool retval = true;
  if(svt.Is_valid(session_counter) && svt.ret_session(session_counter).Is_program_avaiable()) {
    Send_message_to_gui(_("executing program: ")+svt.ret_session(session_counter).ret_program_line(this), MSG_DOWNLOAD_INFO);
    int return_status = svt.ret_session(session_counter).Execute_program(this);
    switch(return_status) {
    case Session::SESSION_EXEC_SUCC:
      Send_message_to_gui(_("good exit status"), MSG_DOWNLOAD_SUCCESS);
      retval = true;
      break;
    case Session::SESSION_EXEC_FAIL:
      Send_message_to_gui(_("bad exit status"), MSG_DOWNLOAD_ERROR);
      retval = false;
      break;
    case Session::SESSION_EXEC_IGN_FATAL:
      Send_message_to_gui(_("program exited abnormally"), MSG_DOWNLOAD_ERROR);
    case Session::SESSION_EXEC_IGN:
    default:
      Send_message_to_gui(_("exit status ignored"), MSG_DOWNLOAD_INFO);
      retval = true;
      break;
    }
  }
  return retval;
}

ItemCell::DownloadStatusType ItemCell_HTTP::Post_process()
{
  ItemCell::DownloadStatusType retval = ItemCell::Post_process();
  if(retval == DLSUCCESS) {
    if(!Execute_program()) {
      set_Status(ITEM_EXECERROR);
      set_Command(DLERROR);
      retval = DLERROR;
    }
  }
  session_counter = 1;
  return retval;
}
