//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemCell_HTTP_p.cc,v 1.18 2001/03/19 05:33:40 tujikawa Exp $

#include "ItemCell_HTTP_p.h"

ItemCell_HTTP_p::ItemCell_HTTP_p(const string& url_in,
				 const URLcontainer& urlcon_in,
				 const Options& options_in,
				 const string& initial_log,
				 ItemCell *itemcell_boss,
				 unsigned int order_in,
				 unsigned int start_range_in,
				 unsigned int end_range_in)
  :ItemCell_HTTP(url_in, urlcon_in, options_in, initial_log),
   ItemCell_Partial(url_in, urlcon_in, options_in, initial_log, itemcell_boss, order_in, start_range_in, end_range_in),
   ItemCell(url_in, urlcon_in, options_in, initial_log)
{
}

ItemCell_HTTP_p::~ItemCell_HTTP_p()
{
  // nothing to be done here
}

ItemCell::DownloadStatusType ItemCell_HTTP_p::Download_Main()
{
  unsigned int startingbyte = 0;
  string report;

  try {
    if(ret_Status() != ITEM_CRCERROR &&
       ret_Status() != ITEM_EXECERROR &&
       ret_Status() != ITEM_DOWNLOAD_AGAIN &&
       ret_Options().ret_downm_type() != Options::DOWNM_NORESUME) {
      startingbyte = Get_starting_byte();
    }
    set_Status(ITEM_DOWNLOAD);
    Send_status();
    
    // connect to www server    
    Socket socket;
    if(socket.bad()) {
      throw ITEM_EIO;
    }
    Make_TCP_connection(socket, ret_URL_Container().ret_Port());
  
    // send GET request to www server]
    unsigned int startingbyte_real = startingbyte+start_range;
    Send_Request(socket, startingbyte_real);
  
    // get HTTP header
    HTTPHeaderList httpheaderlist;
    Get_HTTP_header(socket, httpheaderlist);

    // analyze HTTP header and store it to class HTTPcontainer
    HTTPcontainer httpcon;

    httpcon.Parse_HTTP_header(httpheaderlist);
    switch(httpcon.ret_HTTP_Status()) {
    case MultipleChoices:
    case MovedPermanently:
    case Found:
    case SeeOther: {
      URLcontainer urlcon_new;
      string location = URLcontainer::URL_Decode(httpcon.ret_Location());
      if(!urlcon_new.Parse_URL(location)) {
	location += '/';
	if(!urlcon_new.Parse_URL(location)) {
	  throw ITEM_ELOCATION;
	}
      }
      set_URL_Container(urlcon_new);
      
      report = _("redirecting to")+httpcon.ret_Location();
      Send_message_to_gui(report, MSG_DOWNLOAD_INFO);
      
      return DLAGAIN;
    }
    }

    // at this point, httpheaderlist is no longer exists

    if(startingbyte+start_range == end_range && startingbyte != 0) {
      //Ǥ˥ɺ
      // downloading has been competed
      report = "'"+ret_Filename()+_("' is already downloaded");
      Send_message_to_gui(report, MSG_DOWNLOAD_INFO);
      
      set_Size_Total(startingbyte);
      set_Size_Current(startingbyte);
      Send_status();
      Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
      throw ITEM_ESUCCESS;
    } else if(startingbyte+start_range > end_range &&
	      httpcon.ret_HTTP_Status() != PartialContent) {
      // ե륵ۤʤ
      // size mismatch
      Send_message_to_gui(_("size of local file is larger than that of remote file. Download again"), MSG_DOWNLOAD_ERROR);
      startingbyte = 0;
      set_Size_Total(end_range-start_range);
    } else if(start_range+startingbyte > 0 && httpcon.ret_HTTP_Status() == OK) {
      // 쥸塼Բ
      // www server does not support resume
      Send_message_to_gui(_("sorry, cannot resume"), MSG_DOWNLOAD_INFO);
      startingbyte = 0;
      set_Size_Total(end_range-start_range);
      set_Command(DLERRORSTOP);
      throw ITEM_EINTER;
    } else {
      set_Size_Total(end_range-start_range);
    }
    // start downloading
    Start_Download(socket, startingbyte);
    
    Send_message_to_gui(_("connection closed"), MSG_DOWNLOAD_INFO);
    throw ITEM_ESUCCESS;
  } catch (ItemErrorType err) {
    switch(err) {
    case ITEM_ESUCCESS:
      set_Command(DLSUCCESS);
      return DLSUCCESS;
    case ITEM_EINTER:
      return ret_Command();
    case ITEM_EIOFILE:
      set_Command(DLERRORSTOP);
      PERROR(err);
      return DLERRORSTOP;
    default:
      PERROR(err);
      set_Command(DLERROR);
      return DLERROR;
    }
  }
}

//
// ե
//
// return:
//     -1: 顼
//      0: 桼ˤ
//      1: 
//
void ItemCell_HTTP_p::Start_Download(const Socket& socket, unsigned int startingbyte)
{
  ofstream outfile;
  string line;

  Make_directory_if_needed();

  //URLcontainer urlcon_temp = ret_URL_Container();
  //ret_URL_Container().Parse_URL(ret_URL());
  string filename = ret_Options().ret_Store_Dir()+ret_Filename();
  Send_status();

  switch(SplitNumberChanged(filename+".index")) {
  case PARTIAL_NORMAL:
    break;
  case PARTIAL_CHANGED:
    //Send_message_to_gui(_("split count has changed"), MSG_DOWNLOAD_INFO);
  default:
    startingbyte = 0;
  }

  try {
    line = _("a part of '")+ret_URL()+"'"+_(", its index is ")+itos(order)+_(", its range is from ")+itos(start_range, true)+" to "+itos(end_range, true);
    Send_message_to_gui(line, MSG_DOWNLOAD_INFO);
    
    line = _("starting download at ")+itos(start_range+startingbyte, true)+_("bytes");
    Send_message_to_gui(line, MSG_DOWNLOAD_INFO);

    if(startingbyte == 0) {
      // ˥ɳ
      outfile.open(filename.c_str(), ios::out|ios::trunc|ios::binary);
    } else {
      // 쥸塼ह
      outfile.open(filename.c_str(), ios::out|ios::app|ios::binary);
    }
    if(outfile.bad()) {
      throw ITEM_EIOFILE;
    }

    set_Size_Current(startingbyte);
    Download_data(outfile, socket);
    outfile.close();
    //set_URL_Container(urlcon_temp);
  } catch (ItemErrorType err) {
    //set_URL_Container(urlcon_temp);
    throw err;
  }
}

ItemCell::DownloadStatusType ItemCell_HTTP_p::Post_process()
{
  return ItemCell_Partial::Post_process();
}

