//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemCell_Partial.cc,v 1.10 2001/03/09 06:49:10 tujikawa Exp $

#include "ItemCell_Partial.h"

extern ItemCell *console_item;
extern bool CRC_check_main(ItemCell *itemcell);
extern pthread_mutex_t itemlistlock;

ItemCell_Partial::ItemCell_Partial(const string& url_in,
				   const URLcontainer& urlcon_in,
				   const Options& options_in,
				   const string& initial_log,
				   ItemCell *itemcell_boss,
				   unsigned int order_in,
				   unsigned int start_range_in,
				   unsigned int end_range_in)
  : ItemCell(url_in, urlcon_in, options_in, initial_log)
{
  order = order_in;
  start_range = start_range_in;
  end_range = end_range_in;
  boss = itemcell_boss;

  filename_partial = urlcon_in.ret_Filename()+"."+itos(order);
}

ItemCell_Partial::~ItemCell_Partial()
{
}

unsigned int ItemCell_Partial::ret_Order() const
{
  return order;
}

bool ItemCell_Partial::Is_Partial() const
{
  return true;
}

string ItemCell_Partial::ret_Filename() const
{
  return filename_partial;
}

string ItemCell_Partial::ret_Filename_opt() const
{
  return filename_partial;
}

unsigned int ItemCell_Partial::ret_Start_range() const
{
  return start_range;
}
unsigned int ItemCell_Partial::ret_End_range() const
{
  return end_range;
}

ItemCell::DownloadStatusType ItemCell_Partial::Post_process()
{
  DownloadStatusType retval = DLSUCCESS;

  pthread_mutex_lock(&itemlistlock);
  boss->Remove_worker(this);
  if(boss->No_more_worker()) {
    boss->set_Status(ITEM_READY_CONCAT);
    boss->Send_status();
  }
  pthread_mutex_unlock(&itemlistlock);
  return retval;
}

void ItemCell_Partial::WriteSplitInfo(const string& filename)
{
  ofstream indexfile(filename.c_str(), ios::out);
  if(indexfile.bad()) {
    cerr << "index file status:bad" << endl;
    return;
  } else {
    indexfile << start_range << '\t' << end_range;
  }
}

SplitStatusType ItemCell_Partial::SplitNumberChanged(const string& filename)
{
  struct stat file_stat;
  if(!stat(filename.c_str(), &file_stat)) {
    if(S_ISREG(file_stat.st_mode)) {
      // read file
      ifstream indexfile(filename.c_str(), ios::in);
      if(indexfile.bad()) return PARTIAL_NOINDEX;
      unsigned int start_range_in, end_range_in;
      indexfile >> start_range_in >> end_range_in;
      indexfile.close();
      if(start_range == start_range_in && end_range == end_range_in) {
	return PARTIAL_NORMAL;
      } else {
	WriteSplitInfo(filename);
	return PARTIAL_CHANGED;
      }
    } else {
      return PARTIAL_NOINDEX;
    }
  } else {
    WriteSplitInfo(filename);
    return PARTIAL_NOINDEX;
  }
}
