//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: ItemList.cc,v 1.38 2001/03/22 08:42:17 tujikawa Exp $

// implementaion of class ItemList

#include "ItemList.h"

//#define stoi(ARG, ARG2) strtol(ARG.c_str(), (char**)NULL, ARG2)

extern void Set_clist_column__crc(int rowindex, const string& crc_string);
extern void Add_new_item_to_downloadlist(ItemCell *itemcell);
extern void Set_clist_column_style__complete(int rowindex);
extern void Set_clist_column_style__ready(int rowindex);
extern void Set_clist_column__progress(int rowindex, int progress);
extern void Set_clist_column__icon(int rowindex , ItemCell::ItemStatusType status);
extern void create_default_filter_nodown_target_list(list<string>& filter_target_list);
extern void create_default_parse_target_list(list<string>& parse_target_list);
extern GtkWidget *itemlistwidget;
extern ItemCell *console_item;
extern AppOption *appoption;
extern ItemManager *item_manager;
extern ItemManager *item_manager_paste;
extern ProxyList *http_proxy_list;
extern UseragentList *useragent_list;
extern pthread_mutex_t itemlistlock;
extern PasteWindow *paste_window;
extern unsigned int thread_limit;

void parse_extention_target_list(list<string>& target_list, string source_string)
{
  while(source_string.size()) {
    string target = Token_splitter(source_string, " \t");
    if(target.size()) {
      target_list.push_back(target);
    }
  }
}

ItemList::ItemList()
{
  // default save files
  string home_dir = g_get_home_dir();
  string save_dir = home_dir+"/.aria/save/";
  
  file_default_item_settings = save_dir+"default.aria";
  file_default_item_list = save_dir+"list.aria";
  file_app_settings = save_dir+"app.aria";
  file_server_settings = save_dir+"server.aria";
  file_http_proxy_list = save_dir+"http_proxy.aria";
  file_useragent_list = save_dir+"useragent.aria";
  file_command_list = save_dir+"command.aria";
  file_history = save_dir+"history.aria";
}

list<ItemCell*> ItemList::Recursive_add_http_item(ItemCell *itemcell)
{
  const URLcontainer& p_urlcon = itemcell->ret_URL_Container();
  const Options& p_options = itemcell->ret_Options();
  const string& documentroot_dir = itemcell->ret_documentroot_dir();
  const string& root_url = itemcell->ret_root_url();

  int count = 0;
  list<ItemCell*> item_list;
  //cerr << documentroot_dir << endl;
  int n_rec = p_options.ret_recurse_count()-1;
  const string filename = p_options.ret_Store_Dir()+p_urlcon.ret_Filename();
  const string base_url = p_urlcon.ret_Protocol()+"//"+p_urlcon.ret_Hostname()+p_urlcon.ret_Dir();
  try {
    HTMLparse htmlparse(base_url,
			root_url,
			documentroot_dir,
			p_options,
			filename);
    if(htmlparse.in_bad()) {
      return item_list;
    }
    while(1) {
      Options options = p_options;
      URLcontainer urlcon = htmlparse.get_href(options);
      if(urlcon.bad() || p_options.ret_recurse_count() <= 1) continue;//fixed 2001/3/14

      string url = urlcon.ret_URL();
      if(!item_manager->search_by_url(url)) {
	options.set_recurse_count(n_rec);
	if(p_options.ret_Referer_override()) {// referer override
	  options.set_Referer_Type(Options::REFERER_USER_DEFINED);
	  options.set_Referer(p_urlcon.ret_URL());
	}
	ItemCell *itemcell;
	if(urlcon.ret_Protocol() == "http:") {
	  itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
	} else if(urlcon.ret_Protocol() == "ftp:") {
	  options.set_use_authentication(false);// added 2001/3/14
	  itemcell = new ItemCell_FTP(url, urlcon, options, "created");
	} else {
	  continue;
	}
	if(urlcon.ret_Protocol() == "http:") {
	  itemcell->set_documentroot_dir(documentroot_dir);
	  itemcell->set_root_url(root_url);
	}
	++count;
	//Add_new_item_to_downloadlist(itemcell);
	item_list.push_back(itemcell);
      }
    }
    throw HTMLPARSE_EOF;
  } catch (HTMLparseExceptionType err) {
    switch(err) {
    case HTMLPARSE_EOF:
      {
	/*
	if(count) {
	  console_item->Send_message_to_gui(itos(count)+_("item(s) added"), MSG_SYS_INFO);
	}
	*/
	break;
      }
    default:
      {
	/*
	console_item->Send_message_to_gui(_("parse error in \"")+filename+_("\". Abort further parsing operation"), MSG_SYS_ERROR);
	retval = false;
	*/
      }
    }
    if(p_options.ret_use_down_filter() &&
       !p_options.Is_in_activated_filter_down_target_list(p_urlcon.ret_Filename())) {
      unlink(filename.c_str());
      itemcell->get_Options_Lock();
      itemcell->ret_Options_opt().set_Delete_When_Finish(true);
      itemcell->ret_Options_opt().set_Dont_Delete_Without_CRC(false);
      itemcell->release_Options_Lock();
    }
  }
  return item_list;
}

bool ItemList::Read_CRC_from_file(const string& filename)
{
  ifstream infile;

  infile.open(filename.c_str(), ios::skipws|ios::in);
  if(infile.bad()) {
    return false;
  }
  list<CRCList*> crc_list;
  while(1) {
    string filename_entry;
    string crc_string;
    string size_string;
    //char filename_entry_c[512];
    //char crc_string_c[512];
    //char size_string_c[512];
    string line;
    getline(infile, line, '\n');
    ////modified 2001/3/1
    //sscanf(line, "%s %s %s", filename_entry_c, crc_string_c, size_string_c);
    filename_entry = Token_splitter(line, " ");
    crc_string = Token_splitter(line, " ");
    size_string = Token_splitter(line, " ");

    //filename_entry = filename_entry_c;
    //crc_string = crc_string_c;
    //size_string = size_string_c;
    if(filename_entry.size() && crc_string.size()) {
      CRCList *crc_entry = new CRCList(filename_entry, crc_string);
      crc_list.push_back(crc_entry);
    }
    if(infile.eof()) break;
  }

  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  const list<int>& download_list = item_manager->ret_id_list();
  for(list<int>::const_iterator id_itr = download_list.begin(); id_itr != download_list.end(); ++id_itr) {
    ItemCell *itemcell = item_manager->ret_itemaddr(*id_itr);
    for(list<CRCList*>::iterator crc_list_itr = crc_list.begin(); crc_list_itr != crc_list.end(); ++crc_list_itr) {
      if(itemcell->ret_Filename() == (*crc_list_itr)->ref_Filename()) {
	int rowindex;
	switch((*crc_list_itr)->ref_CRC_string().size()) {
	case 8:
	  itemcell->set_CRC(stoui((*crc_list_itr)->ref_CRC_string(), 16));
	  itemcell->set_CRC_Type(ItemCell::CRC_32);
	  rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	  Set_clist_column__crc(rowindex, (*crc_list_itr)->ref_CRC_string());

	  crc_list.remove(*crc_list_itr);
	  delete *crc_list_itr;
	  break;
	case 4:
	  itemcell->set_CRC(stoui((*crc_list_itr)->ref_CRC_string(), 16));
	  itemcell->set_CRC_Type(ItemCell::CRC_16);
	  rowindex = gtk_clist_find_row_from_data(GTK_CLIST(itemlistwidget), itemcell);
	  Set_clist_column__crc(rowindex, (*crc_list_itr)->ref_CRC_string());
	  crc_list.remove(*crc_list_itr);
	  delete *crc_list_itr;
	  break;
	default:
	  cerr << "WARNING: unsupported CRC format" << endl;
	}
	break;
      }
    }
  }
  for(list<CRCList*>::iterator crc_list_itr = crc_list.begin(); crc_list_itr != crc_list.end(); ++crc_list_itr) {
    delete *crc_list_itr;
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  return true;
}

bool ItemList::Find_Hyperlink_from_file(const string& filename, const string& base_url, int mode)
{
  ifstream infile;
  infile.open(filename.c_str(), ios::skipws|ios::in);
  if(infile.bad()) {
    cerr << "ERROR:Unable to read from file " << filename << "\n";
    return false;
  }
  int count = 0;
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  while(1) {
    string urlbuf;
    getline(infile, urlbuf);
    while(!urlbuf.empty()) {
      string url;
      //if(mode == ItemList::STRICT_HREF) {
      //url = URLcontainer::Find_HREF_strict(urlbuf, base_url);
      //} else {
      url = URLcontainer::Find_HREF(urlbuf, base_url);
      //}
      // parse URL
      URLcontainer urlcon;
      if(urlcon.Parse_URL(url) && !item_manager->search_by_url(urlcon.ret_URL()) &&
	 (!appoption->ret_use_ignore_extention_list() ||
	  !appoption->Is_in_ignore_extention_list(urlcon.ret_File()))) {
	Options options;
	
	options = console_item->ret_Options_opt();
	ItemCell *itemcell;
	if(urlcon.ret_Protocol() == "http:") {
	  itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
	} else if(urlcon.ret_Protocol() == "ftp:") {
	  itemcell = new ItemCell_FTP(url, urlcon, options, "created");
	} else {
	  continue;
	}
	++count;
	if(mode == ItemList::FINDHREF_PASTE) {
	  paste_window->add_item(itemcell);
	} else if(mode == ItemList::FINDHREF_ADD || mode == ItemList::STRICT_HREF) {
	  //gdk_threads_enter();
	  pthread_mutex_lock(&itemlistlock);
	  Add_new_item_to_downloadlist(itemcell);
	  pthread_mutex_unlock(&itemlistlock);
	  //gdk_threads_leave();
	} else {
	  delete itemcell;
	}
      }
    }
    if(infile.eof()) break;
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  if(count && mode == ItemList::FINDHREF_PASTE) {
    paste_window->show();
  }
  return true;
}
  
  bool ItemList::Read_URL_from_file(const string& filename)
{
  ifstream infile;

  // if filename is "-", get URLs from stdin
  if(filename == "-") {
    cerr << "currently '-f -' is not functional. sorry\n";
    return false;
  } else {
    infile.open(filename.c_str(), ios::skipws|ios::in);
    if(infile.bad()) {
      cerr << "ERROR:Unable to read from file " << filename << "\n";
      return false;
    }
  }
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  while(1) {
    string urlbuf;
    getline(infile, urlbuf, '\n'); //mod
    //infile >> urlbuf;//mod
    //while(!urlbuf.empty()) {
    string url = URLcontainer::Find_URL(urlbuf); //mod
    //string url = urlbuf;//mod
      // parse URL
      URLcontainer urlcon;
      if(urlcon.Parse_URL(url) && !item_manager->search_by_url(urlcon.ret_URL())) {
	Options options = console_item->ret_Options_opt();

	ItemCell *itemcell;
	if(urlcon.ret_Protocol() == "http:") {
	  itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
	} else if(urlcon.ret_Protocol() == "ftp:") {
	  itemcell = new ItemCell_FTP(url, urlcon, options, "created");
	} else {
	  continue;
	}

	char *clist_item[TOTALCOL];
	/*
	//modified start
	for(int i = 0; i < TOTALCOL; ++i) {
	  clist_item[i] = NULL;
	}

	int rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
	gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
	item_manager->regist_item_back(itemcell);
	//modified end
	ItemStatusDynamic itemstatus(itemcell,
				     itemcell->ret_Status(),
				     0,
				     0);
	itemstatus.Update();
*/
	clist_item[COL_ICON] = NULL;
	if(itemcell->ret_Filename().empty()) {
	  clist_item[COL_FILENAME] = g_strdup(_("<directory>"));
	} else {
	  clist_item[COL_FILENAME] = g_strdup(itemcell->ret_Filename().c_str());
	}
	clist_item[COL_CURSIZE] = "0";
	clist_item[COL_TOTSIZE] = _("unknown");
	clist_item[COL_PROGRESS] = NULL;
	clist_item[COL_SPEED] = "";
	clist_item[COL_RTIME] = "";
	clist_item[COL_RETRY] = g_strdup_printf("0/%d", itemcell->ret_Options().ret_Retry());

	int n_rec;
	if(urlcon.ret_Protocol() == "http:") {
	  n_rec = itemcell->ret_Options().ret_recurse_count();
	} else {
	  n_rec = itemcell->ret_Options().ret_FTP_recurse_count();
	}
	clist_item[COL_REC] = g_strdup_printf("%d", n_rec);
	clist_item[COL_STATUS] = _("Ready");
	clist_item[COL_CRC] = "";
	clist_item[COL_SAVE] = g_strdup(itemcell->ret_Options().ret_Store_Dir().c_str());
	clist_item[COL_URL] = g_strdup(itemcell->ret_URL().c_str());
	int rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
	//GdkColor color = {0, 0, 0};
	//gtk_clist_set_foreground(GTK_CLIST(itemlistwidget), rowindex, &color);
	g_free(clist_item[COL_FILENAME]);
	g_free(clist_item[COL_RETRY]);
	g_free(clist_item[COL_REC]);
	g_free(clist_item[COL_SAVE]);
	g_free(clist_item[COL_URL]);
	Set_clist_column__progress(rowindex, 0);
	Set_clist_column__icon(rowindex, ItemCell::ITEM_READY);
	gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
	item_manager->regist_item_back(itemcell);
      }
      //}
    if(infile.eof()) break;
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  return true;
}

bool ItemList::Restore_default_item_option_sub(ifstream& infile)
{
  map<string, pair<string, int> > default_map;
  while(1) {
    string item;
    string line;
    getline(infile, line, '\n');
    item = Token_splitter(line, " \t");
    if(item == "%Begin_Items_Section" || infile.eof()) {
      unsigned int use_authentication = stoi(default_map[FI_USE_AUTHENTICATION].first);
      Userdata user(default_map[FI_USER].first, default_map[FI_PASSWORD].first);
      string storedir = default_map[FI_STORE_DIRECTORY].first;
      Options::RefererType referer_type = (Options::RefererType)stoi(default_map[FI_REFERER_TYPE].first);
      string http_version = default_map[FI_HTTP_VERSION].first;
      string referer_string = default_map[FI_REFERER_STRING].first;
      string useragent = default_map[FI_USERAGENT].first;
      unsigned int random_useragent = stoi(default_map[FI_RANDOM_USERAGENT].first);
      unsigned int cookie_delete_on_restart = stoi(default_map[FI_COOKIE_DELETE_ON_RESTART].first);
      unsigned int cookie_nosend = stoi(default_map[FI_COOKIE_NOSEND].first);
      int use_proxy = stoi(default_map[FI_USE_HTTP_PROXY].first);
      int use_http_cache = stoi(default_map[FI_USE_HTTP_CACHE].first);
      unsigned int use_http_proxy_authentication = stoi(default_map[FI_USE_HTTP_PROXY_AUTHENTICATION].first);      
      Userdata http_proxy_user(default_map[FI_HTTP_PROXY_USER].first,
			       default_map[FI_HTTP_PROXY_PASSWORD].first);
      Proxyserver proxyserver(default_map[FI_HTTP_PROXY_SERVER].first, stoi(default_map[FI_HTTP_PROXY_PORT].first));
      int timedout = stoi(default_map[FI_TIMED_OUT].first);
      int retry = stoi(default_map[FI_RETRY].first);
      int retry_repeat = stoi(default_map[FI_RETRY_REPEAT].first);
      bool use_retry_404 = (bool)stoi(default_map[FI_FORCE_RETRY_404].first);
      bool use_retry_503 = (bool)stoi(default_map[FI_FORCE_RETRY_503].first);
      int divide = stoi(default_map[FI_HOW_MANY_PARTS].first);
      int rollback_bytes = stoi(default_map[FI_ROLLBACK_BYTES].first);
      int recurse_count = stoi(default_map[FI_RECURSE_COUNT].first);

      list<string> filter_down_target_list;
      list<string> activated_filter_down_target_list;
      if(default_map[FI_FILTER_DOWN_TARGET_LIST].second == 1) {
	parse_extention_target_list(filter_down_target_list, default_map[FI_FILTER_DOWN_TARGET_LIST].first);
	parse_extention_target_list(activated_filter_down_target_list, default_map[FI_ACTIVATED_FILTER_DOWN_TARGET_LIST].first);
      }
      bool use_down_filter = (bool)stoi(default_map[FI_USE_DOWN_FILTER].first, 10);

      list<string> filter_nodown_target_list;
      list<string> activated_filter_nodown_target_list;
      if(default_map[FI_FILTER_NODOWN_TARGET_LIST].second == 1) {
	parse_extention_target_list(filter_nodown_target_list, default_map[FI_FILTER_NODOWN_TARGET_LIST].first);
	parse_extention_target_list(activated_filter_nodown_target_list, default_map[FI_ACTIVATED_FILTER_NODOWN_TARGET_LIST].first);
      } else {
	create_default_filter_nodown_target_list(filter_nodown_target_list);
	activated_filter_nodown_target_list = filter_nodown_target_list;
      }

      list<string> parse_target_list;
      list<string> activated_parse_target_list;
      if(default_map[FI_PARSE_TARGET_LIST].second == 1) {
	parse_extention_target_list(parse_target_list, default_map[FI_PARSE_TARGET_LIST].first);
	parse_extention_target_list(activated_parse_target_list, default_map[FI_ACTIVATED_PARSE_TARGET_LIST].first);
      } else {
	create_default_parse_target_list(parse_target_list);
	activated_parse_target_list = parse_target_list;
      }
      // various options for recursive download
      bool recurse_with_hostname_dir = true;
      if(default_map[FI_WITH_HOSTNAME_DIR].second == 1) {
	recurse_with_hostname_dir = (bool)stoi(default_map[FI_WITH_HOSTNAME_DIR].first);
      }
      bool recurse_abs2rel = (bool)stoi(default_map[FI_ABS2REL].first);
      bool recurse_force_convert = (bool)stoi(default_map[FI_FORCE_CONVERT].first);
      bool recurse_del_comment = (bool)stoi(default_map[FI_DEL_COMMENT].first);
      bool recurse_del_javascript = (bool)stoi(default_map[FI_DEL_JAVASCRIPT].first);
      bool recurse_no_other_host = (bool)stoi(default_map[FI_NO_OTHER_HOST].first);
      bool recurse_no_ascend = (bool)stoi(default_map[FI_NO_ASCEND].first);
      bool recurse_relative_only = (bool)stoi(default_map[FI_ONLY_RELATIVE_LINKS].first);
      bool recurse_referer_override = (bool)stoi(default_map[FI_REFERER_OVERRIDE].first);
      bool recurse_follow_ftp_link = (bool)stoi(default_map[FI_FOLLOW_FTP_LINK].first);
      bool recurse_convert_tilde = (bool)stoi(default_map[FI_CONVERT_TILDE].first);
      bool use_tag_href = true;
      if(default_map[FI_USE_TAG_HREF].second == 1) {
	use_tag_href = (bool)stoi(default_map[FI_USE_TAG_HREF].first);
      }
      bool use_tag_src = true;
      if(default_map[FI_USE_TAG_SRC].second == 1) {
	use_tag_src = (bool)stoi(default_map[FI_USE_TAG_SRC].first);
      }
      bool use_tag_background = true;
      if(default_map[FI_USE_TAG_BACKGROUND].second == 1) {
	use_tag_background = (bool)stoi(default_map[FI_USE_TAG_BACKGROUND].first);
      }
      bool use_tag_code = true;
      if(default_map[FI_USE_TAG_CODE].second == 1) {
	use_tag_code = (bool)stoi(default_map[FI_USE_TAG_CODE].first);
      }

      list<string> ign_domain_list, activated_ign_domain_list;
      parse_extention_target_list(ign_domain_list, default_map[FI_IGN_DOMAIN_LIST].first);
      parse_extention_target_list(activated_ign_domain_list, default_map[FI_ACTIVATED_IGN_DOMAIN_LIST].first);

      Options::DownloadMethodType downm_type = (Options::DownloadMethodType)stoi(default_map[FI_DOWNM_TYPE].first);
      //int speed_limit = stoi(default_map[FI_SPEED_LIMIT].first);
      //cerr << default_map[FI_SPEED_LIMIT].first.c_str() << endl;
      float speed_limit = strtod(default_map[FI_SPEED_LIMIT].first.c_str(), NULL);
      //cerr << "in file" << speed_limit << endl;
      Options::FTP_Mode ftp_mode = (Options::FTP_Mode)stoi(default_map[FI_FTP_MODE].first);
      Options::FTPretModeType ftp_ret_mode = (Options::FTPretModeType)stoi(default_map[FI_FTP_RET_MODE].first);
      bool ftp_nosend_quit = (bool)stoi(default_map[FI_FTP_NOSEND_QUIT].first);
      int ftp_recurse_count = 1;
      if(default_map[FI_FTP_RECURSE_COUNT].second == 1) {
	ftp_recurse_count = stoi(default_map[FI_FTP_RECURSE_COUNT].first);
      }
      bool ftp_use_filter = (bool)stoi(default_map[FI_FTP_USE_FILTER].first);
      bool ftp_allow_crawl_subdir = (bool)stoi(default_map[FI_FTP_ALLOW_CRAWL_SUBDIR].first);
      bool ftp_no_ascend = true;
      if(default_map[FI_FTP_NO_ASCEND].second == 1) {
	ftp_no_ascend = (bool)stoi(default_map[FI_FTP_NO_ASCEND].first);
      }
      bool ftp_get_symlink_as_realfile = DEFAULT_FTP_GET_SYMLINK_AS_REALFILE;
      if(default_map[FI_FTP_GET_SYMLINK_AS_REALFILE].second == 1) {
	ftp_get_symlink_as_realfile = (bool)stoi(default_map[FI_FTP_GET_SYMLINK_AS_REALFILE].first);
      }
      list<string> ftp_filter_target_list, ftp_activated_filter_target_list;
      parse_extention_target_list(ftp_filter_target_list, default_map[FI_FTP_FILTER_TARGET_LIST].first);
      parse_extention_target_list(ftp_activated_filter_target_list, default_map[FI_FTP_ACTIVATED_FILTER_TARGET_LIST].first);

      bool delete_when_finish = (bool)stoi(default_map[FI_DELETE_WHEN_COMPLETED].first, 10);
      bool dont_delete_without_crc = (bool)stoi(default_map[FI_DONT_DELETE_WITHOUT_CRC].first, 10);
      bool no_crc_check = (bool)stoi(default_map[FI_NO_CRC_CHECKING].first);
      bool ignore_crc_error = (bool)stoi(default_map[FI_IGNORE_CRC_ERROR].first);
      // set default item settings
      console_item->ret_Options_opt().Change_Values((bool)use_authentication,
						    user,
						    storedir,
						    http_version,
						    referer_type,
						    referer_string,
						    useragent,
						    (bool)random_useragent,
						    (bool)use_proxy,
						    (bool)use_http_cache,
						    use_http_proxy_authentication,
						    http_proxy_user,
						    proxyserver,
						    timedout,
						    divide,
						    rollback_bytes,
						    recurse_count,
						    recurse_with_hostname_dir,
						    recurse_abs2rel,
						    recurse_force_convert,
						    recurse_del_comment,
						    recurse_del_javascript,
						    recurse_no_other_host,
						    recurse_no_ascend,
						    recurse_relative_only,
						    recurse_referer_override,
						    recurse_follow_ftp_link,
						    recurse_convert_tilde,
						    use_tag_href,
						    use_tag_src,
						    use_tag_background,
						    use_tag_code,
						    use_down_filter,
						    filter_down_target_list,
						    filter_nodown_target_list,
						    parse_target_list,
						    ign_domain_list,
						    ftp_mode,
						    ftp_ret_mode,
						    ftp_nosend_quit,
						    ftp_recurse_count,
						    ftp_use_filter,
						    ftp_allow_crawl_subdir,
						    ftp_no_ascend,
						    ftp_get_symlink_as_realfile,
						    ftp_filter_target_list,
						    delete_when_finish,
						    retry,
						    retry_repeat,
						    use_retry_404,
						    use_retry_503,
						    dont_delete_without_crc,
						    no_crc_check,
						    ignore_crc_error,
						    (bool)cookie_delete_on_restart,
						    (bool)cookie_nosend,
						    downm_type,
						    speed_limit);

      console_item->ret_Options_opt().activate_filter_down_target_list(activated_filter_down_target_list);
      console_item->ret_Options_opt().activate_filter_nodown_target_list(activated_filter_nodown_target_list);
      console_item->ret_Options_opt().activate_parse_target_list(activated_parse_target_list);
      console_item->ret_Options_opt().activate_ign_domain_list(activated_ign_domain_list);
      console_item->ret_Options_opt().set_FTP_activated_filter_target_list(ftp_activated_filter_target_list);
      break;
    } else {
      default_map[item].first = Remove_white(line);
      default_map[item].second = 1;
    }
  }
  /*
  struct stat dir_stat;
  if(stat(console_item->ret_Options_opt().ret_Store_Dir().c_str(), &dir_stat) == -1 || !S_ISDIR(dir_stat.st_mode)) {
    string line = _("directory '")+console_item->ret_Options_opt().ret_Store_Dir()+_("' does not exist. Use your home directory instead");
    console_item->Send_message_to_gui(line, MSG_SYS_ERROR);
    char *home_dir = g_get_home_dir();
    console_item->ret_Options_opt().set_Store_Dir(home_dir);
  }
  */
  return true;
}

bool ItemList::Restore_saved_list()
{
  return Restore_saved_list(file_default_item_list);
}

bool ItemList::Restore_saved_list(const string& filename)
{
  ifstream infile;
  char *clist_item[TOTALCOL];
  unsigned int rowindex;

  infile.open(filename.c_str(), ios::skipws|ios::in);
  if(infile.bad()) return(false);
  // read magic number
  string header;
  getline(infile, header, '\n');
  if(header != ARIA_VERSION) {
    console_item->Send_message_to_gui(_("warning:different version number found in file header"), MSG_SYS_ERROR);
    //return false;
  }
  // restore default saved list
  //Restore_default_item_option_sub(infile);
  // restore item list
  string line;
  getline(infile, line, '\n');
  //item = Token_splitter(line, " \t");
  if(line != "%Begin_Items_Section") return false;
  gtk_clist_freeze(GTK_CLIST(itemlistwidget));
  while(1) {
    string item;
    getline(infile, line, '\n');
    if(line == "%End_Items_Section") break;
    map<string, string> item_map;
    while(1) {
      getline(infile, line, '\n');
      item = Token_splitter(line, " \t");
      
      if(item == "%End_Item") {
	string url = item_map[FI_URL];
	ItemCell::ItemStatusType status = (ItemCell::ItemStatusType)stoi(item_map[FI_STATUS], 10);
	int size_current = stoi(item_map[FI_SIZE_CURRENT]);
	int size_total = stoi(item_map[FI_SIZE_TOTAL]);
	ItemCell::CRC_Type crc_type = (ItemCell::CRC_Type)stoi(item_map[FI_CRC_TYPE], 10);
	unsigned int crc = stoui(item_map[FI_CRC], 16);
	
	int use_authentication = stoi(item_map[FI_USE_AUTHENTICATION]);
	Userdata user(item_map[FI_USER], item_map[FI_PASSWORD]);
	string storedir = item_map[FI_STORE_DIRECTORY];
	Options::RefererType referer_type = (Options::RefererType)stoi(item_map[FI_REFERER_TYPE]);
	string referer_string = item_map[FI_REFERER_STRING];
	string http_version = item_map[FI_HTTP_VERSION];
	string useragent = item_map[FI_USERAGENT];
	unsigned int random_useragent = stoi(item_map[FI_RANDOM_USERAGENT]);
	unsigned int cookie_delete_on_restart = stoi(item_map[FI_COOKIE_DELETE_ON_RESTART]);
	unsigned int cookie_nosend = stoi(item_map[FI_COOKIE_NOSEND]);
	int use_proxy = stoi(item_map[FI_USE_HTTP_PROXY]);
	int use_http_cache = stoi(item_map[FI_USE_HTTP_CACHE]);
	int use_http_proxy_authentication = stoi(item_map[FI_USE_HTTP_PROXY_AUTHENTICATION]);
	Userdata http_proxy_user(item_map[FI_HTTP_PROXY_USER],
				 item_map[FI_HTTP_PROXY_PASSWORD]);
	Proxyserver proxyserver(item_map[FI_HTTP_PROXY_SERVER], stoi(item_map[FI_HTTP_PROXY_PORT]));

	int timedout = stoi(item_map[FI_TIMED_OUT]);
	int retry = stoi(item_map[FI_RETRY]);
	int retry_repeat = stoi(item_map[FI_RETRY_REPEAT]);
	bool use_retry_404 = (bool)stoi(item_map[FI_FORCE_RETRY_404]);
	bool use_retry_503 = (bool)stoi(item_map[FI_FORCE_RETRY_503]);
	int divide = stoi(item_map[FI_HOW_MANY_PARTS]);
	int rollback_bytes = stoi(item_map[FI_ROLLBACK_BYTES]);
	int recurse_count = stoi(item_map[FI_RECURSE_COUNT]);
	string documentroot_dir = item_map[FI_DOCUMENTROOT_DIR];
	string root_url = item_map[FI_ROOT_URL];

	bool use_down_filter = (bool)stoi(item_map[FI_USE_DOWN_FILTER]);
	list<string> filter_down_target_list;
	parse_extention_target_list(filter_down_target_list, item_map[FI_FILTER_DOWN_TARGET_LIST]);
	list<string> activated_filter_down_target_list;
	parse_extention_target_list(activated_filter_down_target_list, item_map[FI_ACTIVATED_FILTER_DOWN_TARGET_LIST]);

	list<string> filter_nodown_target_list;
	parse_extention_target_list(filter_nodown_target_list, item_map[FI_FILTER_NODOWN_TARGET_LIST]);
	list<string> activated_filter_nodown_target_list;
	parse_extention_target_list(activated_filter_nodown_target_list, item_map[FI_ACTIVATED_FILTER_NODOWN_TARGET_LIST]);
	list<string> parse_target_list;
	parse_extention_target_list(parse_target_list, item_map[FI_PARSE_TARGET_LIST]);
	list<string> activated_parse_target_list;
	parse_extention_target_list(activated_parse_target_list, item_map[FI_ACTIVATED_PARSE_TARGET_LIST]);
	
	Options::DownloadMethodType downm_type = (Options::DownloadMethodType)stoi(item_map[FI_DOWNM_TYPE]);
	//int speed_limit = stoi(item_map[FI_SPEED_LIMIT]);
	float speed_limit = strtod(item_map[FI_SPEED_LIMIT].c_str(), NULL);
	//cerr << "speed " << speed_limit << endl;
	// various options for recursive download
	bool recurse_with_hostname_dir = (bool)stoi(item_map[FI_WITH_HOSTNAME_DIR]);
	bool recurse_abs2rel = (bool)stoi(item_map[FI_ABS2REL]);
	bool recurse_force_convert = (bool)stoi(item_map[FI_FORCE_CONVERT]);
	bool recurse_del_comment = (bool)stoi(item_map[FI_DEL_COMMENT]);
	bool recurse_del_javascript = (bool)stoi(item_map[FI_DEL_JAVASCRIPT]);
	bool recurse_no_other_host = (bool)stoi(item_map[FI_NO_OTHER_HOST]);
	bool recurse_no_ascend = (bool)stoi(item_map[FI_NO_ASCEND]);
	bool recurse_relative_only = (bool)stoi(item_map[FI_ONLY_RELATIVE_LINKS]);
	bool recurse_referer_override = (bool)stoi(item_map[FI_REFERER_OVERRIDE]);
	bool recurse_follow_ftp_link = (bool)stoi(item_map[FI_FOLLOW_FTP_LINK]);
	bool recurse_convert_tilde = (bool)stoi(item_map[FI_CONVERT_TILDE]);
	bool use_tag_href = (bool)stoi(item_map[FI_USE_TAG_HREF]);
	bool use_tag_src = (bool)stoi(item_map[FI_USE_TAG_SRC]);
	bool use_tag_background = (bool)stoi(item_map[FI_USE_TAG_BACKGROUND]);
	bool use_tag_code = (bool)stoi(item_map[FI_USE_TAG_CODE]);

	list<string> ign_domain_list, activated_ign_domain_list;
	parse_extention_target_list(ign_domain_list, item_map[FI_IGN_DOMAIN_LIST]);
	parse_extention_target_list(activated_ign_domain_list, item_map[FI_ACTIVATED_IGN_DOMAIN_LIST]);

	Options::FTP_Mode ftp_mode = (Options::FTP_Mode)stoi(item_map[FI_FTP_MODE]);
	Options::FTPretModeType ftp_ret_mode = (Options::FTPretModeType)stoi(item_map[FI_FTP_RET_MODE]);
	bool ftp_nosend_quit = (bool)stoi(item_map[FI_FTP_NOSEND_QUIT]);

	int ftp_recurse_count = stoi(item_map[FI_FTP_RECURSE_COUNT]);

	bool ftp_use_filter = (bool)stoi(item_map[FI_FTP_USE_FILTER]);
	bool ftp_allow_crawl_subdir = (bool)stoi(item_map[FI_FTP_ALLOW_CRAWL_SUBDIR]);
	bool ftp_no_ascend = (bool)stoi(item_map[FI_FTP_NO_ASCEND]);
	
	bool ftp_get_symlink_as_realfile = (bool)stoi(item_map[FI_FTP_GET_SYMLINK_AS_REALFILE]);
	list<string> ftp_filter_target_list, ftp_activated_filter_target_list;
	parse_extention_target_list(ftp_filter_target_list, item_map[FI_FTP_FILTER_TARGET_LIST]);
	parse_extention_target_list(ftp_activated_filter_target_list, item_map[FI_FTP_ACTIVATED_FILTER_TARGET_LIST]);

	bool delete_when_finish = (bool)stoi(item_map[FI_DELETE_WHEN_COMPLETED]);
	bool dont_delete_without_crc = (bool)stoi(item_map[FI_DONT_DELETE_WITHOUT_CRC]);
	bool no_crc_check = (bool)stoi(item_map[FI_NO_CRC_CHECKING]);
	bool ignore_crc_error = (bool)stoi(item_map[FI_IGNORE_CRC_ERROR]);
	// create URLcontainer
	URLcontainer urlcon;
	urlcon.Parse_URL(url);
	// create Options
	Options options((bool)use_authentication,
			user,
			storedir,
			http_version,
			referer_type,
			referer_string,
			useragent,
			(bool)random_useragent,
			(bool)use_proxy,
			(bool)use_http_cache,
			use_http_proxy_authentication,
			http_proxy_user,
			proxyserver,
			timedout,
			divide,
			rollback_bytes,
			recurse_count,
			recurse_with_hostname_dir,
			recurse_abs2rel,
			recurse_force_convert,
			recurse_del_comment,
			recurse_del_javascript,
			recurse_no_other_host,
			recurse_no_ascend,
			recurse_relative_only,
			recurse_referer_override,
			recurse_follow_ftp_link,
			recurse_convert_tilde,
			use_tag_href,
			use_tag_src,
			use_tag_background,
			use_tag_code,
			use_down_filter,
			filter_down_target_list,
			filter_nodown_target_list,
			parse_target_list,
			ign_domain_list,
			ftp_mode,
			ftp_ret_mode,
			ftp_nosend_quit,
			ftp_recurse_count,
			ftp_use_filter,
			ftp_allow_crawl_subdir,
			ftp_no_ascend,
			ftp_get_symlink_as_realfile,
			ftp_filter_target_list,
			delete_when_finish,
			retry,
			retry_repeat,
			use_retry_404,
			use_retry_503,
			dont_delete_without_crc,
			no_crc_check,
			ignore_crc_error,
			(bool)cookie_delete_on_restart,
			(bool)cookie_nosend,
			downm_type,
			speed_limit
			);
	options.activate_filter_down_target_list(activated_filter_down_target_list);
	options.activate_filter_nodown_target_list(activated_filter_nodown_target_list);
	options.activate_parse_target_list(activated_parse_target_list);
	options.activate_ign_domain_list(activated_ign_domain_list);
	options.set_FTP_activated_filter_target_list(ftp_activated_filter_target_list);
	// create ItemCell
	ItemCell *itemcell;
	if(urlcon.ret_Protocol() == "http:") {
	  itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
	} else if(urlcon.ret_Protocol() == "ftp:") {
	  itemcell = new ItemCell_FTP(url, urlcon, options, "created");
	} else {
	  continue;
	}
	itemcell->set_documentroot_dir(documentroot_dir);
	itemcell->set_root_url(root_url);
	itemcell->set_Status(status);
	itemcell->set_Size_Current(size_current);
	itemcell->set_Size_Total(size_total);
	itemcell->set_CRC_Type(crc_type);
	itemcell->set_CRC(crc);
	/*
	//modified start
	for(int i = 0; i < TOTALCOL; ++i) {
	  clist_item[i] = NULL;
	}
	string crc_string;
	switch(itemcell->ret_CRC_Type()) {
	case CRC_16:
	  crc_string = itos_hex(itemcell->ret_CRC(), 4, '0');
	  break;
	case CRC_32:
	  crc_string = itos_hex(itemcell->ret_CRC(), 8, '0');
	  break;
	default:
	  break;
	}
	clist_item[COL_CRC] = g_strdup(crc_string.c_str());

	rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
	gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
	item_manager->regist_item_back(itemcell);
	//modified end
	ItemStatusDynamic itemstatus(itemcell,
				     itemcell->ret_Status(),
				     size_current,
				     size_total);
	itemstatus.Update();
	delete [] clist_item[COL_CRC];
	*/
	clist_item[COL_ICON] = NULL;
	if(itemcell->ret_Filename().empty()) {
	  clist_item[COL_FILENAME] = g_strdup(_("<directory>"));
	} else {
	  clist_item[COL_FILENAME] = g_strdup(itemcell->ret_Filename().c_str());
	}
	clist_item[COL_SPEED] = "";
	clist_item[COL_RTIME] = "";
	clist_item[COL_CRC] = g_strdup(item_map[FI_CRC].c_str());
	/*
	switch(itemcell->ret_Status()) {
	case ITEM_COMPLETE:
	  clist_item[COL_STATUS] = _("DOWNLOAD COMPLETE");
	  break;
	case ITEM_STOP:
	  clist_item[COL_STATUS] = _("STOPPED");
	} else {
	  clist_item[COL_STATUS] = _("Ready");
	}
	*/
	clist_item[COL_STATUS] = "";
	int progress;
	string cursize;
	string totsize;
	if(itemcell->ret_Size_Total() > 0) {
	  if(appoption->ret_use_size_human_readable()) {
	    cursize = get_human_readable_size(size_current);
	    totsize = get_human_readable_size(size_total);
	  } else {
	    cursize = itos(itemcell->ret_Size_Current(), true);
	    totsize = itos(itemcell->ret_Size_Total(), true);
	  }
	  progress = (int)((float)itemcell->ret_Size_Current()/itemcell->ret_Size_Total()*100);
	} else {
	  if(appoption->ret_use_size_human_readable()) {
	    cursize = get_human_readable_size(size_current);
	  } else {
	    cursize = itos(size_current);
	  }
	  totsize = _("unknown");
	  progress = 0;
	}
	clist_item[COL_CURSIZE] = g_strdup(cursize.c_str());
	clist_item[COL_TOTSIZE] = g_strdup(totsize.c_str());
	clist_item[COL_PROGRESS] = NULL;//g_strdup(progress.c_str());

	clist_item[COL_RETRY] = g_strdup_printf("0/%d", itemcell->ret_Options().ret_Retry());
	int n_rec;
	if(urlcon.ret_Protocol() == "http:") {
	  n_rec = itemcell->ret_Options().ret_recurse_count();
	} else {
	  n_rec = itemcell->ret_Options().ret_FTP_recurse_count();
	}

	clist_item[COL_REC] = g_strdup_printf("%d", n_rec);

	clist_item[COL_SAVE] = g_strdup(itemcell->ret_Options().ret_Store_Dir().c_str());
	clist_item[COL_URL] = g_strdup(itemcell->ret_URL().c_str());
	rowindex = gtk_clist_append(GTK_CLIST(itemlistwidget), clist_item);
	//GdkColor color = { 0, 0, 0};
	//gtk_clist_set_foreground(GTK_CLIST(itemlistwidget), rowindex, &color);
	/*
	if(itemcell->ret_Status() == ITEM_COMPLETE) {
	  Set_clist_column_style__complete(rowindex);
	} else {
	  Set_clist_column_style__ready(rowindex);
	}
	*/
	g_free(clist_item[COL_FILENAME]);
	g_free(clist_item[COL_CRC]);
	g_free(clist_item[COL_CURSIZE]);
	g_free(clist_item[COL_TOTSIZE]);
	//delete [] clist_item[COL_PROGRESS];
	g_free(clist_item[COL_RETRY]);
	g_free(clist_item[COL_REC]);
	g_free(clist_item[COL_SAVE]);
	g_free(clist_item[COL_URL]);
	Set_clist_column__icon(rowindex, itemcell->ret_Status());
	Set_clist_column__progress(rowindex, progress);
	gtk_clist_set_row_data(GTK_CLIST(itemlistwidget), rowindex, itemcell);
	item_manager->regist_item_back(itemcell);
	break;
      } else {
	item_map[item] = Remove_white(line);
      }
    }
  }
  gtk_clist_thaw(GTK_CLIST(itemlistwidget));

  return true;
}

bool ItemList::Save_current_list()
{
  return Save_current_list(file_default_item_list);
}

bool ItemList::Save_default_item_option_sub(ofstream& outfile)
{
  const Options& options = console_item->ret_Options_opt();

  // save default item options
  outfile << FI_USE_AUTHENTICATION << " " << options.Whether_use_authentication() << '\n';
  if(!options.ret_User().empty()) {
    outfile << FI_USER << " " << options.ret_User() << '\n';
  }
  if(!options.ret_Password().empty()) {
    outfile << FI_PASSWORD << " " << options.ret_Password() << '\n';
  }
  outfile << FI_STORE_DIRECTORY << " " << options.ret_Store_Dir() << '\n';
  outfile << FI_HTTP_VERSION << " " << options.ret_HTTP_version() << '\n';
  outfile << FI_REFERER_TYPE << " " << dec << (unsigned int)options.ret_Referer_Type() << '\n';
  if(!options.ret_Referer().empty()) {
    outfile << FI_REFERER_STRING << " " << options.ret_Referer() << '\n';
  }
  // Useragent
  if(!options.ret_Useragent().empty()) {
    outfile << FI_USERAGENT << " " << options.ret_Useragent() << '\n';
  }
  outfile << FI_RANDOM_USERAGENT << " " << options.ret_Random_useragent() << '\n';
  // Cookie
  outfile << FI_COOKIE_DELETE_ON_RESTART << " " << options.ret_Cookie_delete_on_restart() << '\n';
  outfile << FI_COOKIE_NOSEND << " " << options.ret_Cookie_nosend() << '\n';
  // HTTP Proxy
  outfile << FI_USE_HTTP_PROXY_AUTHENTICATION << " " << options.ret_use_http_proxy_authentication() << '\n';
  if(!options.ret_http_proxy_User().empty()) {
    outfile << FI_HTTP_PROXY_USER << " " << options.ret_http_proxy_User() << '\n';
  }
  if(!options.ret_http_proxy_Password().empty()) {
    outfile << FI_HTTP_PROXY_PASSWORD << " " << options.ret_http_proxy_Password() << '\n';
  }
  outfile << FI_USE_HTTP_PROXY << " " << dec << options.ret_use_http_proxy() << '\n';
  outfile << FI_USE_HTTP_CACHE << " " << dec << options.ret_use_http_cache() << '\n';
  if(!options.ret_http_proxy().ret_Server().empty()) {
    outfile << FI_HTTP_PROXY_SERVER << " " << options.ret_http_proxy().ret_Server() << '\n';
    outfile << FI_HTTP_PROXY_PORT << " " << dec << options.ret_http_proxy().ret_Port() << '\n';
  }
  outfile << FI_DOWNM_TYPE << " " << dec << options.ret_downm_type() << '\n';
  outfile << FI_SPEED_LIMIT << " " << dec << options.ret_speed_limit() << '\n';
  // Time out
  outfile << FI_TIMED_OUT << " " << dec << options.ret_Timed_Out() << '\n';

  // Divide
  outfile << FI_HOW_MANY_PARTS << " " << dec << options.ret_Divide() << '\n';
  // Rollback in bytes
  outfile << FI_ROLLBACK_BYTES << " " << dec << options.ret_Rollback_bytes() << '\n';

  // FTP
  outfile << FI_FTP_MODE << " " << dec << (unsigned int)options.ret_FTP_Mode() << '\n';
  outfile << FI_FTP_RET_MODE << " " << dec << (unsigned int)options.ret_FTP_ret_mode() << '\n';
  outfile << FI_FTP_NOSEND_QUIT << " " << dec << (unsigned int)options.ret_FTP_nosend_quit() << '\n';
  outfile << FI_FTP_RECURSE_COUNT << " " << dec << (unsigned int)options.ret_FTP_recurse_count() << '\n';
  outfile << FI_FTP_NO_ASCEND << " " << dec << (unsigned int)options.ret_FTP_no_ascend() << '\n';
  outfile << FI_FTP_ALLOW_CRAWL_SUBDIR << " " << dec << (unsigned int)options.ret_FTP_allow_crawl_subdir() << '\n';
  outfile << FI_FTP_GET_SYMLINK_AS_REALFILE << " " << dec << (unsigned int)options.ret_FTP_get_symlink_as_realfile() << '\n';
  outfile << FI_FTP_USE_FILTER << " " << dec << (unsigned int)options.ret_FTP_use_filter() << '\n';
  outfile << FI_FTP_FILTER_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_FTP_filter_target_list().begin(); itr != options.ret_FTP_filter_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_FTP_ACTIVATED_FILTER_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_FTP_activated_filter_target_list().begin(); itr != options.ret_FTP_activated_filter_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';

  outfile << FI_RETRY << " " << dec << options.ret_Retry() << '\n';
  outfile << FI_RETRY_REPEAT << " " << dec << options.ret_Retry_repeat() << '\n';
  outfile << FI_FORCE_RETRY_404 << " " << dec << options.ret_force_retry_404() << '\n';
  outfile << FI_FORCE_RETRY_503 << " " << dec << options.ret_force_retry_503() << '\n';
  outfile << FI_DELETE_WHEN_COMPLETED << " " << dec << (unsigned int)options.ret_Delete_When_Finish() << '\n';
  outfile << FI_DONT_DELETE_WITHOUT_CRC << " " << dec << (unsigned int)options.ret_Dont_Delete_Without_CRC() << '\n';
  outfile << FI_NO_CRC_CHECKING << " " << dec << (unsigned int)options.ret_no_crc_checking() << '\n';
  outfile << FI_IGNORE_CRC_ERROR << " " << dec << (unsigned int)options.ret_ignore_crc_error() << '\n';

  // recursive download
  outfile << FI_RECURSE_COUNT << " " << dec << options.ret_recurse_count() << '\n';
  //outfile << FI_DOCUMENTROOT_DIR << " " << itemcell->ret_documentroot_dir() << '\n';

  outfile << FI_WITH_HOSTNAME_DIR << " " << dec << (unsigned int)options.ret_with_hostname_dir() << '\n';
  outfile << FI_ABS2REL << " " << dec << (unsigned int)options.ret_abs2rel_url() << '\n';
  outfile << FI_FORCE_CONVERT << " " << dec << (unsigned int)options.ret_force_convert() << '\n';
  outfile << FI_DEL_COMMENT << " " << dec << (unsigned int)options.ret_delete_comment() << '\n';
  outfile << FI_DEL_JAVASCRIPT << " " << dec << (unsigned int)options.ret_delete_javascript() << '\n';
  outfile << FI_NO_OTHER_HOST << " " << dec << (unsigned int)options.ret_no_other_host() << '\n';
  outfile << FI_NO_ASCEND << " " << dec << (unsigned int)options.ret_no_ascend() << '\n';
  outfile << FI_ONLY_RELATIVE_LINKS << " " << dec << (unsigned int)options.ret_only_relative_links() << '\n';
  outfile << FI_REFERER_OVERRIDE << " " << dec << (unsigned int)options.ret_Referer_override() << '\n';
  outfile << FI_FOLLOW_FTP_LINK << " " << dec << (unsigned int)options.ret_Follow_ftp_link() << '\n';
  outfile << FI_CONVERT_TILDE << " " << dec << (unsigned int)options.ret_convert_tilde() << '\n';

  outfile << FI_USE_TAG_HREF << " " << dec << (unsigned int)options.ret_use_tag_href() << '\n';
  outfile << FI_USE_TAG_SRC << " " << dec << (unsigned int)options.ret_use_tag_src() << '\n';
  outfile << FI_USE_TAG_BACKGROUND << " " << dec << (unsigned int)options.ret_use_tag_background() << '\n';
  outfile << FI_USE_TAG_CODE << " " << dec << (unsigned int)options.ret_use_tag_code() << '\n';

  outfile << FI_IGN_DOMAIN_LIST;
  for(list<string>::const_iterator itr = options.ret_ign_domain_list().begin(); itr != options.ret_ign_domain_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_ACTIVATED_IGN_DOMAIN_LIST;
  for(list<string>::const_iterator itr = options.ret_activated_ign_domain_list().begin(); itr != options.ret_activated_ign_domain_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';

  outfile << FI_USE_DOWN_FILTER << " " << (unsigned int)options.ret_use_down_filter() << '\n';

  outfile << FI_FILTER_DOWN_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_filter_down_target_list().begin(); itr != options.ret_filter_down_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_ACTIVATED_FILTER_DOWN_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_activated_filter_down_target_list().begin(); itr != options.ret_activated_filter_down_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';

  outfile << FI_FILTER_NODOWN_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_filter_nodown_target_list().begin(); itr != options.ret_filter_nodown_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_ACTIVATED_FILTER_NODOWN_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_activated_filter_nodown_target_list().begin(); itr != options.ret_activated_filter_nodown_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_PARSE_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_parse_target_list().begin(); itr != options.ret_parse_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';
  outfile << FI_ACTIVATED_PARSE_TARGET_LIST;
  for(list<string>::const_iterator itr = options.ret_activated_parse_target_list().begin(); itr != options.ret_activated_parse_target_list().end(); ++itr) {
    outfile << " " << *itr;
  }
  outfile << '\n';

  return true;
}

bool ItemList::Save_current_list(const string& filename)
{
  ofstream list_out;

  list_out.open(filename.c_str(), ios::out, 0600);

  if(list_out.bad()) {
    cerr << "ERROR:Unable to save download list" << endl;
    return false;
  }
  // file header
  list_out << ARIA_VERSION << '\n';

  //Save_default_item_option_sub(list_out);

  // save items
  list_out << "%Begin_Items_Section" << '\n';
  const list<int>& download_list = item_manager->ret_id_list();
  for(list<int>::const_iterator id_itr = download_list.begin(); id_itr != download_list.end(); ++id_itr) {
    ItemCell *itemcell = item_manager->ret_itemaddr(*id_itr);
    if(itemcell->Is_Partial()) continue;
    const Options& options = itemcell->ret_Options_opt();
    list_out << "%Begin_Item" << '\n';
    //URL
    list_out << FI_URL << " " << itemcell->ret_URL() << '\n';
    //status
    list_out << FI_STATUS << " ";
    switch(itemcell->ret_Status()) {
    case ItemCell::ITEM_DOWNLOAD:
    case ItemCell::ITEM_INUSE:
    case ItemCell::ITEM_DOWNLOAD_AGAIN:
    case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
    case ItemCell::ITEM_INUSE_AGAIN:
    case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    case ItemCell::ITEM_READY_CONCAT:
    case ItemCell::ITEM_READY_AGAIN:
      list_out << dec << (unsigned int)ItemCell::ITEM_READY;
      break;
    default:
      list_out << dec << (unsigned int)itemcell->ret_Status();
      break;
    }
    list_out << '\n';

    //size
    list_out << FI_SIZE_CURRENT << " " << dec << itemcell->ret_Size_Current() << '\n';
    list_out << FI_SIZE_TOTAL << " " << dec << itemcell->ret_Size_Total() << '\n';
    //CRC
    switch(itemcell->ret_CRC_Type()) {
    case ItemCell::CRC_16:
      list_out << FI_CRC_TYPE << " " << (unsigned int)itemcell->ret_CRC_Type() << '\n';
      list_out << FI_CRC << " " << hex << setw(4) << setfill('0') << itemcell->ret_CRC() << '\n';
      break;
    case ItemCell::CRC_32:
      list_out << FI_CRC_TYPE << " " << (unsigned int)itemcell->ret_CRC_Type() << '\n';
      list_out << FI_CRC << " " << hex << setw(8) << setfill('0') << itemcell->ret_CRC() << '\n';
      break;
    default:
      list_out << FI_CRC_TYPE << " " << dec << (unsigned int)itemcell->ret_CRC_Type() << '\n';
      break;
    }
    //authentication
    list_out << FI_USE_AUTHENTICATION << " " << options.Whether_use_authentication() << '\n';
    if(!options.ret_User().empty()) {
      list_out << FI_USER << " " << options.ret_User() << '\n';
    }
    if(!itemcell->ret_Options_opt().ret_Password().empty()) {
      list_out << FI_PASSWORD << " " << options.ret_Password() << '\n';
    }
    list_out << FI_STORE_DIRECTORY << " " << options.ret_Store_Dir() << '\n';

    // HTTP version
    list_out << FI_HTTP_VERSION << " " << options.ret_HTTP_version() << '\n';
    // Referer
    list_out << FI_REFERER_TYPE << " " << dec << (unsigned int)options.ret_Referer_Type() << '\n';
    if(!options.ret_Referer().empty()) {
      list_out << FI_REFERER_STRING << " " << options.ret_Referer() << '\n';
    }

    // Useragent
    if(!options.ret_Useragent().empty()) {
      list_out << FI_USERAGENT << " " << options.ret_Useragent() << '\n';
    }
    list_out << FI_RANDOM_USERAGENT << " " << options.ret_Random_useragent() << '\n';

    // Cookie
    list_out << FI_COOKIE_DELETE_ON_RESTART << " " << options.ret_Cookie_delete_on_restart() << '\n';
    list_out << FI_COOKIE_NOSEND << " " << options.ret_Cookie_nosend() << '\n';

    // HTTP Proxy
    list_out << FI_USE_HTTP_PROXY_AUTHENTICATION << " " << options.ret_use_http_proxy_authentication() << '\n';
    if(!options.ret_http_proxy_User().empty()) {
      list_out << FI_HTTP_PROXY_USER << " " << options.ret_http_proxy_User() << '\n';
    }
    if(!options.ret_http_proxy_Password().empty()) {
      list_out << FI_HTTP_PROXY_PASSWORD << " " << options.ret_http_proxy_Password() << '\n';
    }

    list_out << FI_USE_HTTP_PROXY << " " << dec << options.ret_use_http_proxy() << '\n';
    list_out << FI_USE_HTTP_CACHE << " " << dec << options.ret_use_http_cache() << '\n';
    if(!options.ret_http_proxy().ret_Server().empty()) {
      list_out << FI_HTTP_PROXY_SERVER << " " << options.ret_http_proxy().ret_Server() << '\n';
      list_out << FI_HTTP_PROXY_PORT << " " << dec << options.ret_http_proxy().ret_Port() << '\n';
    }
    list_out << FI_DOWNM_TYPE << " " << dec << options.ret_downm_type() << '\n';
    list_out << FI_SPEED_LIMIT << " " << dec << options.ret_speed_limit() << '\n';

    //time out
    list_out << FI_TIMED_OUT << " " << dec << options.ret_Timed_Out() << '\n';
    //split
    list_out << FI_HOW_MANY_PARTS << " " << dec << options.ret_Divide() << '\n';
  // Rollback in bytes
    list_out << FI_ROLLBACK_BYTES << " " << dec << options.ret_Rollback_bytes() << '\n';

    // recursive download
    list_out << FI_RECURSE_COUNT << " " << dec << options.ret_recurse_count() << '\n';
    list_out << FI_DOCUMENTROOT_DIR << " " << itemcell->ret_documentroot_dir() << '\n';
    list_out << FI_ROOT_URL << " " << itemcell->ret_root_url() << '\n';
    list_out << FI_WITH_HOSTNAME_DIR << " " << dec << (unsigned int)options.ret_with_hostname_dir() << '\n';
    list_out << FI_ABS2REL << " " << dec << (unsigned int)options.ret_abs2rel_url() << '\n';
    list_out << FI_FORCE_CONVERT << " " << dec << (unsigned int)options.ret_force_convert() << '\n';
    list_out << FI_DEL_COMMENT << " " << dec << (unsigned int)options.ret_delete_comment() << '\n';
    list_out << FI_DEL_JAVASCRIPT << " " << dec << (unsigned int)options.ret_delete_javascript() << '\n';
    list_out << FI_NO_OTHER_HOST << " " << dec << (unsigned int)options.ret_no_other_host() << '\n';
    list_out << FI_NO_ASCEND << " " << dec << (unsigned int)options.ret_no_ascend() << '\n';
    list_out << FI_ONLY_RELATIVE_LINKS << " " << dec << (unsigned int)options.ret_only_relative_links() << '\n';
    list_out << FI_REFERER_OVERRIDE << " " << dec << (unsigned int)options.ret_Referer_override() << '\n';
    list_out << FI_FOLLOW_FTP_LINK << " " << dec << (unsigned int)options.ret_Follow_ftp_link() << '\n';
    list_out << FI_CONVERT_TILDE << " " << dec << (unsigned int)options.ret_convert_tilde() << '\n';

    list_out << FI_USE_TAG_HREF << " " << dec << (unsigned int)options.ret_use_tag_href() << '\n';
    list_out << FI_USE_TAG_SRC << " " << dec << (unsigned int)options.ret_use_tag_src() << '\n';
    list_out << FI_USE_TAG_BACKGROUND << " " << dec << (unsigned int)options.ret_use_tag_background() << '\n';
    list_out << FI_USE_TAG_CODE << " " << dec << (unsigned int)options.ret_use_tag_code() << '\n';

    list_out << FI_IGN_DOMAIN_LIST;
    for(list<string>::const_iterator itr = options.ret_ign_domain_list().begin(); itr != options.ret_ign_domain_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_ACTIVATED_IGN_DOMAIN_LIST;
    for(list<string>::const_iterator itr = options.ret_activated_ign_domain_list().begin(); itr != options.ret_activated_ign_domain_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';

    list_out << FI_USE_DOWN_FILTER << " " << dec << (unsigned int)options.ret_use_down_filter() << '\n';
    list_out << FI_FILTER_DOWN_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_filter_down_target_list().begin(); itr != options.ret_filter_down_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_ACTIVATED_FILTER_DOWN_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_activated_filter_down_target_list().begin(); itr != options.ret_activated_filter_down_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';

    list_out << FI_FILTER_NODOWN_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_filter_nodown_target_list().begin(); itr != options.ret_filter_nodown_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_ACTIVATED_FILTER_NODOWN_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_activated_filter_nodown_target_list().begin(); itr != options.ret_activated_filter_nodown_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_PARSE_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_parse_target_list().begin(); itr != options.ret_parse_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_ACTIVATED_PARSE_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_activated_parse_target_list().begin(); itr != options.ret_activated_parse_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    //FTP mode
    list_out << FI_FTP_MODE << " " << dec << (unsigned int)options.ret_FTP_Mode() << '\n';
    list_out << FI_FTP_RET_MODE << " " << dec << (unsigned int)options.ret_FTP_ret_mode() << '\n';
    list_out << FI_FTP_NOSEND_QUIT << " " << dec << (unsigned int)options.ret_FTP_nosend_quit() << '\n';
    list_out << FI_FTP_RECURSE_COUNT << " " << dec << (unsigned int)options.ret_FTP_recurse_count() << '\n';
    list_out << FI_FTP_NO_ASCEND << " " << dec << (unsigned int)options.ret_FTP_no_ascend() << '\n';
    list_out << FI_FTP_ALLOW_CRAWL_SUBDIR << " " << dec << (unsigned int)options.ret_FTP_allow_crawl_subdir() << '\n';
    list_out << FI_FTP_GET_SYMLINK_AS_REALFILE << " " << dec << (unsigned int)options.ret_FTP_get_symlink_as_realfile() << '\n';
    list_out << FI_FTP_USE_FILTER << " " << dec << (unsigned int)options.ret_FTP_use_filter() << '\n';
    list_out << FI_FTP_FILTER_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_FTP_filter_target_list().begin(); itr != options.ret_FTP_filter_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';
    list_out << FI_FTP_ACTIVATED_FILTER_TARGET_LIST;
    for(list<string>::const_iterator itr = options.ret_FTP_activated_filter_target_list().begin(); itr != options.ret_FTP_activated_filter_target_list().end(); ++itr) {
      list_out << " " << *itr;
    }
    list_out << '\n';

    //retry
    list_out << FI_RETRY << " " << dec << options.ret_Retry() << '\n';
    list_out << FI_RETRY_REPEAT << " " << dec << options.ret_Retry_repeat() << '\n';
    list_out << FI_FORCE_RETRY_404 << " " << dec << options.ret_force_retry_404() << '\n';
    list_out << FI_FORCE_RETRY_503 << " " << dec << options.ret_force_retry_503() << '\n';
  
    //some flags
    list_out << FI_DELETE_WHEN_COMPLETED << " " << dec << (unsigned int)options.ret_Delete_When_Finish() << '\n';
    list_out << FI_DONT_DELETE_WITHOUT_CRC << " " << dec << (unsigned int)options.ret_Dont_Delete_Without_CRC() << '\n';
    list_out << FI_NO_CRC_CHECKING << " " << dec << (unsigned int)options.ret_no_crc_checking() << '\n';
    list_out << FI_IGNORE_CRC_ERROR << " " << dec << (unsigned int)options.ret_ignore_crc_error() << '\n';

    list_out << "%End_Item" << '\n';
  }
  list_out << "%End_Items_Section" << '\n';

  return true;
}

bool ItemList::Save_app_settings()
{
  ofstream app_out;

  app_out.open(file_app_settings.c_str(), ios::out, 0600);
  if(app_out.bad()) {
    cerr << "ERROR:Unable to save app settings" << endl;
    return false;
  }

  app_out << ARIA_VERSION << '\n';
  app_out << "%Threads: " << appoption->ret_Maxthread() << '\n';
  app_out << "%Automatic_Start: " << (unsigned int)appoption->Whether_use_automatic_start() << '\n';
  app_out << "%Use-Ignore-Error-Item: " << (unsigned int)appoption->ret_use_ignore_error_item() << '\n';
  app_out << "%USE-IGNORE-EXTENTION-LIST: " << (unsigned int)appoption->ret_use_ignore_extention_list() << '\n';
  if(!appoption->ret_ignore_extention_list().empty()) {
    app_out << "%IGNORE-EXTENTION-LIST:";
    for(list<string>::const_iterator itr = appoption->ret_ignore_extention_list().begin(); itr != appoption->ret_ignore_extention_list().end(); ++itr) {
      app_out << " " << *itr;
    }
    app_out << '\n';
  }
  app_out << "%Use-Autosave: " << (unsigned int)appoption->ret_use_autosave() << '\n';
  app_out << "%Autosave-Interval: " << (unsigned int)appoption->ret_autosave_interval() << '\n';

  app_out << "%Use-Arbitrary-Command: " << (unsigned int)appoption->ret_use_arb_command() << '\n';
  app_out << "%Arbitrary-Command: " << appoption->ret_arb_command() << '\n';
  app_out << "%Use-Timer: " << (unsigned int)appoption->ret_use_timer() << '\n';
  app_out << "%Timer-Start-Hour: " << appoption->ret_timer_hour_start() << '\n';
  app_out << "%Timer-Start-Min: " << appoption->ret_timer_min_start() << '\n';
  app_out << "%Timer-Stop-Hour: " << appoption->ret_timer_hour_stop() << '\n';;
  app_out << "%Timer-Stop-Min: " << appoption->ret_timer_min_stop() << '\n';;

  app_out << "%CONFORM-DELETE: " << (unsigned int)appoption->ret_conform_delete() << '\n';
  app_out << "%CONFORM-EXIT: " << (unsigned int)appoption->ret_conform_exit() << '\n';
  app_out << "%USE-SERVERTEMPLATE: " << (unsigned int)appoption->ret_use_servertemplate() << '\n';
  list<string> names;
  app_out << "%SERVER-TEMPLATE-IN-USE: ";
  names = appoption->ret_svt_name_list();
  for(list<string>::iterator itr = names.begin(); itr != names.end(); ++itr) {
    app_out << " " << *itr;
  }
  app_out << '\n';
  app_out << "%COMMAND-LIST-IN-USE: ";
  names = appoption->ret_com_name_list();
  for(list<string>::iterator itr = names.begin(); itr != names.end(); ++itr) {
    app_out << " " << *itr;
  }
  app_out << '\n';

  app_out << "%USE-COMMANDLIST: " << (unsigned int)appoption->ret_use_commandlist() << '\n';
  app_out << "%USE-SIZE-HUMAN-READABLE: " << (unsigned int)appoption->ret_use_size_human_readable() << '\n';
  app_out << "%ICON-DIR: " << appoption->ret_icon_dir() << '\n';

  return true;
}

bool ItemList::Save_default_item_settings()
{
  ofstream default_out(file_default_item_settings.c_str(), ios::out, 0600);
  if(default_out.bad()) {
    cerr << "ERROR:Unable to save default settings" << endl;
    return false;
  }
  // file header
  default_out << ARIA_VERSION << '\n';

  Save_default_item_option_sub(default_out);

  return true;
}

const string& ItemList::ret_file_http_proxy_list() const
{
  return file_http_proxy_list;
}

const string& ItemList::ret_file_useragent_list() const
{
  return file_useragent_list;
}

const string& ItemList::ret_file_server_settings() const
{
  return file_server_settings;
}

const string& ItemList::ret_file_command_list() const
{
  return file_command_list;
}

const string& ItemList::ret_file_history() const
{
  return file_history;
}

bool ItemList::Restore_saved_app_settings()
{
  ifstream infile;
  string line;

  infile.open(file_app_settings.c_str(), ios::skipws|ios::in);
  if(infile.bad()) return false;

  getline(infile, line, '\n');
  if(line != ARIA_VERSION) {
    console_item->Send_message_to_gui(_("warning:different version number found in file header"), MSG_SYS_ERROR);
  }
  map<string, string> app_map;
  while(1) {
    string item;
    infile >> item;
    if(infile.eof()) break;
    getline(infile, app_map[item], '\n');
    app_map[item] = Remove_white(app_map[item]);
  }
  infile.close();

  unsigned int maxthread = stoui(app_map["%Threads:"], 10);

  bool use_autosave = (bool)stoi(app_map["%Use-Autosave:"]);
  unsigned int autosave_interval = stoi(app_map["%Autosave-Interval:"]);

  bool use_arb_command = (bool)stoi(app_map["%Use-Arbitrary-Command:"]);
  string arb_command = app_map["%Arbitrary-Command:"];

  bool use_ignore_error_item = (bool)stoi(app_map["%Use-Ignore-Error-Item:"]);
  bool use_timer = (bool)stoi(app_map["%Use-Timer:"]);
  int hour_start = stoi(app_map["%Timer-Start-Hour:"]);
  int min_start = stoi(app_map["%Timer-Start-Min:"]);
  int hour_stop = stoi(app_map["%Timer-Stop-Hour:"]);
  int min_stop = stoi(app_map["%Timer-Stop-Min:"]);
  TimerData timerdata(hour_start, min_start, hour_stop, min_stop);

  bool automatic_start = (bool)stoi(app_map["%Automatic_Start:"]);
  if(maxthread < 0) { 
    maxthread = 1;
  } else if(maxthread > thread_limit) {
    maxthread = thread_limit;
  }
    
  bool conform_delete = (bool)stoi(app_map["%CONFORM-DELETE:"]);
  bool conform_exit = (bool)stoi(app_map["%CONFORM-EXIT:"]);
  bool use_ignore_extention_list = (bool)stoi(app_map["%USE-IGNORE-EXTENTION-LIST:"]);
  bool use_commandlist = (bool)stoi(app_map["%USE-COMMANDLIST:"]);
  list<string> ignore_extention_list;
  while(!app_map["%IGNORE-EXTENTION-LIST:"].empty()) {
    ignore_extention_list.push_back(Token_splitter(app_map["%IGNORE-EXTENTION-LIST:"], " ,"));
  }
  bool use_size_human_readable = (bool)stoi(app_map["%USE-SIZE-HUMAN-READABLE:"]);
  string icon_dir = app_map["%ICON-DIR:"];
  bool use_servertemplate = (bool)stoi(app_map["%USE-SERVERTEMPLATE:"]);
  list<string> svt_name_list;
  while(!app_map["%SERVER-TEMPLATE-IN-USE:"].empty()) {
    svt_name_list.push_back(Token_splitter(app_map["%SERVER-TEMPLATE-IN-USE:"], " ,"));
  }
  list<string> com_name_list;
  while(!app_map["%COMMAND-LIST-IN-USE:"].empty()) {
    com_name_list.push_back(Token_splitter(app_map["%COMMAND-LIST-IN-USE:"], " ,"));
  }

  appoption->Set_Option_Values(maxthread, automatic_start, use_ignore_error_item, use_ignore_extention_list, ignore_extention_list, use_autosave, autosave_interval, use_arb_command, arb_command, use_timer, timerdata, conform_delete, conform_exit, use_servertemplate, use_commandlist, use_size_human_readable, icon_dir, svt_name_list, com_name_list);
  
  return true;
}

bool ItemList::Restore_saved_default_item_settings()
{
  ifstream infile(file_default_item_settings.c_str(), ios::skipws|ios::in);
  if(infile.bad() || infile.eof()) return false;

  string header;
  getline(infile, header, '\n');
  if(header != ARIA_VERSION) {
    console_item->Send_message_to_gui(_("warning:different version number found in file header"), MSG_SYS_ERROR);
  }
  // restore default item option
  Restore_default_item_option_sub(infile);

  return true;
}
