//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: PasteWindow.cc,v 1.12 2001/03/21 08:24:40 tujikawa Exp $

#include "PasteWindow.h"
#include "ItemCell_HTTP.h"
#include "ItemCell_FTP.h"

extern ItemManager *item_manager;
extern pthread_mutex_t itemlistlock;
extern ItemManager *item_manager_paste;
extern void Add_new_item_to_downloadlist(ItemCell *itemcell);
extern AppOption *appoption;
extern ThreadManager *thread_manager;
extern pthread_cond_t itemlistnotempty;
extern ItemCell *console_item;
extern GtkWidget *itemlistwidget;
extern void Set_sensitive__list_not_empty();
extern unsigned int Find_nearest_selected_row(GtkWidget *clist, unsigned int row_current);
extern void Setup_dnd(GtkWidget *toplevel, void (*Signal_Func)(GtkWidget  *w,
							GdkDragContext *context,
							int x,
							int y,
							GtkSelectionData *data,
							unsigned int info,
							unsigned int time));

static GtkWidget *item_individual_item;
static GtkWidget *reset_to_default_item;
static GtkWidget *reset_to_default_nosd_item;
//static GtkWidget *reset_to_default_norec_item;
static GtkWidget *item_default_item;
static GtkWidget *reset_to_default_all_item;
static GtkWidget *reset_to_default_all_nosd_item;
//static GtkWidget *reset_to_default_all_norec_item;

static void Paste_set_sensitive__no_item_selected()
{
  gtk_widget_set_sensitive(item_individual_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_nosd_item, FALSE);
}

static void Paste_set_sensitive__items_selected()
{
  gtk_widget_set_sensitive(item_individual_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_nosd_item, TRUE);
}

static void Paste_set_sensitive__list_empty()
{
  gtk_widget_set_sensitive(reset_to_default_all_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_all_nosd_item, FALSE);
}

static void Paste_set_sensitive__list_not_empty()
{
  gtk_widget_set_sensitive(reset_to_default_all_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_all_nosd_item, TRUE);  
}

static void Set_clist_column__save(GtkWidget *clist, int rowindex, const string& save_string)
{
  gtk_clist_set_text(GTK_CLIST(clist), rowindex, 1, save_string.c_str());
}

// paste_windowOKܥ򤪤ȸƤӽФ
static gboolean Paste_OK(GtkWidget *w, GtkWidget *paste_list)
{
  ItemCell *itemcell;
  string line;

  gtk_widget_hide(GTK_WIDGET(gtk_object_get_user_data(GTK_OBJECT(w))));
  if(GTK_CLIST(paste_list)->rows == 0) return(TRUE);

  pthread_mutex_lock(&itemlistlock);
  int maxrow = GTK_CLIST(itemlistwidget)->rows;

  int count = 0;// added 2001/3/21
  // add to main download list
  for(unsigned int i = 0;(itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(paste_list), i)) != NULL; i++) {
    // modified
    if(!item_manager->search_by_url(itemcell->ret_URL())) {
      itemcell->set_Options(itemcell->ret_Options_opt());
      itemcell->set_URL_Container(itemcell->ret_URL_Container_opt());
      itemcell->set_URL(itemcell->ret_URL_opt());
      Add_new_item_to_downloadlist(itemcell);
      ++count;// added 2001/3/21
    } else {
      delete itemcell;
    }
  }

  if(appoption->Whether_use_automatic_start() ||
     !thread_manager->Is_all_threads_sleeping()) {
    pthread_cond_broadcast(&itemlistnotempty);
  }
  gtk_clist_moveto(GTK_CLIST(itemlistwidget),
		   maxrow,
		   0,
		   0.0, 0.0);
  pthread_mutex_unlock(&itemlistlock);

  if(count) {// added 2001/3/21
    line = itos(count)+_("item(s) pasted");
    console_item->Send_message_to_gui(line, MSG_SYS_INFO);
    Set_sensitive__list_not_empty();
  }

  gtk_clist_clear(GTK_CLIST(paste_list));
  item_manager_paste->all_clear();

  return TRUE;
}

static gboolean Option_item_individual(GtkWidget *w, GtkWidget *paste_list)
{
  GList* node = GTK_CLIST(paste_list)->selection;
  if(node == NULL) return(TRUE);
  ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), GPOINTER_TO_UINT(node->data));

  pthread_mutex_lock(&itemlistlock);
  itemcell->Show_option_window();
  pthread_mutex_unlock(&itemlistlock);

  return TRUE;
}

static gboolean Option_reset_to_default(GtkWidget *w, GtkWidget *paste_list)
{
  GList *node = GTK_CLIST(paste_list)->selection;
  const Options& console_option = console_item->ret_Options_opt();
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), rowindex);
    // acquire lock;
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt() = console_option;

    Set_clist_column__save(paste_list, rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    //itemcell->Raise_option_update_flag();
    // release lock
    itemcell->release_Options_Lock();
    node = g_list_next(node);
  }
  return TRUE;
}

static gboolean Option_reset_to_default_nosd(GtkWidget *w, GtkWidget *paste_list)
{
  GList *node = GTK_CLIST(paste_list)->selection;
  const Options& console_option = console_item->ret_Options_opt();
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), rowindex);
    // acquire lock
    itemcell->get_Options_Lock();
    string storedir_temp = itemcell->ret_Options_opt().ret_Store_Dir();
    itemcell->ret_Options_opt() = console_option;
    itemcell->ret_Options_opt().set_Store_Dir(storedir_temp);

    //Set_clist_column__save(paste_list, rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    //itemcell->Raise_option_update_flag();
    // release
    itemcell->release_Options_Lock();
    node = g_list_next(node);
  }
  return TRUE;
}

static gboolean Option_item_default(GtkWidget *w, GtkWidget *paste_list)
{
  console_item->Show_option_window();
  return TRUE;
}

static gboolean Option_reset_to_default_all(GtkWidget *w, GtkWidget *paste_list)
{
  const Options& console_option = console_item->ret_Options_opt();
  for(int rowindex = 0; rowindex < GTK_CLIST(paste_list)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), rowindex);
    // acquire lock
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt() = console_option;

    Set_clist_column__save(paste_list, rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    //itemcell->Raise_option_update_flag();
    // release lock
    itemcell->release_Options_Lock();
  }
  return TRUE;
}

static gboolean Option_reset_to_default_all_nosd(GtkWidget *w, GtkWidget *paste_list)
{
  const Options& console_option = console_item->ret_Options_opt();
  for(int rowindex = 0; rowindex < GTK_CLIST(paste_list)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), rowindex);
    // acquire lock
    itemcell->get_Options_Lock();
    string storedir_temp = itemcell->ret_Options_opt().ret_Store_Dir();
    itemcell->ret_Options_opt() = console_option;
    itemcell->ret_Options_opt().set_Store_Dir(storedir_temp);

    Set_clist_column__save(paste_list, rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    //itemcell->Raise_option_update_flag();
    // release lock
    itemcell->release_Options_Lock();
  }
  return TRUE;
}

// paste windowdeleteܥ󤬲줿Ȥν
// CLIST additem_list򤵤Ƥ륢ƥबФ롣
static gboolean Paste_delete(GtkWidget *w, GtkWidget *paste_list)
{
  ItemCell *itemcell;
  GList *node = GTK_CLIST(paste_list)->selection;
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(paste_list), rowindex);
    item_manager_paste->unregist_item(itemcell);
    gtk_clist_remove(GTK_CLIST(paste_list), rowindex);
    delete itemcell;
    itemcell = NULL;
    node = GTK_CLIST(paste_list)->selection;
  }

  return TRUE;
}

// paste_window򱣤
static gboolean Paste_cancel(GtkWidget *paste_window)
{
  gtk_widget_hide(paste_window);
  GtkWidget *paste_list = (GtkWidget *)gtk_object_get_user_data(GTK_OBJECT(paste_window));
  ItemCell *itemcell;
  for(unsigned int i = 0;(itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(paste_list), i)) != NULL; i++) {
    delete itemcell;
  }
  gtk_clist_clear(GTK_CLIST(paste_list));
  item_manager_paste->all_clear();
  return TRUE;
}

/*
static gboolean Paste_selection_made(GtkWidget *paste_clist,
				     unsigned int row,
				     unsigned int column,
				     GdkEventButton *event,
				     gpointer data)
{
  static bool enterflag = false;
  unsigned int nearest_selected_row;

  if(enterflag == true || event == NULL) return TRUE;
  enterflag = true;
  if(event->button == 1) {
    if(event->state & GDK_CONTROL_MASK) {
    } else if(event->state & GDK_SHIFT_MASK) {
      nearest_selected_row = Find_nearest_selected_row(paste_clist, row);
      if(nearest_selected_row < row) {
	for(unsigned int i = nearest_selected_row+1; i < row; i++) {
	  gtk_clist_select_row(GTK_CLIST(paste_clist), i, 0);
	}
      } else {
	for(unsigned int i = row+1; i < nearest_selected_row; i++) {
	  gtk_clist_select_row(GTK_CLIST(paste_clist), i, 0);
	}
      }
    } else {
      gtk_clist_unselect_all(GTK_CLIST(paste_clist));
      gtk_clist_select_row(GTK_CLIST(paste_clist), row, 0);
    }
  }
  enterflag = false;
  return TRUE;
}

static gboolean Paste_unselection_made(GtkWidget *paste_clist,
				       unsigned int row,
				       unsigned int column,
				       GdkEventButton *event,
				       gpointer data)
{
  static bool enterflag = false;

  if(enterflag == true || event == NULL) return(TRUE);

  enterflag = true;
  if(event->button == 1) {
    if(event->state & (GDK_CONTROL_MASK || GDK_SHIFT_MASK)) {
    } else {
      gtk_clist_unselect_all(GTK_CLIST(paste_clist));
      gtk_clist_select_row(GTK_CLIST(paste_clist), row, 0);
    }
  }
  enterflag = false;
  return TRUE;
}
*/


static gboolean Paste_selection_made(GtkWidget *download_list,
				     unsigned int row,
				     unsigned int column,
				     GdkEventButton *event,
				     gpointer data)
{
  static bool enterflag = false;
  unsigned int nearest_selected_row;
  unsigned int i;

  if(enterflag == true || event == NULL) return TRUE;
  //pthread_mutex_t *history_lock_ptr = (pthread_mutex_t *)(gtk_object_get_user_data(GTK_OBJECT(download_list)));
  //pthread_mutex_lock(history_lock_ptr);
  enterflag = true;

 if(event->type == GDK_2BUTTON_PRESS) {
   ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(download_list), row);
   itemcell->Show_option_window();
  } else {
  switch(event->button) {
  case 1:
  case 3:
    if(event->state & GDK_CONTROL_MASK) {
      //Show_download_log(row);
    } else if(event->state & GDK_SHIFT_MASK) {
      nearest_selected_row = Find_nearest_selected_row(download_list, row);
      if(nearest_selected_row < row) {
	for(i = nearest_selected_row+1; i < row; i++) {
	  gtk_clist_select_row(GTK_CLIST(download_list), i, 0);
	}
      } else {
	for(i = row+1; i < nearest_selected_row; i++) {
	  gtk_clist_select_row(GTK_CLIST(download_list), i, 0);
	}
      }
      //Show_download_log(row);
    } else {// with no mask
      gtk_clist_unselect_all(GTK_CLIST(download_list));
      gtk_clist_select_row(GTK_CLIST(download_list), row, 0);
      //Show_download_log(row);
    }
    Paste_set_sensitive__items_selected();
    break;
  }
  }
  enterflag = false;
  //pthread_mutex_unlock(history_lock_ptr);
  return TRUE;
}

static gboolean Paste_unselection_made(GtkWidget *download_list,
				       unsigned int row,
				       unsigned int column,
				       GdkEventButton *event,
				       gpointer data)
{
  static bool enterflag = false;

  if(enterflag == true || event == NULL) return TRUE;
  //pthread_mutex_t *history_lock_ptr = (pthread_mutex_t *)(gtk_object_get_user_data(GTK_OBJECT(download_list)));
  //pthread_mutex_lock(history_lock_ptr);
  enterflag = true;
  switch(event->button) {
  case 1:
  case 3:
    if(event->state & (GDK_CONTROL_MASK | GDK_SHIFT_MASK)) {
      GList *node = GTK_CLIST(download_list)->selection;
      if(node == NULL) {
	/*
	unsigned int length = gtk_text_get_length(GTK_TEXT(text));
	gtk_text_set_point(GTK_TEXT (text), length);
	gtk_text_backward_delete(GTK_TEXT (text), length);
	gtk_notebook_set_tab_label_text(GTK_NOTEBOOK(text_notebook),
					item_log_page, _("not selected"));
	*/
	Paste_set_sensitive__no_item_selected();
      } else {
	gtk_clist_unselect_row(GTK_CLIST(download_list), row, 0);
	GList *node = GTK_CLIST(download_list)->selection;
	if(node == NULL) {
	  Paste_set_sensitive__no_item_selected();
	}
	//Show_download_log(row);
      }
    } else {
      GList *node = GTK_CLIST(download_list)->selection;
      if(node != NULL) {
	gtk_clist_unselect_all(GTK_CLIST(download_list));
	gtk_clist_select_row(GTK_CLIST(download_list), row, 0);
	Paste_set_sensitive__items_selected();
	//Show_download_log(row);
      } else {
	Paste_set_sensitive__no_item_selected();
      }
    }
    break;
  }
  enterflag = false;
  //pthread_mutex_unlock(history_lock_ptr);
  return TRUE;
}

static void add_url_on_dnd(GtkWidget  *paste_list,
			   GdkDragContext *context,
			   int x,
			   int y,
			   GtkSelectionData *data,
			   unsigned int info,
			   unsigned int time)
{
  string url_string = (char *)data->data;
  bool ws_quark;
  if(info == MIME_TEXT_PLAIN) {
    ws_quark = true;
  } else {
    ws_quark = false;
  }

  while(url_string.size()) {
    string url = URLcontainer::Find_URL(url_string, ws_quark);
    URLcontainer urlcon;
    if(urlcon.Parse_URL(url) &&
       //!item_manager->search_by_url(urlcon.ret_URL()) &&
       !item_manager_paste->search_by_url(urlcon.ret_URL())) {
      Options options = console_item->ret_Options_opt();

      ItemCell *itemcell;
      if(urlcon.ret_Protocol() == "http:") {
	itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
      } else if(urlcon.ret_Protocol() == "ftp:") {
	itemcell = new ItemCell_FTP(url, urlcon, options, "created");
      } else {
	// unsupported protocols
	continue;
      }
      char *clist_item[2];
      clist_item[0] = strdup(urlcon.ret_URL().c_str());
      clist_item[1] = strdup(itemcell->ret_Options_opt().ret_Store_Dir().c_str());
      int rowindex = gtk_clist_append(GTK_CLIST(paste_list), clist_item);
      //GdkColor color = {0, 0, 0};
      //gtk_clist_set_foreground(GTK_CLIST(paste_list), rowindex, &color);

      delete [] clist_item[0];
      delete [] clist_item[1];
      gtk_clist_set_row_data(GTK_CLIST(paste_list), rowindex, itemcell);
      item_manager_paste->regist_item_back(itemcell);
    }
  }
}

void PasteWindow::Create_option_menu(GtkWidget *window, GtkWidget *menu_bar, GtkAccelGroup *accel_group) {
  GtkWidget* option_menu;
  GtkWidget* option_item;
  GtkWidget* separator1_item;

  //create option menu
  option_menu = gtk_menu_new();

  item_individual_item = gtk_menu_item_new_with_label(_("selected item option"));
  reset_to_default_item = gtk_menu_item_new_with_label(_("apply default item option to selected items"));
  reset_to_default_nosd_item = gtk_menu_item_new_with_label(_("apply default item option to selected items(except for save dir)"));
  separator1_item = gtk_menu_item_new();
  item_default_item = gtk_menu_item_new_with_label(_("default item option"));

  reset_to_default_all_item = gtk_menu_item_new_with_label(_("apply default item option to all items"));
  reset_to_default_all_nosd_item = gtk_menu_item_new_with_label(_("apply default item option to all items(except for save dir)"));
  
  gtk_menu_append(GTK_MENU(option_menu), item_individual_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_nosd_item);
  gtk_menu_append(GTK_MENU(option_menu), separator1_item);
  gtk_menu_append(GTK_MENU(option_menu), item_default_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_all_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_all_nosd_item);

  // signal handling
  gtk_signal_connect(GTK_OBJECT(item_individual_item), "activate",
		     GTK_SIGNAL_FUNC(Option_item_individual), GTK_OBJECT(paste_list));
  gtk_signal_connect(GTK_OBJECT(reset_to_default_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default), GTK_OBJECT(paste_list));
  gtk_signal_connect(GTK_OBJECT(reset_to_default_nosd_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_nosd), GTK_OBJECT(paste_list));
  gtk_signal_connect(GTK_OBJECT(item_default_item), "activate",
		     GTK_SIGNAL_FUNC(Option_item_default), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_all_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_all), GTK_OBJECT(paste_list));
  gtk_signal_connect(GTK_OBJECT(reset_to_default_all_nosd_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_all_nosd), GTK_OBJECT(paste_list));

  // accelerator key
  gtk_widget_add_accelerator(item_individual_item,
			     "activate",
			     accel_group,
			     'O',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(item_default_item,
			     "activate",
			     accel_group,
			     'Z',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(reset_to_default_item,
			     "activate",
			     accel_group,
			     'J',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(reset_to_default_nosd_item,
			     "activate",
			     accel_group,
			     'K',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(reset_to_default_all_item,
			     "activate",
			     accel_group,
			     'G',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);
  gtk_widget_add_accelerator(reset_to_default_all_nosd_item,
			     "activate",
			     accel_group,
			     'H',
			     GDK_MOD1_MASK,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_show(item_individual_item);
  gtk_widget_show(reset_to_default_item);
  gtk_widget_show(reset_to_default_nosd_item);
  gtk_widget_show(separator1_item);
  gtk_widget_show(item_default_item);
  gtk_widget_show(reset_to_default_all_item);
  gtk_widget_show(reset_to_default_all_nosd_item);

  Paste_set_sensitive__no_item_selected();
  Paste_set_sensitive__list_empty();

  option_item = gtk_menu_item_new_with_label(_("Option"));
  gtk_widget_show(option_item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(option_item), option_menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), option_item);
}

GtkWidget *PasteWindow::Create_menu_bar(GtkWidget *window)
{
  GtkAccelGroup *accel_group;

  GtkWidget *menu_bar = gtk_menu_bar_new();
  gtk_menu_bar_set_shadow_type(GTK_MENU_BAR(menu_bar), GTK_SHADOW_ETCHED_OUT);
  gtk_widget_show(menu_bar);

  accel_group = gtk_accel_group_new();
  gtk_accel_group_attach(accel_group, GTK_OBJECT(window));

  //Create_file_menu(window, menu_bar, accel_group);
  //Create_edit_menu(window, menu_bar, accel_group);
  //Create_download_menu(menu_bar, accel_group);
  Create_option_menu(window, menu_bar, accel_group);
  //Create_help_menu(window, menu_bar, accel_group);
  return menu_bar;
}

PasteWindow::PasteWindow(GtkWindow *toplevel = NULL)
{
  char *title[2];

  //URLŽդѤΥκ
  window = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(window), _("Paste URL list"));
  gtk_widget_set_usize(GTK_WIDGET(window), 680, 300);
  gtk_signal_connect_object(GTK_OBJECT(window),
			    "delete_event",
			    GTK_SIGNAL_FUNC(Paste_cancel),
			    GTK_OBJECT(window));

  // create CList
  title[0] = _("URL");
  title[1] = _("Save");
  paste_list = gtk_clist_new_with_titles(2, title);
  Setup_dnd(paste_list, add_url_on_dnd);

  // create menu bar
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox), Create_menu_bar(window), FALSE, TRUE, 0);

  // CList åȤѥå륹ɥɥ
  GtkWidget *scrolled_window = gtk_scrolled_window_new (NULL, NULL);
  
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scrolled_window),
				 GTK_POLICY_ALWAYS, GTK_POLICY_ALWAYS);
  
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->vbox),
		     scrolled_window, TRUE, TRUE, 0);
  gtk_widget_show(scrolled_window);  
  
  // CList 롣Ǥ 2 Ȥ
  gtk_widget_set_name(paste_list, "clist");

  gtk_object_set_user_data(GTK_OBJECT(window), paste_list);
  gtk_clist_set_column_auto_resize(GTK_CLIST(paste_list),
				   0,
				   TRUE);
  gtk_clist_set_selection_mode(GTK_CLIST(paste_list), GTK_SELECTION_MULTIPLE);
  
  gtk_signal_connect(GTK_OBJECT(paste_list), "select_row",
		     GTK_SIGNAL_FUNC(Paste_selection_made),
		     NULL);
  gtk_signal_connect(GTK_OBJECT(paste_list), "unselect_row",
		     GTK_SIGNAL_FUNC(Paste_unselection_made),
		     NULL);
  // ˱Ƥդɬפʤɤʤиɤɤʤ :)
  gtk_clist_set_shadow_type(GTK_CLIST(paste_list), GTK_SHADOW_ETCHED_OUT);

  // set column width
  gtk_clist_set_column_width(GTK_CLIST(paste_list), 0, 360);
  gtk_clist_set_column_width(GTK_CLIST(paste_list), 1, 360);

  // CList åȤľܥå˲äɽ
  gtk_container_add(GTK_CONTAINER(scrolled_window), paste_list);
  gtk_widget_show(paste_list);
  gtk_clist_column_titles_passive(GTK_CLIST(paste_list));

  //// action area
  GtkWidget *vbox = gtk_vbox_new(FALSE, 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(window)->action_area),
		     vbox, FALSE, FALSE, 0);
  gtk_widget_show(vbox);

  {
    GtkWidget *bbox = gtk_hbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_SPREAD);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
    gtk_box_pack_start(GTK_BOX(vbox),
		       bbox, FALSE, FALSE, 0);
    gtk_widget_show(bbox);
    
    // Paste button
    GtkWidget *Paste_button = gtk_button_new_with_label(_("Paste"));
    GTK_WIDGET_SET_FLAGS(Paste_button, GTK_CAN_DEFAULT);
    gtk_window_set_default(GTK_WINDOW(window), Paste_button);
    gtk_signal_connect(GTK_OBJECT(Paste_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(Paste_OK),
		       GTK_OBJECT(paste_list));
    gtk_box_pack_start(GTK_BOX(bbox),
		       Paste_button, TRUE, TRUE, 0);
    gtk_widget_show(Paste_button);
    gtk_object_set_user_data(GTK_OBJECT(Paste_button), window);

    // item Option button
    GtkWidget *Option_button = gtk_button_new_with_label(_("Option"));
    GTK_WIDGET_SET_FLAGS(Option_button, GTK_CAN_DEFAULT);
    gtk_signal_connect(GTK_OBJECT(Option_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(Option_item_individual),
		       GTK_OBJECT(paste_list));
    gtk_box_pack_start(GTK_BOX(bbox),
		       Option_button, TRUE, TRUE, 0);
    gtk_widget_show(Option_button);
    
    // Delete button
    GtkWidget *Delete_button = gtk_button_new_with_label(_("Delete"));
    GTK_WIDGET_SET_FLAGS(Delete_button, GTK_CAN_DEFAULT);
    gtk_signal_connect(GTK_OBJECT(Delete_button),
		       "clicked",
		       GTK_SIGNAL_FUNC(Paste_delete),
		       GTK_OBJECT(paste_list));
    gtk_box_pack_start(GTK_BOX(bbox),
		       Delete_button, TRUE, TRUE, 0);
    gtk_widget_show(Delete_button);

    // Cancel button
    GtkWidget *Cancel_button = gtk_button_new_with_label(_("CANCEL"));
    GTK_WIDGET_SET_FLAGS(Cancel_button, GTK_CAN_DEFAULT);
    gtk_signal_connect_object(GTK_OBJECT(Cancel_button),
			      "clicked",
			      GTK_SIGNAL_FUNC(Paste_cancel),
			      GTK_OBJECT(window));
    gtk_box_pack_start(GTK_BOX(bbox),
		       Cancel_button, TRUE, TRUE, 0);
    gtk_widget_show(Cancel_button);
  }

  // setup dnd

  //Setup_dnd(paste_list, add_url_on_dnd);
  /*
  static GtkTargetEntry target_table[] = {
    { "x-url/http", 0, 0 },
    { "x-url/ftp", 0, 0 },
    { "_NETSCAPE_URL", 0, 0 }
  };

  gtk_drag_dest_set(paste_list,
		    (GtkDestDefaults)(GTK_DEST_DEFAULT_MOTION |
				      GTK_DEST_DEFAULT_HIGHLIGHT |
				      GTK_DEST_DEFAULT_DROP),
		    target_table, 3,
		    GDK_ACTION_COPY);
  gtk_signal_connect(GTK_OBJECT(paste_list), "drag_data_received",
		     GTK_SIGNAL_FUNC(add_url_on_dnd),
		     NULL);
  */
  //// window property
  // modified
  gtk_window_set_modal(GTK_WINDOW(window), FALSE);
  if(toplevel != NULL)
    gtk_window_set_transient_for(GTK_WINDOW(window),
				 GTK_WINDOW(toplevel));
}

GtkWidget *PasteWindow::ret_paste_list()
{
  return paste_list;
}

// clipboard_contentsˤϥåץܡɤƤnull-terminated
// ʸǳǼƤ롣CLIST paste_listURLȻפ
// ɲä롣ޤclass ItemCellΥ󥹥󥹤롣
bool PasteWindow::add_url(string url_string, int mime_info = MIME_URL)
{
  bool retval = false;
  bool ws_quark;
  //cerr << mime_info << endl;
  if(mime_info == MIME_TEXT_PLAIN) {
    ws_quark = true;
  } else {
    ws_quark = false;
  }
  while(url_string.size()) {
    string url = URLcontainer::Find_URL(url_string, ws_quark);
    if(add_item(url)) retval = true;
  }
  return retval;
}

bool PasteWindow::add_url_with_unfold(string url_string)
{
  bool retval = false;
  while(url_string.size()) {
    list<string> url_list = URLcontainer::Unfold_URL(URLcontainer::Find_URL(url_string));
    //while(unfolded_urls.size()) {
    //string url = URLcontainer::Find_URL(unfolded_urls);
    for(list<string>::const_iterator itr = url_list.begin(); itr != url_list.end(); ++itr) {
      if(add_item(*itr)) retval = true;
    }
  }
  return retval;
}

bool PasteWindow::add_item(ItemCell *itemcell)
{
  char *clist_item[2];
  clist_item[0] = strdup(itemcell->ret_URL().c_str());
  clist_item[1] = strdup(itemcell->ret_Options_opt().ret_Store_Dir().c_str());
  int rowindex = gtk_clist_append(GTK_CLIST(paste_list), clist_item);
  //GdkColor color = {0, 0, 0};
  //gtk_clist_set_foreground(GTK_CLIST(paste_list), rowindex, &color);

  delete [] clist_item[0];
  delete [] clist_item[1];
  gtk_clist_set_row_data(GTK_CLIST(paste_list), rowindex, itemcell);
  item_manager_paste->regist_item_back(itemcell);
  return true;
}

bool PasteWindow::add_item(const string& url)
{
  URLcontainer urlcon;

  //urlcon.Parse_URL(url);
  if(urlcon.Parse_URL(url) &&
     //!item_manager->search_by_url(urlcon.ret_URL()) &&
     !item_manager_paste->search_by_url(urlcon.ret_URL())) {
    ItemCell *itemcell;
    Options options = console_item->ret_Options_opt();
    if(urlcon.ret_Protocol() == "http:") {
      itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
    } else if(urlcon.ret_Protocol() == "ftp:") {
      itemcell = new ItemCell_FTP(url, urlcon, options, "created");
    } else {
      // unsupported protocols
      return false;
    }
    char *clist_item[2];
    clist_item[0] = strdup(urlcon.ret_URL().c_str());
    clist_item[1] = strdup(itemcell->ret_Options_opt().ret_Store_Dir().c_str());
    int rowindex = gtk_clist_append(GTK_CLIST(paste_list), clist_item);
    //GdkColor color = {0, 0, 0};
    //gtk_clist_set_foreground(GTK_CLIST(paste_list), rowindex, &color);

    delete [] clist_item[0];
    delete [] clist_item[1];
    gtk_clist_set_row_data(GTK_CLIST(paste_list), rowindex, itemcell);
    item_manager_paste->regist_item_back(itemcell);
    return true;
  } else {
    return false;
  }
}

void PasteWindow::show()
{
  Paste_set_sensitive__list_not_empty();
  gtk_window_set_position(GTK_WINDOW(window), GTK_WIN_POS_MOUSE);
  gtk_widget_show(window);
}

void PasteWindow::hide()
{
  gtk_widget_hide(window);
}
