//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: Socket.cc,v 1.8 2001/03/05 09:43:27 tujikawa Exp $

#include "Socket.h"
#include "aria.h"

Socket::Socket()
{
  // make a socket
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if(sockfd >= 0) {
    socklen_t sockopt = 1;
    if(setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof(int)) < 0) {
      close(sockfd);
      sockfd = -1;
    }
  }
}

Socket::Socket(const Socket& src)
{
  sockfd = dup(src.ret_Desc());
}

Socket::Socket(int sockfd_in, SocketModeType mode)
{
  switch(mode) {
  case DUPE:
    sockfd = dup(sockfd_in);
    break;
  case DEFAULT:
  default:
    sockfd = sockfd_in;
    break;
  }
}

Socket::~Socket()
{
  //shutdown(sockfd, 2);
  close(sockfd);
}

Socket& Socket::operator=(const Socket& src)
{
  if(&src != this) {
    if(sockfd >= 0) {
      close(sockfd);
    }
    sockfd = dup(src.sockfd);
    del_flag = src.del_flag;
  }
  return *this;
}

bool Socket::bad() const
{
  if(sockfd < 0) return true;
  else return false;
}

int Socket::ret_Desc() const
{
  return sockfd;
}

bool Socket::reopen()
{
  if(sockfd >= 0) {
    //shutdown(sockfd, 2);
    close(sockfd);
  }
  // make a socket
  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if(sockfd >= 0) {
    socklen_t sockopt = 1;
    if(setsockopt(sockfd, SOL_SOCKET, SO_REUSEADDR, &sockopt, sizeof(int)) < 0) {
      close(sockfd);
      sockfd = -1;
      return false;
    }
  }
  return true;
}

int Socket::Getflags() const
{
  return fcntl(sockfd, F_GETFL, 0);
}

int Socket::Setflags(int flags) const
{
  return fcntl(sockfd, F_SETFL, flags);
}

void Socket::Shutdown(int mode) const
{
  shutdown(sockfd, mode);
}

int Socket::Connect(const struct sockaddr_in& serv_addr) const
{
  return connect(sockfd, (struct sockaddr*)&serv_addr, sizeof(serv_addr));
}

// if there is an error, return true;
bool Socket::check_error() const
{
  int retval;
  int arglen = sizeof(int);
  if(getsockopt(sockfd, SOL_SOCKET, SO_ERROR, &retval, (socklen_t *)&arglen) < 0) {
    return true;
  }
  if(retval > 0) {
    errno = retval;
    return true;
  } else {
    return false;
  }
}

int Socket::is_readready(int timeout, list<int>& rfds_list) const
{
  fd_set rfds;
  int maxfd = sockfd;
  FD_ZERO(&rfds);
  FD_SET(sockfd, &rfds);

  for(list<int>::iterator itr = rfds_list.begin(); itr != rfds_list.end(); ++itr) {

    FD_SET(*itr, &rfds);
    if(maxfd < *itr) maxfd = *itr;
  }

  int retval;
  if(timeout == -1) {
    retval = select(maxfd+1, &rfds, NULL, NULL, NULL);
  } else {
    struct timeval tv;
    tv.tv_sec = timeout;
    tv.tv_usec = 0;  
    retval = select(maxfd+1, &rfds, NULL, NULL, &tv);
  }
  list<int> rfds_list_temp = rfds_list;
  rfds_list.clear();
  if(retval > 0) {
    for(list<int>::iterator itr = rfds_list_temp.begin(); itr != rfds_list_temp.end(); ++itr) {
      if(FD_ISSET(*itr, &rfds)) {
	rfds_list.push_back(*itr);
      }
    }
    if(FD_ISSET(sockfd, &rfds)) {
      rfds_list.push_back(sockfd);
    }
  }

  return retval;
}

bool Socket::is_set(const list<int>& fds_list) const
{
  if(fds_list.end() != find(fds_list.begin(), fds_list.end(), sockfd)) {
    return true;
  } else {
    return false;
  }
}

bool Socket::is_set(int fd, const list<int>& fds_list)
{
  if(fds_list.end() != find(fds_list.begin(), fds_list.end(), fd)) {
    return true;
  } else {
    return false;
  }
}

int Socket::is_readwriteready(int timeout, list<int>& rfds_list, list<int>& wfds_list) const
{
  int maxfd = sockfd;
  fd_set rfds;
  FD_ZERO(&rfds);
  FD_SET(sockfd, &rfds);
  for(list<int>::iterator itr = rfds_list.begin(); itr != rfds_list.end(); ++itr) {
    FD_SET(*itr, &rfds);
    if(maxfd < *itr) maxfd = *itr;
  }
  fd_set wfds;
  FD_ZERO(&wfds);
  FD_SET(sockfd, &wfds);
  for(list<int>::iterator itr = wfds_list.begin(); itr != wfds_list.end(); ++itr) {
    FD_SET(*itr, &wfds);
    if(maxfd < *itr) maxfd = *itr;
  }

  int retval;
  if(timeout == -1) {
    retval = select(maxfd+1, &rfds, &wfds, NULL, NULL);
  } else {
    struct timeval tv;
    tv.tv_sec = timeout;
    tv.tv_usec = 0;  
    retval = select(maxfd+1, &rfds, &wfds, NULL, &tv);
  }

  list<int> rfds_list_temp = rfds_list;
  list<int> wfds_list_temp = wfds_list;
  rfds_list.clear();
  wfds_list.clear();
  if(retval > 0) {
    for(list<int>::iterator itr = rfds_list_temp.begin(); itr != rfds_list_temp.end(); ++itr) {
      if(FD_ISSET(*itr, &rfds)) {
	rfds_list.push_back(*itr);
      }
    }
    for(list<int>::iterator itr = wfds_list_temp.begin(); itr != wfds_list_temp.end(); ++itr) {
      if(FD_ISSET(*itr, &wfds)) {
	wfds_list.push_back(*itr);
      }
    }
    if(FD_ISSET(sockfd, &rfds)) {
      rfds_list.push_back(sockfd);
    }    
    if(FD_ISSET(sockfd, &wfds)) {
      wfds_list.push_back(sockfd);
    }
  }
  return retval;
}

int Socket::is_writeready(int timeout, list<int>& wfds_list) const
{
  fd_set wfds;
  FD_ZERO(&wfds);
  FD_SET(sockfd, &wfds);

  int maxfd = sockfd;
  for(list<int>::iterator itr = wfds_list.begin(); itr != wfds_list.end(); ++itr) {
    FD_SET(*itr, &wfds);
    if(maxfd < *itr) maxfd = *itr;
  }
  int retval;
  if(timeout == -1) {
    retval = select(maxfd+1, NULL, &wfds, NULL, NULL);
  } else {
    struct timeval tv;
    tv.tv_sec = timeout;
    tv.tv_usec = 0;  
    retval = select(maxfd+1, NULL, &wfds, NULL, &tv);
  }

  list<int> wfds_list_temp = wfds_list;
  wfds_list.clear();
  if(retval > 0) {
    for(list<int>::iterator itr = wfds_list_temp.begin(); itr != wfds_list_temp.end(); ++itr) {
      if(FD_ISSET(*itr, &wfds)) {
	wfds_list.push_back(*itr);
      }
    }
    if(FD_ISSET(sockfd, &wfds)) {
      wfds_list.push_back(sockfd);
    }
  }
  return retval;
}

int Socket::Send(const string& command, int flags) const
{
  return send(sockfd, command.c_str(), command.length(), flags);
}
int Socket::Send(const string& command) const
{
  int flags = 0;
  return send(sockfd, command.c_str(), command.length(), flags);
}

int Socket::Recv(void* buf, size_t len, int flags = 0) const
{
  return recv(sockfd, buf, len, flags);
}

int Socket::Recvfrom(void* buf, size_t len, int from) const
{
  struct sockaddr_in addr_in;
  if(Getsockname(addr_in) < 0) {
    return -1;
  }
  int addrlen = sizeof(addr_in);
  return recvfrom(sockfd, buf, len, 0, (struct sockaddr *)&addr_in, (socklen_t *)&addrlen);
}


int Socket::Getsockname(struct sockaddr_in& addr) const
{
  int len = sizeof(struct sockaddr);
  return getsockname(sockfd, (struct sockaddr*)&addr, (socklen_t*)&len);
}

int Socket::Bind(struct sockaddr_in& addr) const
{
  return bind(sockfd, (struct sockaddr*)&addr, sizeof(addr));
}

int Socket::Listen(int i) const
{
  return listen(sockfd, 1);
}

int Socket::Listen() const
{
  return listen(sockfd, 1);
}

Socket Socket::Accept(struct sockaddr_in& addr) const
{
  int len = sizeof(struct sockaddr);
  int fd = accept(sockfd, (struct sockaddr*)&addr, (socklen_t*)&len);
  if(fd < 0) {
    throw SOCKET_EACCEPT;
  }
  Socket socket(fd, Socket::DEFAULT);
  return socket;
}

Socket Socket::Accept() const
{
  struct sockaddr addr;
  int len = sizeof(struct sockaddr_in);
  int fd = accept(sockfd, (struct sockaddr*)&addr, (socklen_t*)&len);
  // exception handling rquired here
  if(fd < 0) {
    throw SOCKET_EACCEPT;
  }
  Socket socket(fd, Socket::DEFAULT);
  return socket;
}
