//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_file.cc,v 1.19 2001/03/15 05:35:38 tujikawa Exp $

#include <fstream.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "aria.h"
#include "ItemCell.h"
#include "ThreadManager.h"
#include "AppOption.h"
#include "ItemList.h"
#include "Dialog.h"
#include "HistoryWindow.h"
#include "ShortCutKey.h"

//// new item window
extern gboolean File_new_item(GtkWidget *w, gpointer data);
extern void Create_new_item_window(GtkWidget *toplevel);

//// find hyperlink window
extern gboolean File_find_hyperlink(GtkWidget *w, gpointer data);
extern void Create_find_hyperlink_window(GtkWidget *toplevel);

//// Open URL list
extern gboolean File_open_URL_list(GtkWidget *w, gpointer unused);
//// Open CRC list
extern gboolean File_open_CRC_list(GtkWidget *w, gpointer unused);
//// Open saved list
extern gboolean File_open_Saved_list(GtkWidget *w, gpointer unused);
//// Save list
extern gboolean File_save_list(GtkWidget *w, gpointer unused);
extern gboolean Download_start_all(GtkWidget *w, gpointer unused);
extern gboolean Download_stop_all(GtkWidget* w, gpointer unused);

extern int pipetogui[2];
extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;
extern ItemCell *console_item;
extern ProxyList *http_proxy_list;
extern GtkWidget *itemlistwidget;
extern GtkWidget *text;
extern ItemList *itemlist_global;
extern AppOption *appoption;
extern ThreadManager *thread_manager;
extern Dialog *cdialog;
extern HistoryWindow *history_window;
// global variables

// window򱣤
gboolean Hide_window(GtkWidget *window, gpointer unused)
{
  gtk_widget_hide(window);

  return TRUE;
}

// Autosave
gboolean Autosave_list(gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  itemlist_global->Save_current_list();
  itemlist_global->Save_default_item_settings();
  itemlist_global->Save_app_settings();
  http_proxy_list->Save_proxy_list(itemlist_global->ret_file_http_proxy_list());
  history_window->write_into_file(itemlist_global->ret_file_history());
  pthread_mutex_unlock(&itemlistlock);
  console_item->Send_message_to_gui(_("autosave done"), MSG_SYS_INFO);
  return TRUE;
}

gboolean Timer_start(gpointer data)
{
  //cerr << "in function timer start" << endl;
  int curtime = time(NULL);
  int diff = appoption->ret_timer_start_time()-curtime;
  if(diff <= 60 && diff > 0) {
    appoption->Start_start_timer(1000);
  } else if(diff <= 0) {
    //cerr << "  now is the time to start!!!" << endl;
    Download_start_all(NULL, NULL);
    appoption->Update_timer_start();
    appoption->Start_start_timer(60000);
  }
  return TRUE;
}

gboolean Timer_stop(gpointer data)
{
  //cerr << "function timer stop" << endl;
  int curtime = time(NULL);
  
  int diff = appoption->ret_timer_stop_time()-curtime;
  if(diff <= 60 && diff > 0) {
    appoption->Start_stop_timer(1000);
  } else if(diff <= 0) {
    //cerr << "  now is the time to stop!!!" << endl;
    Download_stop_all(NULL, NULL);
    appoption->Update_timer_stop();
    appoption->Start_stop_timer(60000);
  }
  return TRUE;
}

// ץཪλåƥåɤã
static void Send_halt_message()
{
  pthread_mutex_lock(&itemlistlock);
  thread_manager->set_halt_flag();

  for(int rowindex = 0; rowindex < GTK_CLIST(itemlistwidget)->rows; rowindex++) {
    ItemCell* itemcell = (ItemCell *)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    switch(itemcell->ret_Status()) {

    case ItemCell::ITEM_DOWNLOAD:
    case ItemCell::ITEM_INUSE:
    case ItemCell::ITEM_DOWNLOAD_AGAIN:
    case ItemCell::ITEM_DOWNLOAD_INTERNAL_AGAIN:
    case ItemCell::ITEM_INUSE_AGAIN:
      {
	ItemCommand itemcommand;
	itemcommand.commandtype = ItemCommand::COMMAND_HALT;
	itemcell->set_Status(ItemCell::ITEM_READY);
	write(itemcell->ret_Desc_w(), &itemcommand, sizeof(ItemCommand));
      }
      break;
    case ItemCell::ITEM_DOWNLOAD_PARTIAL:
    case ItemCell::ITEM_READY_CONCAT:
    case ItemCell::ITEM_READY_AGAIN:
      itemcell->set_Status(ItemCell::ITEM_READY);
      break;
    default:
      break;
    }
  }
  pthread_cond_broadcast(&itemlistnotempty);
  pthread_mutex_unlock(&itemlistlock);
  thread_manager->Wait_for_join();
}

static void Save_default_settings()
{
  itemlist_global->Save_default_item_settings();
}

static void Save_app_settings()
{
  itemlist_global->Save_app_settings();

}

static void Save_proxy_list()
{
  http_proxy_list->Save_proxy_list(itemlist_global->ret_file_http_proxy_list());
}

static void Save_history()
{
  history_window->write_into_file(itemlist_global->ret_file_history());
}
// ߤΥꥹȤ¸λ
gboolean File_quit_wos(GtkWidget *w, GtkWidget *window)
{
  cdialog->hide();
  Send_halt_message();  
  Save_default_settings();
  Save_app_settings();
  Save_proxy_list();
  Save_history();

  close(pipetogui[0]);
  close(pipetogui[1]);

  exit(0);

  return TRUE;
}

// ߤΥꥹȤ¸ƽλ
gboolean File_quit(GtkWidget *w, GtkWidget *unused)
{
  cdialog->hide();
  Send_halt_message();
  Save_default_settings();
  Save_app_settings();
  Save_proxy_list();

  Save_history();

  // save current list and its crc
  pthread_mutex_lock(&itemlistlock);
  itemlist_global->Save_current_list();
  pthread_mutex_unlock(&itemlistlock);

  close(pipetogui[0]);
  close(pipetogui[1]);

  exit(0);

  return TRUE;
}

gboolean File_quit_c(GtkWidget *w, gboolean (*Signal_Func)(GtkWidget *x, GtkWidget *window))
{
  if(appoption->ret_conform_exit()) {
    cdialog->setup(_("Quit program"),
		   _("Are you sure to quit?"),
		   Signal_Func);
    cdialog->set_cancel_button_visible(false);
    cdialog->show();
  } else {
    (*Signal_Func)(NULL, NULL);
  }
  return TRUE;
}

/*
gboolean File_signal_func(GtkWidget* w, gboolean (*Signal_Func)())
{
  (*Signal_Func)();
  return TRUE;
}
*/

// ե˥塼
void Create_file_menu(GtkWidget* toplevel, GtkWidget* menu_bar, GtkAccelGroup* accel_group)
{
  GtkWidget *new_item_item;
  GtkWidget *find_hyperlink_item;
  GtkWidget *file_menu, *file_item, *URLopen_item, *CRCopen_item, *SavedListopen_item, *Listsave_item, *quit_item, *quit_wos_item, *sep_file;

      //create file menu
  file_menu = gtk_menu_new();

  sep_file = gtk_menu_item_new();
  new_item_item = gtk_menu_item_new_with_label(_("New Item"));

  URLopen_item = gtk_menu_item_new_with_label(_("Open URL list"));
  CRCopen_item = gtk_menu_item_new_with_label(_("Open CRC list"));
  find_hyperlink_item = gtk_menu_item_new_with_label(_("Find Hyperlink"));
  SavedListopen_item = gtk_menu_item_new_with_label(_("Open saved list"));
  Listsave_item = gtk_menu_item_new_with_label(_("Save current list"));
  quit_item = gtk_menu_item_new_with_label(_("Quit with saving list"));
  quit_wos_item = gtk_menu_item_new_with_label(_("Quit without saving list"));

 
  // append
  gtk_menu_append(GTK_MENU(file_menu), new_item_item);
  gtk_menu_append(GTK_MENU(file_menu), URLopen_item);
  gtk_menu_append(GTK_MENU(file_menu), CRCopen_item);
  gtk_menu_append(GTK_MENU(file_menu), find_hyperlink_item);
  gtk_menu_append(GTK_MENU(file_menu), SavedListopen_item);
  gtk_menu_append(GTK_MENU(file_menu), Listsave_item);
  gtk_menu_append(GTK_MENU(file_menu), sep_file);
  gtk_menu_append(GTK_MENU(file_menu), quit_item);
  gtk_menu_append(GTK_MENU(file_menu), quit_wos_item);

  // accelerator
  gtk_widget_add_accelerator(new_item_item,
			     "activate",
			     accel_group,
			     SC_NEWITEM,
			     SCM_NEWITEM,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(URLopen_item,
			     "activate",
			     accel_group,
			     SC_OPENURL,
			     SCM_OPENURL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(CRCopen_item,
			     "activate",
			     accel_group,
			     SC_OPENCRC,
			     SCM_OPENCRC,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(SavedListopen_item,
			     "activate",
			     accel_group,
			     SC_OPEN,
			     SCM_OPEN,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(Listsave_item,
			     "activate",
			     accel_group,
			     SC_SAVE,
			     SCM_SAVE,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(find_hyperlink_item,
			    "activate",
			     accel_group,
			     SC_FINDHREF,
			     SCM_FINDHREF,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(quit_item,
			     "activate",
			     accel_group,
			     SC_QUIT,
			     SCM_QUIT,
			     GTK_ACCEL_VISIBLE);

  // create new item window
  Create_new_item_window(toplevel);

  // signal handling function
  gtk_signal_connect(GTK_OBJECT(new_item_item), "activate",
		     GTK_SIGNAL_FUNC(File_new_item),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(URLopen_item), "activate",
		     GTK_SIGNAL_FUNC(File_open_URL_list),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(CRCopen_item), "activate",
		     GTK_SIGNAL_FUNC(File_open_CRC_list),
		     NULL);

  Create_find_hyperlink_window(toplevel);
  gtk_signal_connect(GTK_OBJECT(find_hyperlink_item), "activate",
		     GTK_SIGNAL_FUNC(File_find_hyperlink),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(SavedListopen_item), "activate",
		     GTK_SIGNAL_FUNC(File_open_Saved_list),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(Listsave_item), "activate",
		     GTK_SIGNAL_FUNC(File_save_list),
		     NULL);

  gtk_signal_connect(GTK_OBJECT(quit_item), "activate",
		     GTK_SIGNAL_FUNC(File_quit_c),
		     (void*)File_quit);

  gtk_signal_connect(GTK_OBJECT(quit_wos_item), "activate",
		     GTK_SIGNAL_FUNC(File_quit_c),
		     (void*)File_quit_wos);

  // show widgets
  gtk_widget_show(new_item_item);
  gtk_widget_show(URLopen_item);
  gtk_widget_show(CRCopen_item);
  gtk_widget_show(find_hyperlink_item);
  gtk_widget_show(SavedListopen_item);
  gtk_widget_show(Listsave_item);
  gtk_widget_show(sep_file);
  gtk_widget_show(quit_item);
  gtk_widget_show(quit_wos_item);

  file_item = gtk_menu_item_new_with_label(_("File"));
  gtk_widget_show(file_item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(file_item), file_menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), file_item);
}

  //gtk_toolips_set_tip(tooltips, menuitem, szTip, NULL);
  //GTK_TOGGLE_BUTTON(widget)->active;
  //GTK_CHECK_MENU_ITEM(widget)->acitve;
  //toolbar = gtk_toolbar_new(GTK_ORIENTAION_HORIZONTAOL,
  //  GTK_TOOLBAR_ICONS);
  //gtk_widget_show(toolbar);
  //gtk_toolbar_append_item(GTK_TOOLBAR(toolbar),
  //NULL, "New windw", NULL,
  //CreateWidgetFromXpm(vbox_main, (gchar **)xpm_new),
  //(GtkSIgnalFunc)ButtonClicked,
  //NULL);
