//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_file_find_hyperlink.cc,v 1.6 2001/03/02 04:36:01 tujikawa Exp $
#include "aria.h"
#include "ItemCell_HTTP.h"
#include "ItemCell_FTP.h"
#include "ItemList.h"
#include "ThreadManager.h"
#include "AppOption.h"
#include "FileBrowser.h"
using namespace std;

// external functions
extern void Set_sensitive__list_not_empty();
extern gboolean Hide_window(GtkWidget *window, gpointer unused);

// external variables
extern ThreadManager *thread_manager;
extern ItemList *itemlist_global;
extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;
extern ItemCell *console_item;
extern AppOption *appoption;
extern GtkWidget *itemlistwidget;
extern FileBrowser *cfilebrowser;

// global variables
static GtkWidget *base_url_window;
static GtkWidget *base_url_entry;

static gboolean File_ok_find_hyperlink(GtkWidget *w, GtkWidget *fs)
{
  const char *filename;

  cfilebrowser->hide();
  if((filename = gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs))) != NULL) {
    pthread_mutex_lock(&itemlistlock);
    string base_url = Remove_white(gtk_entry_get_text(GTK_ENTRY(base_url_entry)));
    if(!itemlist_global->Find_Hyperlink_from_file(filename, base_url, ItemList::FINDHREF_PASTE)) {
      pthread_mutex_unlock(&itemlistlock);
      console_item->Send_message_to_gui(_("Error occurd while reading html file"), MSG_SYS_ERROR);
    } else {
      if(appoption->Whether_use_automatic_start() ||
	 !thread_manager->Is_all_threads_sleeping()) {
	pthread_cond_broadcast(&itemlistnotempty);
      }
      pthread_mutex_unlock(&itemlistlock);
      console_item->Send_message_to_gui(_("Hyperlink finding complete"), MSG_SYS_INFO);
    }
    if(GTK_CLIST(itemlistwidget)->rows > 0) {
      Set_sensitive__list_not_empty();
    }
  }

  return TRUE;
}

static gboolean File_find_hyperlink_OK_callback(GtkWidget *w, GtkWidget *base_url_window)
{
  gtk_widget_hide(base_url_window);

  cfilebrowser->setup(_("Find Hyperlink"),
		      File_ok_find_hyperlink);
  cfilebrowser->show();

  return TRUE;
}

gboolean File_find_hyperlink(GtkWidget *w, gpointer data)
{
  gtk_widget_show(base_url_window);
  gtk_widget_grab_focus(base_url_entry);

  return TRUE;
}

void Create_find_hyperlink_window(GtkWidget *toplevel)
{
  base_url_window = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(base_url_window), _("Find Hyperlink"));
  gtk_widget_set_usize(base_url_window, 500, 100);
  // delete event
  gtk_signal_connect_object(GTK_OBJECT(base_url_window),
			    "delete_event",
			    GTK_SIGNAL_FUNC(Hide_window),
			    GTK_OBJECT(base_url_window));
  
  //// action area
  GtkWidget *bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(base_url_window)->action_area),
		     bbox, FALSE, FALSE, 0);
  gtk_widget_show(bbox);
  // OK button
  GtkWidget *OK_button = gtk_button_new_with_label(_("OK"));
  GTK_WIDGET_SET_FLAGS(OK_button, GTK_CAN_DEFAULT);
  gtk_window_set_default(GTK_WINDOW(base_url_window), OK_button);
  gtk_signal_connect(GTK_OBJECT(OK_button),
		     "clicked",
		     GTK_SIGNAL_FUNC(File_find_hyperlink_OK_callback),
		     GTK_OBJECT(base_url_window));
  gtk_box_pack_start(GTK_BOX(bbox), OK_button, TRUE, TRUE, 0);
  gtk_widget_show(OK_button);
  // Cancel button
  GtkWidget *Cancel_button = gtk_button_new_with_label(_("Cancel"));
  //GTK_WIDGET_SET_FLAGS(Cancel_button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT(Cancel_button),
			    "clicked",
			    GTK_SIGNAL_FUNC(Hide_window),
			    GTK_OBJECT(base_url_window));
  gtk_box_pack_start(GTK_BOX(bbox), Cancel_button, TRUE, TRUE, 0);
  gtk_widget_show(Cancel_button);

  //// function area
  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
  gtk_widget_show(hbox);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(base_url_window)->vbox),
		     hbox, TRUE, TRUE, 10);
  // label
  GtkWidget *label = gtk_label_new("BASE URL:");
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
  // text entry for URL
  base_url_entry = gtk_entry_new_with_max_length(512);
  gtk_signal_connect_object(GTK_OBJECT(base_url_entry), "activate",
			    GTK_SIGNAL_FUNC(gtk_button_clicked),
			    GTK_OBJECT(OK_button));
  gtk_box_pack_start(GTK_BOX(hbox), base_url_entry, TRUE, TRUE, 10);
  gtk_widget_show(base_url_entry);

  // window property
  gtk_window_set_modal(GTK_WINDOW(base_url_window), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(base_url_window),
			       GTK_WINDOW(toplevel));
}
