//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_file_new_item.cc,v 1.7 2001/03/15 05:35:38 tujikawa Exp $

#include "aria.h"
#include "ItemCell_HTTP.h"
#include "ItemCell_FTP.h"
#include "ThreadManager.h"
#include "AppOption.h"
#include "PasteWindow.h"
using namespace std;

// external functions
extern void Set_sensitive__list_not_empty();
extern gboolean Hide_window(GtkWidget *window, gpointer unused);
extern void Add_new_item_to_downloadlist(ItemCell *itemcell);

// external variables
extern ThreadManager *thread_manager;
extern pthread_cond_t itemlistnotempty;
extern pthread_mutex_t itemlistlock;
extern ItemCell *console_item;
extern AppOption *appoption;
extern GtkWidget *itemlistwidget;
extern PasteWindow *paste_window;
extern ItemManager *item_manager;

// global variables
static GtkWidget *new_item_window;
static GtkWidget *new_url_entry;

static gboolean File_new_item_Unfold_callback(GtkWidget *w, GtkWidget *new_item_window)
{
  gtk_widget_hide(new_item_window);
  string url = Remove_white(gtk_entry_get_text(GTK_ENTRY(new_url_entry)));
  if(paste_window->add_url_with_unfold(url)) {
    paste_window->show();
  }

  return TRUE;
}

bool Create_new_item(const string& url, const string& dir = "", const string& referer = "")
{
  //gtk_widget_hide(new_item_window);
  URLcontainer urlcon;
  if(urlcon.Parse_URL(url) && !item_manager->search_by_url(urlcon.ret_URL())) {
    Options options = console_item->ret_Options_opt();
    if(dir.size()) {
      options.set_Store_Dir(dir);
    }
    if(referer.size()) {
      options.set_Referer_Type(Options::REFERER_USER_DEFINED);
      options.set_Referer(referer);
    }
    ItemCell *itemcell;
    if(urlcon.ret_Protocol() == "http:") {
      itemcell = new ItemCell_HTTP(url, urlcon, options, "created");
    } else if(urlcon.ret_Protocol() == "ftp:") {
      itemcell = new ItemCell_FTP(url, urlcon, options, "created");
    } else {
      return true;
    }

    pthread_mutex_lock(&itemlistlock);
    Add_new_item_to_downloadlist(itemcell);
    if(appoption->Whether_use_automatic_start() ||
       !thread_manager->Is_all_threads_sleeping()) {
      pthread_cond_broadcast(&itemlistnotempty);
    }
    gtk_clist_moveto(GTK_CLIST(itemlistwidget),
		     GTK_CLIST(itemlistwidget)->rows-1,
		     0,
		     0.0,
		     0.0);
    pthread_mutex_unlock(&itemlistlock);
    console_item->Send_message_to_gui(_("1item added"), MSG_SYS_INFO);
    if(GTK_CLIST(itemlistwidget)->rows > 0) {
      Set_sensitive__list_not_empty();
    }
  }

  return true;
}

static gboolean File_new_item_Add_callback(GtkWidget *w, GtkWidget *new_item_window)
{
  gtk_widget_hide(new_item_window);
  string url = Remove_white(gtk_entry_get_text(GTK_ENTRY(new_url_entry)));
  if(paste_window->add_url(url)) {
    paste_window->show();
  }
  ////modified 2001/3/1
  //return Create_new_item(Remove_white(gtk_entry_get_text(GTK_ENTRY(new_url_entry))));
  return TRUE;
}

gboolean File_new_item(GtkWidget *w, gpointer data)
{
  gtk_entry_set_text(GTK_ENTRY(new_url_entry), "");
  gtk_widget_grab_focus(new_url_entry);
  gtk_widget_show(GTK_WIDGET(new_item_window));

  return TRUE;
}

void Create_new_item_window(GtkWidget *toplevel)
{
  new_item_window = gtk_dialog_new();
  gtk_window_set_title(GTK_WINDOW(new_item_window), _("New item"));
  gtk_widget_set_usize(new_item_window, 500, 100);

  // delete event
  gtk_signal_connect_object(GTK_OBJECT(new_item_window),
			    "delete_event",
			    GTK_SIGNAL_FUNC(Hide_window),
			    GTK_OBJECT(new_item_window));
  
  //// action area
  GtkWidget *bbox = gtk_hbutton_box_new();
  gtk_button_box_set_layout(GTK_BUTTON_BOX(bbox), GTK_BUTTONBOX_END);
  gtk_button_box_set_spacing(GTK_BUTTON_BOX(bbox), 5);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_item_window)->action_area),
		     bbox, FALSE, FALSE, 0);
  gtk_widget_show(bbox);
  // Add button
  GtkWidget *Add_button = gtk_button_new_with_label(_("Add"));
  GTK_WIDGET_SET_FLAGS(Add_button, GTK_CAN_DEFAULT);
  gtk_window_set_default(GTK_WINDOW(new_item_window), Add_button);
  gtk_signal_connect(GTK_OBJECT(Add_button),
		     "clicked",
		     GTK_SIGNAL_FUNC(File_new_item_Add_callback),
		     GTK_OBJECT(new_item_window));
  gtk_box_pack_start(GTK_BOX(bbox), Add_button, TRUE, TRUE, 0);
  gtk_widget_show(Add_button);
  // unfold button
  GtkWidget *Unfold_button = gtk_button_new_with_label(_("Unfold"));
  //GTK_WIDGET_SET_FLAGS(Unfold_button, GTK_CAN_DEFAULT);
  gtk_signal_connect(GTK_OBJECT(Unfold_button),
		     "clicked",
		     GTK_SIGNAL_FUNC(File_new_item_Unfold_callback),
		     GTK_OBJECT(new_item_window));
  gtk_box_pack_start(GTK_BOX(bbox), Unfold_button, TRUE, TRUE, 0);
  gtk_widget_show(Unfold_button);
  // Cancel button
  GtkWidget *Cancel_button = gtk_button_new_with_label(_("Cancel"));
  //GTK_WIDGET_SET_FLAGS(Cancel_button, GTK_CAN_DEFAULT);
  gtk_signal_connect_object(GTK_OBJECT(Cancel_button),
			    "clicked",
			    GTK_SIGNAL_FUNC(Hide_window),
			    GTK_OBJECT(new_item_window));
  gtk_box_pack_start(GTK_BOX(bbox), Cancel_button, TRUE, TRUE, 0);
  gtk_widget_show(Cancel_button);

  //// function area
  GtkWidget *hbox = gtk_hbox_new(FALSE, 0);
  gtk_box_pack_start(GTK_BOX(GTK_DIALOG(new_item_window)->vbox),
		     hbox, TRUE, TRUE, 10);
  gtk_widget_show(hbox);
  // label
  GtkWidget *label = gtk_label_new("URL:");
  gtk_widget_show(label);
  gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 10);
  // text entry for new URL
  new_url_entry = gtk_entry_new_with_max_length(512);
  gtk_signal_connect_object(GTK_OBJECT(new_url_entry), "activate",
			    GTK_SIGNAL_FUNC(gtk_button_clicked),
			    GTK_OBJECT(Add_button));
  gtk_box_pack_start(GTK_BOX(hbox), new_url_entry, TRUE, TRUE, 10);
  gtk_widget_show(new_url_entry);

  //// window property
  gtk_window_set_modal(GTK_WINDOW(new_item_window), TRUE);
  gtk_window_set_transient_for(GTK_WINDOW(new_item_window),
			       GTK_WINDOW(toplevel));
}
