//
//  aria - yet another download tool
//  Copyright (C) 2000, 2001 Tatsuhiro Tsujikawa
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

// $Id: gui_option.cc,v 1.17 2001/03/19 05:33:41 tujikawa Exp $

#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <sys/types.h>
#include "aria.h"
#include "ItemCell.h"
#include "AppOption.h"
#include "HistoryWindow.h"
#include "ItemList.h"
#include "ShortCutKey.h"

void Set_clist_column__url(int rowindex, const string& url_string);
void Set_clist_column__save(int rowindex, const string& save_string);
void Set_clist_column__try(int rowindex, const string& try_string);
void Set_clist_column__rec(int rowindex, const string& rec_string);

extern int pipetogui[2];
extern pthread_mutex_t itemlistlock;
extern AppOption *appoption;
extern GtkWidget *itemlistwidget, *text, *console_text;
extern ItemCell *console_item;
extern ItemList *itemlist_global;

static GtkWidget *item_individual_item;
static GtkWidget *item_default_item;
static GtkWidget *program_item;
static GtkWidget *history_item;
static GtkWidget *reset_to_default_item;
static GtkWidget *reset_to_default_nosd_item;
static GtkWidget *reset_to_default_norec_item;
static GtkWidget *reset_to_default_all_item;
static GtkWidget *reset_to_default_all_nosd_item;
static GtkWidget *reset_to_default_all_norec_item;
HistoryWindow *history_window;

void Option_set_sensitive__no_item_selected()
{
  gtk_widget_set_sensitive(item_individual_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_nosd_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_norec_item, FALSE);
}

void Option_set_sensitive__items_selected()
{
  gtk_widget_set_sensitive(item_individual_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_nosd_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_norec_item, TRUE);
}

void Option_set_sensitive__list_empty()
{
  gtk_widget_set_sensitive(reset_to_default_all_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_all_nosd_item, FALSE);
  gtk_widget_set_sensitive(reset_to_default_all_norec_item, FALSE);
}

void Option_set_sensitive__list_not_empty()
{
  gtk_widget_set_sensitive(reset_to_default_all_item, TRUE);
  gtk_widget_set_sensitive(reset_to_default_all_nosd_item, TRUE);  
  gtk_widget_set_sensitive(reset_to_default_all_norec_item, TRUE);  
}

gboolean Option_history(GtkWidget *w, gpointer data)
{
  history_window->show();
  return TRUE;
}

gboolean Option_item_individual(GtkWidget *w, gpointer data)
{
  GList* node = GTK_CLIST(itemlistwidget)->selection;
  if(node == NULL) return(TRUE);
  ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), GPOINTER_TO_UINT(node->data));
  pthread_mutex_lock(&itemlistlock);
  itemcell->Show_option_window();
  pthread_mutex_unlock(&itemlistlock);

  return TRUE;
}

gboolean Option_item_default(GtkWidget *w, gpointer data)
{
  console_item->Show_option_window();
  return TRUE;
}

static gboolean Option_reset_to_default_all(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  const Options& console_option = console_item->ret_Options_opt();
  for(unsigned int rowindex = 0; rowindex < (unsigned int)GTK_CLIST(itemlistwidget)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt() = console_option;

    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));
    Set_clist_column__save(rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    if(itemcell->ret_URL_Container_opt().ret_Protocol() == "http:") {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options_opt().ret_recurse_count()));
    } else {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options_opt().ret_FTP_recurse_count()));
    }

    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

// same as Option_reset_to_default_all except that store directory does not change
static gboolean Option_reset_to_default_all_nosd(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  const Options& console_option = console_item->ret_Options_opt();
  for(unsigned int rowindex = 0; rowindex < (unsigned int)GTK_CLIST(itemlistwidget)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();
    string storedir_temp = itemcell->ret_Options_opt().ret_Store_Dir();
    itemcell->ret_Options_opt() = console_option;
    itemcell->ret_Options_opt().set_Store_Dir(storedir_temp);

    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));

    if(itemcell->ret_URL_Container_opt().ret_Protocol() == "http:") {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options_opt().ret_recurse_count()));
    } else {
      Set_clist_column__rec(rowindex, itos(itemcell->ret_Options_opt().ret_FTP_recurse_count()));
    }

    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

void recursive_option_backup(Options& console_option, const Options& options)
{
  console_option.set_recurse_count(options.ret_recurse_count());
  console_option.set_with_hostname_dir(options.ret_with_hostname_dir());
  console_option.set_abs2rel_url(options.ret_abs2rel_url());
  console_option.set_force_convert(options.ret_force_convert());
  console_option.set_delete_comment(options.ret_delete_comment());
  console_option.set_delete_javascript(options.ret_delete_javascript());
  console_option.set_no_other_host(options.ret_no_other_host());
  console_option.set_no_ascend(options.ret_no_ascend());
  console_option.set_only_relative_links(options.ret_only_relative_links());
  console_option.set_Referer_override(options.ret_Referer_override());
  console_option.set_Follow_ftp_link(options.ret_Follow_ftp_link());
  console_option.set_convert_tilde(options.ret_convert_tilde());

  console_option.set_parse_target_list(options.ret_parse_target_list());
  console_option.set_use_down_filter(options.ret_use_down_filter());
  console_option.set_filter_down_target_list(options.ret_filter_down_target_list());
  console_option.set_filter_nodown_target_list(options.ret_filter_nodown_target_list());
  console_option.set_ign_domain_list(options.ret_ign_domain_list());
  console_option.set_activated_parse_target_list(options.ret_activated_parse_target_list());
  console_option.set_activated_filter_down_target_list(options.ret_activated_filter_down_target_list());
  console_option.set_activated_filter_nodown_target_list(options.ret_activated_filter_nodown_target_list());
  console_option.set_activated_ign_domain_list(options.ret_activated_ign_domain_list());
  console_option.set_use_tag_href(options.ret_use_tag_href());
  console_option.set_use_tag_src(options.ret_use_tag_src());
  console_option.set_use_tag_background(options.ret_use_tag_background());
  console_option.set_use_tag_code(options.ret_use_tag_code());
  // FTP recursive
  console_option.set_FTP_no_ascend(options.ret_FTP_no_ascend());
  console_option.set_FTP_use_filter(options.ret_FTP_use_filter());
  console_option.set_FTP_allow_crawl_subdir(options.ret_FTP_allow_crawl_subdir());
  console_option.set_FTP_get_symlink_as_realfile(options.ret_FTP_get_symlink_as_realfile());
  console_option.set_FTP_recurse_count(options.ret_FTP_recurse_count());
  console_option.set_FTP_filter_target_list(options.ret_FTP_filter_target_list());
  console_option.set_FTP_activated_filter_target_list(options.ret_FTP_activated_filter_target_list());
}

static gboolean Option_reset_to_default_all_norec(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  Options console_option = console_item->ret_Options_opt();
  for(unsigned int rowindex = 0; rowindex < (unsigned int)GTK_CLIST(itemlistwidget)->rows; ++rowindex) {
    ItemCell *itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();
    const Options& options = itemcell->ret_Options_opt();
    recursive_option_backup(console_option, options);
    // save directory
    console_option.set_Store_Dir(options.ret_Store_Dir());

    itemcell->ret_Options_opt() = console_option;


    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));

    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
  }
  pthread_mutex_unlock(&itemlistlock);
  return TRUE;
}

static gboolean Option_reset_to_default(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  const Options& console_option = console_item->ret_Options_opt();
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();
    itemcell->ret_Options_opt() = console_option;

    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));
    Set_clist_column__save(rowindex, itemcell->ret_Options_opt().ret_Store_Dir());
    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  return(TRUE);
}

static gboolean Option_reset_to_default_nosd(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  const Options& console_option = console_item->ret_Options_opt();
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();

    string storedir_temp = itemcell->ret_Options_opt().ret_Store_Dir();
    itemcell->ret_Options_opt() = console_option;
    itemcell->ret_Options_opt().set_Store_Dir(storedir_temp);

    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));

    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  return(TRUE);
}

static gboolean Option_reset_to_default_norec(GtkWidget *w, gpointer data)
{
  pthread_mutex_lock(&itemlistlock);
  GList *node = GTK_CLIST(itemlistwidget)->selection;
  Options console_option = console_item->ret_Options_opt();
  while(node) {
    unsigned int rowindex = GPOINTER_TO_UINT(node->data);
    ItemCell* itemcell = (ItemCell*)gtk_clist_get_row_data(GTK_CLIST(itemlistwidget), rowindex);
    itemcell->get_Options_Lock();

    const Options& options = itemcell->ret_Options_opt();
    recursive_option_backup(console_option, options);

    // save directory
    console_option.set_Store_Dir(options.ret_Store_Dir());

    itemcell->ret_Options_opt() = console_option;

    Set_clist_column__try(rowindex, itos(itemcell->ret_Count())+"/"+itos(itemcell->ret_Options_opt().ret_Retry()));

    itemcell->Raise_option_update_flag();
    itemcell->release_Options_Lock();
    node = g_list_next(node);
  }
  pthread_mutex_unlock(&itemlistlock);
  return(TRUE);
}

static gboolean Option_program(GtkWidget* w, gpointer data)
{
  appoption->Show_option_window();
  return(TRUE);
}

void Create_option_menu(GtkWidget *toplevel, GtkWidget* menu_bar, GtkAccelGroup* accel_group)
{
  GtkWidget* option_menu;
  GtkWidget* option_item;
  GtkWidget* separator1_item;
  GtkWidget* separator2_item;

  //create option menu
  option_menu = gtk_menu_new();

  item_individual_item = gtk_menu_item_new_with_label(_("selected item option"));
  reset_to_default_item = gtk_menu_item_new_with_label(_("apply default item option to selected items"));
  reset_to_default_nosd_item = gtk_menu_item_new_with_label(_("apply default item option to selected items(except for save dir)"));
  reset_to_default_norec_item = gtk_menu_item_new_with_label(_("apply default item option to selected items(except for save dir and recursive settings)"));
  separator1_item = gtk_menu_item_new();

  item_default_item = gtk_menu_item_new_with_label(_("default item option"));
  reset_to_default_all_item = gtk_menu_item_new_with_label(_("apply default item option to all items"));
  reset_to_default_all_nosd_item = gtk_menu_item_new_with_label(_("apply default item option to all items(except for save dir)"));
  reset_to_default_all_norec_item = gtk_menu_item_new_with_label(_("apply default item option to all items(except for save dir and recursive settings)"));
  
  separator2_item = gtk_menu_item_new();
  program_item = gtk_menu_item_new_with_label(_("application settings"));
  GtkWidget *separator3_item = gtk_menu_item_new();
  history_item = gtk_menu_item_new_with_label(_("download history"));

  gtk_menu_append(GTK_MENU(option_menu), item_individual_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_nosd_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_norec_item);
  gtk_menu_append(GTK_MENU(option_menu), separator1_item);
  gtk_menu_append(GTK_MENU(option_menu), item_default_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_all_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_all_nosd_item);
  gtk_menu_append(GTK_MENU(option_menu), reset_to_default_all_norec_item);
  gtk_menu_append(GTK_MENU(option_menu), separator2_item);
  gtk_menu_append(GTK_MENU(option_menu), program_item);
  gtk_menu_append(GTK_MENU(option_menu), separator3_item);
  gtk_menu_append(GTK_MENU(option_menu), history_item);


  // signal handling
  gtk_signal_connect(GTK_OBJECT(item_individual_item), "activate",
		     GTK_SIGNAL_FUNC(Option_item_individual), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_nosd_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_nosd), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_norec_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_norec), NULL);
  gtk_signal_connect(GTK_OBJECT(item_default_item), "activate",
		     GTK_SIGNAL_FUNC(Option_item_default), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_all_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_all), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_all_nosd_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_all_nosd), NULL);
  gtk_signal_connect(GTK_OBJECT(reset_to_default_all_norec_item), "activate",
		     GTK_SIGNAL_FUNC(Option_reset_to_default_all_norec), NULL);
  gtk_signal_connect(GTK_OBJECT(program_item), "activate",
		     GTK_SIGNAL_FUNC(Option_program), NULL);
  gtk_signal_connect(GTK_OBJECT(history_item), "activate",
		     GTK_SIGNAL_FUNC(Option_history), NULL);

  // accelerator key
  gtk_widget_add_accelerator(item_individual_item,
			     "activate",
			     accel_group,
			     SC_ITEMOPTION,
			     SCM_ITEMOPTION,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(item_default_item,
			     "activate",
			     accel_group,
			     SC_ITEMOPTION_DEFAULT,
			     SCM_ITEMOPTION_DEFAULT,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(program_item,
			     "activate",
			     accel_group,
			     SC_APPOPTION,
			     SCM_APPOPTION,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(reset_to_default_item,
			     "activate",
			     accel_group,
			     SC_SETDEFAULT,
			     SCM_SETDEFAULT,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(reset_to_default_nosd_item,
			     "activate",
			     accel_group,
			     SC_SETDEFAULT_NOSAVEDIR,
			     SCM_SETDEFAULT_NOSAVEDIR,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(reset_to_default_all_item,
			     "activate",
			     accel_group,
			     SC_SETDEFAULT_ALL,
			     SCM_SETDEFAULT_ALL,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(reset_to_default_all_nosd_item,
			     "activate",
			     accel_group,
			     SC_SETDEFAULT_ALL_NOSAVEDIR,
			     SCM_SETDEFAULT_ALL_NOSAVEDIR,
			     GTK_ACCEL_VISIBLE);

  gtk_widget_add_accelerator(history_item,
			     "activate",
			     accel_group,
			     SC_HISTORY,
			     SCM_HISTORY,
			     GTK_ACCEL_VISIBLE);
  
  gtk_widget_show(item_individual_item);
  gtk_widget_show(reset_to_default_item);
  gtk_widget_show(reset_to_default_nosd_item);
  gtk_widget_show(reset_to_default_norec_item);
  gtk_widget_show(separator1_item);
  gtk_widget_show(item_default_item);
  gtk_widget_show(reset_to_default_all_item);
  gtk_widget_show(reset_to_default_all_nosd_item);
  gtk_widget_show(reset_to_default_all_norec_item);
  gtk_widget_show(separator2_item);
  gtk_widget_show(program_item);
  gtk_widget_show(separator3_item);
  gtk_widget_show(history_item);

  Option_set_sensitive__no_item_selected();
  Option_set_sensitive__list_empty();

  // download history window
  history_window = new HistoryWindow(GTK_WINDOW(toplevel));
  history_window->read_from_file(itemlist_global->ret_file_history());

  option_item = gtk_menu_item_new_with_label(_("Option"));
  gtk_widget_show(option_item);
  gtk_menu_item_set_submenu(GTK_MENU_ITEM(option_item), option_menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), option_item);
}
